%% MATLAB Function to determine the persistence length of 2D polymer shapes
% Run this application to calculate the bending persistence length of a
% polymer by analyzing images of the polymer shape conformations in two 
% spatial dimensions.  
% 
% Input: 
% This requires a stack of the processed images in RGB color and a
% skeltonization of the polymers in the same stack of 
% images in 8-bit both saved as a .tif file.  This can be done using ImageJ
% or FIJI (NIH).
% 
% How it works:
% The function first searches for polymer ends (defined as a point with one
% nearest neighbor), tracks the skeleton of the polymer using a directional
% bias searching algorithm and reconstructs the filament using a 3rd order
% Bezier spline.  The tracking algorithm avoids 
% artifical skeleton branches by directing the search for the skeleton in a 
% forward direction and tracks over breaks in the skeleton up to 2 pixels 
% long.  The reconstruction is used to more accurately represent the polymer
% shape and increases the angular sampling by allowing for more than the 
% discrete pi/4 angle increments between pixels in an image to be sampled.  
% The user can also pick the end from which the filament reconstruction will
% occur and then can screen the filament reconstructions used with mouse 
% input.  The settings section should be modified for micrometer per pixel 
% distance calibration, controlling the minimum number of pixels used to
% define a polymer (must be at least 4 for the reconstruction to work) and
% total number of filament shapes used to create the data set.  Using 
% statistical methods the user should determine when an adequate sample size
% has been met. The user can also choose to reanalyze a stack of images 
% with previously chosen selections that are saved.  The persistence length
% is primarily determined by fitting the cosine of the angular 
% autocorrelations as a function of contour length to the cosine correlation
% function.  The persistence length is determined by fitting the 
% relationship between the end-to-end and contour lengths of a filament.
% The discrete Fourier decomposition of bending modes is also analyzed to
% determine the persistence length.  All data calculated by this function is
% saved as separate files with names appended to the name of the 
% skeletonized image stack file analyzed.  When the same stack is analyzed
% differently a new identifying name should be entered when prompted.  This
% function was written and implemented to determine the persistence length 
% of fluorescently labelled actin filaments imaged using epi-fluorescence 
% microscopy.  This function is compatible with MATLAB R2022b.
% Copyright 2012 Brannon R McCullough, 2023 De La Cruz lab
%% 
%% Main Function Flexural_Rigidity
% Usage: Persistence
function PersistenceVersion_5_jun23()
% Settings
close all
clear

% Setup global variables.
global brch_size single_pixel_gap_reply
home=pwd; %get the current directory and assgn it as home directory
if isempty(dir('PersistenceVersion_5.ini')) %check if there is a file with the previous setting values, if not,   creat one.
    a_iniatial_setting_value = [4, 0.1, 100000, 1, 0, 1, 2, 2, 2, 2, 2, 2];    
    inifile = fopen('PersistenceVersion_5.ini','w');
    fprintf(inifile, '%u\n %f\n %u\n %u\n %u\n %u\n %u\n %u\n %u\n %u\n %u\n %u\n', a_iniatial_setting_value);
    fclose(inifile);
else
    inifile = fopen('PersistenceVersion_5.ini','r'); %if exist, import it as inital setting,
    a_iniatial_setting_value = fscanf(inifile, '%u %f %u %u %u %u %u %u %u %u %u %u');    
    fclose(inifile);
    a_iniatial_setting_value = a_iniatial_setting_value'; %covert column to row vector
end

% Setting up the running parameters and options by function param_setting().
a_setting_value = param_setting(a_iniatial_setting_value); %Use iniatial_setting_value to initialize the setting values
if (sum(a_setting_value ~= a_iniatial_setting_value)>0) % If user changed the initial setting, then save the new setting.
    inifile = fopen('PersistenceVersion_5.ini','w');
    fprintf(inifile, '%u\n %f\n %u\n %u\n %u\n %u\n %u\n %u\n %u\n %u\n %u\n %u\n', a_setting_value);
    fclose(inifile);
end
pix_threshold = a_setting_value(1); % the minimum filament length in pixel.
um_pix = a_setting_value(2); % Pixel size conversion from calibration |um_pix| in microns per pixel.
filamentN = a_setting_value(3); % Limit for total number of filament shapes to be analyzed
start_frame = a_setting_value(4); % Initial frame number.
brch_size = a_setting_value(5); % branch size limit.

%Get and set option values
option_choice = {'Yes'  'No'};
mean_reply = option_choice{a_setting_value(6)}; %calculate average and fit the data
selected_already_reply = option_choice{a_setting_value(7)}; %analyze the shapes previously selected
manual_filament_reply = option_choice{a_setting_value(8)}; %manually construct filament between two clicked points
stretch_reply = option_choice{a_setting_value(9)}; %manually construct filament by stretching from one clicked point
single_pixel_gap_reply = option_choice{a_setting_value(10)}; %whether or not allow single pixel gap
select_ends_reply = option_choice{a_setting_value(11)}; %manually select ends
select_reply = option_choice{a_setting_value(12)}; %want to confirm the selected shapes


% Select images to analyze
close all
clear a_* S
% Setup notification
f_notification=figure('units','pixels',...
        'position',[20 20 440 500],...
        'menubar','none',...
        'name','Notification',...
        'numbertitle','off',...
        'Visible', 'off', ...
        'resize','off');

date_and_time = string(datetime('now', 'format','MM/dd/yyyy HH:mm:SS'));
date_and_time_in_filename = char(datetime('now', 'format','yyMMddHHmm'));
if strcmp(selected_already_reply, 'Yes')
    % cd(im_dir); % Go to directory containing image stacks to be analyzed
    [a_file, a_selections_path] = uigetfile( '*.*', 'Select Selections to load');
    cd(a_selections_path);
    filament_pool=importdata(a_file); % Read the previously saved cell array file. First column filament position map after Bezier reconstruction, second column is the corresponding derivative.       
    a_Import_Construction=size(filament_pool, 1);
    [~,a_selections_file,~] =fileparts(a_file);

    %make directory for saving files
    cd ..;
    a_selections_path_parent = pwd; %upper leverl folder of a_selections_path
    a_current_folder_name = a_selections_path(1, size(a_selections_path_parent )+2:end);
    cd(a_selections_path);
    if ~isequal(a_current_folder_name, 'Fit_Results') % if a_selections_path folder is 'Fit_Results' already and no need to create one, otherwise create one
        if exist('Fit_Results', 'dir') == 7
            cd('Fit_Results');
        else
            mkdir('Fit_Results');
            cd('Fit_Results');
        end
    end
    % get the saving file directory
    while 1
        [savefile, savepath] = uiputfile('*.txt', 'Save Correlations as', a_selections_file);
        if savefile~=0
            break;
        end
    end
    [~,savefile,~] =fileparts(savefile);
    savefile = [savefile, '_',  num2str(date_and_time_in_filename)];
%     [savefile, savepath]=Save_Files(a_selections_file, a_selections_path);    % Set the default directory for saving correlation results under import filament directory.  
    cd(home);
    if strcmp(savefile, 'quit')
        return; % Filename is not suitable. Quit the program.
    end    
    Summary_filename = [savepath,'\', savefile, '_sum.txt']; %Set up file to save the result.
    if exist(Summary_filename, 'file') == 2
        Summary_file = fopen(Summary_filename, 'a');        
    else
        Summary_file = fopen(Summary_filename, 'w');
    end
    fprintf(Summary_file, '%s\n\n', ['Matlab file: ', mfilename, ' (', date_and_time, ')' ]);
    fprintf(Summary_file, '%s\n\n', 'Summary. Continue working from the previously selected filaments');
    fprintf(Summary_file, '%s\n', ['Settings: Min. filament pixels: ', num2str(pix_threshold), ', Size of a pixel (microns): ', num2str(um_pix)]);
    fprintf(Summary_file, '%s\n\n', ['Max. number of filaments: ', num2str(filamentN), ', Starting image frame: ', num2str(start_frame), 'Allowed branch length: ',num2str(brch_size)]);
    fprintf(Summary_file, '%s\n', ['Options: Average and fit? ', mean_reply, ', Already selected? '  selected_already_reply]);
    fprintf(Summary_file, '%s\n', ['Manually constuct filament between two selected points? ', manual_filament_reply, ', manually constuct filament by stretching from one selected point? ', stretch_reply, ', Allow a single pixel gap? ', single_pixel_gap_reply]);
    fprintf(Summary_file, '%s\n\n', ['Select end manually? ', select_ends_reply, 'Confirm filament shapes? ', select_reply]);
    fprintf(Summary_file, '%s\n\n', [num2str(a_Import_Construction), ' filaments have been imported from previous file ', a_selections_path, a_file]);
    fclose(Summary_file);
    
    a_more_selection_reply = questdlg('Want to continue selecting for more filament shapes?("No" to analyze the imported filaments only)','Eh? I am waiting!',...
        'Yes','No','Yes');
    if strcmp(a_more_selection_reply, 'Yes')
        a_last_selected_filament_ID = filament_pool{a_Import_Construction, 3};
        a_start_frame_reply = inputdlg({['Enter start frame #: (Previously stopped at frame ',num2str(a_last_selected_filament_ID(1)), ')']},'Frame Input',[1 35],{num2str(a_last_selected_filament_ID(1))});
                figure('units','pixels',...
            'position',[500 200 290 140],...
            'menubar','none',...
            'name','Look here!',...
            'numbertitle','off',...
            'resize','off');
        uicontrol('Style','text','Position',[20 65 250 60],'FontSize',12,...
            'String',['Continue selections starting from frame ',  a_start_frame_reply],...
            'BackgroundColor',[0.8 0.8 0.8]);
        uicontrol('Position',[45 20 200 40],'String','OK',...
            'Callback','uiresume(gcbf)');
        uiwait(gcf);
        close;
        start_frame = str2double(a_start_frame_reply);
    else
        start_frame = 0; % Set the flag to bypass more filament selection.
        set(f_notification, 'Visible', 'on'); % Notification is on.
        uicontrol(f_notification,'Style','text','Position',[20 220 400 260],'FontSize',12,...
            'HorizontalAlignment', 'left', 'String',['Total ', num2str(a_Import_Construction), ' reconstructions have been imported from ', a_file, '.' , newline ...
            'Now processing the collected filaments. Please be patient ... ', newline ...
            'After finishing calculating data, It takes time to output files because of huge data set. ', ...
            'Please wait patiently. Putting music on helps or have a cup of coffee and come back later...' ],...
            'BackgroundColor',[0.8 0.8 0.8]);
    end
    clear a_*;
end

if strcmp(selected_already_reply, 'No') || start_frame ~= 0
    [file1, path_original] = uigetfile( '*.tif', 'Select Original Image in Grayscale or RGB');   
    cd(path_original)
    [file2, path_skeleton] = uigetfile( '*.tif', 'Select Binary Skeltonized Image');
    [~,basefile,~] =fileparts(file2);
    cd(home)
    if ~exist('savefile', 'var')
        cd(path_original)  %use the original image directory as saving file directory
        %make directory for saving files
        cd ..;
        a_selections_path_parent = pwd; %upper leverl folder of a_selections_path
        a_current_folder_name = path_original(1, size(a_selections_path_parent )+2:end);
        cd(path_original);
        if ~isequal(a_current_folder_name, 'Fit_Results') % if a_selections_path folder is 'Fit_Results' already and no need to create one, otherwise create one
            if exist('Fit_Results', 'dir') == 7
                cd('Fit_Results');
            else
                mkdir('Fit_Results');
                cd('Fit_Results');
            end
        end
        % get the saving file directory
        while 1
            [savefile, savepath] = uiputfile('*.txt', 'Save Correlations as', basefile); %suggest the skeleton image file name as saving file name
            if savefile~=0
                break;
            end
        end
        [~,savefile,~] =fileparts(savefile);
        % [savefile, savepath]=Save_Files(basefile, path_original);  % Set the default directory for saving correlation results under original image directory.
        cd(home)
        if strcmp(savefile, 'quit')
            return; % Filename is not suitable. Quit the program.
        end
        savefile = [savefile, '_',  num2str(date_and_time_in_filename)];
        Summary_filename = [savepath,'\', savefile, '_sum.txt']; %Set up file to save the result.
        Summary_file = fopen(Summary_filename, 'w');
        fprintf(Summary_file, '%s\n\n', ['Matlab file: ', mfilename, ', (', date_and_time, ')' ]);
        fprintf(Summary_file, '%s\n\n', 'Summary:');
        fprintf(Summary_file, '%s\n', ['Settings: Min. filament pixels: ', num2str(pix_threshold), ', Size of a pixel (microns): ', num2str(um_pix)]);
        fprintf(Summary_file, '%s\n\n', ['Max. number of filaments: ', num2str(filamentN), ', Starting image frame: ', num2str(start_frame), 'Allowed branch length: ',num2str(brch_size)]);
        fprintf(Summary_file, '%s\n', ['Options: Average and fit? ', mean_reply, ', Already selected? '  selected_already_reply]);
        fprintf(Summary_file, '%s\n', ['Manually constuct filament between two selected points? ', manual_filament_reply, ', manually constuct filament by stretching from one selected point? ', stretch_reply, ', Allow a single pixel gap? ', single_pixel_gap_reply]);
        fprintf(Summary_file, '%s\n\n', ['Select end manually? ', select_ends_reply, 'Confirm filament shapes? ', select_reply]);
        fprintf(Summary_file, '%s\n', ['Original image loaded: ', path_original,  file1]);
        fprintf(Summary_file, '%s\n\n', ['Skeletonized image loaded: ', path_skeleton,  file2]);
        fclose(Summary_file);
    else
        Summary_file = fopen(Summary_filename, 'a');
        fprintf(Summary_file, '%s\n', ['Original image loaded: ', path_original,  file1]);
        fprintf(Summary_file, '%s\n\n', ['Skeletonized image loaded: ', path_skeleton, file2,]);
        fprintf(Summary_file, '%s\n', ['Continue filament selection from frame: ', num2str(start_frame)]);
        fclose(Summary_file);
    end        
    if strcmp(manual_filament_reply, 'Yes')
        % Display instruction for manually constructing filament by a series of selected points.
        figure('units','pixels',...
            'position',[500 200 440 260],...
            'menubar','none',...
            'name','Look here!',...
            'numbertitle','off',...
            'resize','off');
        uicontrol('Style','text','Position',[20 65 400 200],'FontSize',14,...
            'String','For manually selecting filaments by a series of clicked points, when a crosshairs appears: select points with left mouse button to reconstruct filament continuously and "right arrow" key to save filament. Press "left arrow" key anytime to abort selection or delete last filament saved. To finish in this frame or to quit selection for all use "up arrow" key.',...
            'BackgroundColor',[0.8 0.8 0.8]);
        uicontrol('Position',[120 20 200 40],'String','OK','FontSize',14,...
            'Callback','uiresume(gcbf)');
        uiwait(gcf);
        close;
        % Display instruction for end selection.
    elseif strcmp(stretch_reply, 'Yes') %manual selecting filaments by stretching both directions from one clicked point
        % Display instruction for manual filament selection.
        figure('units','pixels',...
            'position',[500 200 440 260],...
            'menubar','none',...
            'name','Look here!',...
            'numbertitle','off',...
            'resize','off');
        uicontrol('Style','text','Position',[20 65 400 200],'FontSize',14,...
            'String','For manual selecting filaments by stretching both directions from one clicked point, when a crosshair appears : select filament segments with left mouse button and "right arrow" key to save filament. Press "left arrow" key anytime to abort selection or delete last filament saved. To finish in this frame or to quit selection for all use "up arrow" key.',...
            'BackgroundColor',[0.8 0.8 0.8]);
        uicontrol('Position',[120 20 200 40],'String','OK','FontSize',14,...
            'Callback','uiresume(gcbf)');
        uiwait(gcf);
        close;
        % Display instruction for end selection.
    else
        if strcmp(select_ends_reply, 'Yes') || strcmp(select_reply, 'Yes')
            if strcmp(select_ends_reply, 'Yes')
                figure('units','pixels',...
                    'position',[500 200 440 260],...
                    'menubar','none',...
                    'name','Look here!',...
                    'numbertitle','off',...
                    'resize','off');
                uicontrol('Style','text','Position',[20 65 400 200],'FontSize',14,...
                    'String','For selecting ends, when crosshairs appear on Manually Select Filament Ends Figure: select ends with left mouse button and delete the end just selected with left arrow. Up arrow key has a choice between quiting selection for all and continuing selection in the next frame.',...
                    'BackgroundColor',[0.8 0.8 0.8]);
                uicontrol('Position',[120 20 200 40],'String','OK','FontSize',14,...
                    'Callback','uiresume(gcbf)');
                uiwait(gcf);
                close;
            end
            % Display instruction for filament confirmation.
            if strcmp(select_reply, 'Yes')
                figure('units','pixels',...
                    'position',[500 200 440 200],...
                    'menubar','none',...
                    'name','Attention!',...
                    'numbertitle','off',...
                    'resize','off');
                uicontrol('Style','text','Position',[20 65 400 120],'FontSize',14,...
                    'String','For confirming filament construction, when crosshairs appear on Filament Confirmation Figure: right arrow to accept, left arrow  to reject or up arrow to end selections for that frame or for all',...
                    'BackgroundColor',[0.8 0.8 0.8]);
                uicontrol('Position',[120 20 200 40],'String','OK','FontSize',14,...
                    'Callback','uiresume(gcbf)');
                uiwait(gcf);
                close;
            end
        else
            figure('units','pixels',...
                'position',[500 200 290 220],...
                'menubar','none',...
                'name','Achtung!',...
                'numbertitle','off',...
                'resize','off');
            uicontrol('Style','text','Position',[20 65 250 140],'FontSize',14,...
                'String','All filament reconstructions will be analyzed. It may take a while to track all the filaments. Be patient.',...
                'BackgroundColor',[0.8 0.8 0.8]);
            uicontrol('Position',[45 20 200 40],'String','OK','FontSize',14,...
                'Callback','uiresume(gcbf)');
            uiwait(gcf);
            close;
        end
    end

% Filament selection block.
    % Make selections. Size of selections tied to frameN.  Location_values variable created to keep the end locations from which the selections should start in the analysis
    % Find the number of images in the stack.
    cd(path_original);
    info=imfinfo(file1);
    cd(home);
    frameN=size(info,1);
    clear info
    quit_reply='No'; % Want to quit manual end selection.    
    if exist('filament_pool', 'var')
        reconstructions_approved=size(filament_pool, 1); % # of approved fimaments.
    else
        reconstructions_approved=0; % # of approved fimaments.
        filament_pool=cell(200,3); %If not already existing, creat one'
    end
    if strcmp(select_reply,'Yes') && strcmp(manual_filament_reply, 'No') % Want to confirm filament shape reconstructions. Setup a figure for confirmation.
        confirm_fig=figure();
        scrsz = get(0,'ScreenSize');
        set(confirm_fig,'Position',[0 0 scrsz(3) scrsz(4)*.93], 'name', 'Filament Confirmation', 'Visible','off')% To view figure full screen (may need to adjust)
        uicontrol(confirm_fig,'style','text',...
            'FontSize',12,....
            'string',['Instruction for construction confirmation: ',  ...
            'right arrow to accept, left arrow  to reject or up arrow to end selections for that frame or for all'],...
            'position',[10. scrsz(4)*.93/2.-150 100 300]);
    end
    for frame=start_frame:frameN
        cd(path_skeleton);
        a_Skeleton=imread(file2,frame);
        a_Skeleton=logical(a_Skeleton);% Scale image to binary.
        cd(path_original);
        a_Original=imread(file1,frame);
        cd(home);  
        % Superimpose a_Original and a_Skeleton, and then convert the
        % conbination to RGB with the a_Skeleton pixel pink
        a_Original=im2uint8(a_Original);  % Conbvert to 8 bit
        a_Original_temp=a_Original(:, :, 1);
        a_Original_temp(a_Skeleton)=255;
        a_Original(:, :, 1)=a_Original_temp;
        if size(a_Original, 3)==3 % RGB
            a_Original_temp=a_Original(:, :, 2);
            a_Original_temp(a_Skeleton)=80;
            a_Original(:, :, 2)=a_Original_temp;
            a_Original_temp=a_Original(:, :, 3);
            a_Original_temp(a_Skeleton)=80;
            a_Original(:, :, 3)=a_Original_temp;
        else % Not RGB
            a_Original_temp(a_Skeleton)=80;
            a_Original(:, :, 2)=a_Original_temp;
            a_Original(:, :, 3)=a_Original_temp;
        end
        clear a_Original_temp;
        a_Skeleton=pad(pad(a_Skeleton));% Pad image. pad twice.
        location_values=find_end(a_Skeleton); % Find all the posible ends.
        if strcmp(manual_filament_reply, 'Yes') || strcmp(stretch_reply, 'Yes') % Want to manually select filaments wither way.
            [a_filament,quit_reply] = select_filament(a_Original, a_Skeleton, location_values, frame, reconstructions_approved, pix_threshold, stretch_reply); %if both manually selecting ways are Yes, the stretching one over rules
            if reconstructions_approved+size(a_filament, 1)>size(filament_pool,1)
                filament_pool=[filament_pool;cell(reconstructions_approved+size(a_filament, 1),3)]; % If run out of cells, reallocate more.
            end
            for ll=1:size(a_filament, 1)
                a_PositionMap=a_filament{ll, 1};
                pmN=size(a_PositionMap,1);
                a_BezierMap=zeros(1,2);  % initializing 3rd order Bezeir spline smoothed and redigitized filament curve position map as [i, j]
                a_devBezierMap=zeros(1,2); % initializing derivative of posion map of smoothed curve [di/dt, dj/dt]=[-dy/dt, dx/dt], t=[0, 1] is a parameter for redigitizing smoothed curve, dy/dx=-di/dt/dj/dt
                % Averaging three 3rd-order bezier spline one shifted toward back from previous one.
                if  pmN<=4 % minimum of 4 pixel for third order spline.
                    [a_BezierMap,a_devBezierMap]=bezier_reconstruct(a_PositionMap(1:pmN,:),10,3);
                elseif pmN==5
                    [a_BezierMap1,a_devBezierMap1]=bezier_reconstruct(a_PositionMap(1:pmN,:),10,3); %only do 3rd order (4 points)spline curve
                    [a_BezierMap2,a_devBezierMap2]=bezier_reconstruct(a_PositionMap(2:pmN,:),10,3);
                    a_BezierMap(1:3, :)=a_BezierMap1(1:3, :);
                    a_BezierMap(4:size(a_BezierMap1,1), :)=(a_BezierMap1(4:end, :)+a_BezierMap2(1:end-3, :))/2.;
                    a_BezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=a_BezierMap2(end-2:end, :);
                    a_devBezierMap(1:3, :)=a_devBezierMap1(1:3, :);
                    a_devBezierMap(4:size(a_BezierMap1,1), :)=(a_devBezierMap1(4:end, :)+a_devBezierMap2(1:end-3, :))/2.;
                    a_devBezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=a_devBezierMap2(end-2:end, :);
                else
                    [a_BezierMap1,a_devBezierMap1]=bezier_reconstruct(a_PositionMap(1:pmN,:),10,3);
                    [a_BezierMap2,a_devBezierMap2]=bezier_reconstruct(a_PositionMap(2:pmN,:),10,3);
                    [a_BezierMap3,a_devBezierMap3]=bezier_reconstruct(a_PositionMap(3:pmN,:),10,3);
                    a_BezierMap(1:3, :)=a_BezierMap1(1:3, :);
                    a_BezierMap(4:6, :)=(a_BezierMap1(4:6, :)+a_BezierMap2(1:3, :))/2.;
                    a_devBezierMap(1:3, :)=a_devBezierMap1(1:3, :);
                    a_devBezierMap(4:6, :)=(a_devBezierMap1(4:6, :)+a_devBezierMap2(1:3, :))/2.;
                    if size(a_BezierMap2, 1)<size(a_BezierMap1, 1) % size(a_BezierMap,1)== size(a_BezierMap1,1), size(a_BezierMap2,1) == size(a_BezierMap3,1)==size(a_BezierMap1,1)-9.
                        a_BezierMap(7:(size(a_BezierMap1,1)-6), :)=(a_BezierMap1(7:end-6, :)+a_BezierMap2(4:end, :)+a_BezierMap3(1:end-3, :))/3.; % size(a_BezierMap1,1)-12 points
                        a_BezierMap((size(a_BezierMap1,1)-5):(size(a_BezierMap1,1)-3), :)=(a_BezierMap1((size(a_BezierMap1,1)-5):(size(a_BezierMap1,1)-3), :)+a_BezierMap3((end-2):end, :))/2.; % 3 points
                        a_BezierMap((size(a_BezierMap1,1)-2):size(a_BezierMap1,1), :)=a_BezierMap1((size(a_BezierMap1,1)-2):end, :); % 3 points
                        a_devBezierMap(7:(size(a_BezierMap1,1)-6), :)=(a_devBezierMap1(7:end-6, :)+a_devBezierMap2(4:end, :)+a_devBezierMap3(1:end-3, :))/3.;
                        a_devBezierMap((size(a_BezierMap1,1)-5):(size(a_BezierMap1,1)-3), :)=(a_devBezierMap1((size(a_BezierMap1,1)-5):(size(a_BezierMap1,1)-3), :)+a_devBezierMap3((end-2):end, :))/2.;
                        a_devBezierMap((size(a_BezierMap1,1)-2):size(a_BezierMap1,1), :)=a_devBezierMap1((size(a_BezierMap1,1)-2):end, :);
                    elseif size(a_BezierMap3, 1)<size(a_BezierMap2, 1) % size(a_BezierMap,1)==size(a_BezierMap1,1)+3, size(a_BezierMap1,1)==size(a_BezierMap2,1)==size(a_BezierMap3,1)+9.
                        a_BezierMap(7:(size(a_BezierMap1,1)-3), :)=(a_BezierMap1(7:end-3, :)+a_BezierMap2(4:end-6, :)+a_BezierMap3(1:end, :))/3.; % size(a_BezierMap1,1)-9
                        a_BezierMap((size(a_BezierMap1,1)-2):size(a_BezierMap1,1), :)=(a_BezierMap1((size(a_BezierMap1,1)-2):end, :)+a_BezierMap2((end-5):(end-3), :))/2.; % 3 points
                        a_BezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=a_BezierMap2((size(a_BezierMap1,1)-2):end, :); % 3 points
                        a_devBezierMap(7:(size(a_BezierMap1,1)-3), :)=(a_devBezierMap1(7:end-3, :)+a_devBezierMap2(4:end-6, :)+a_devBezierMap3(1:end, :))/3.;
                        a_devBezierMap((size(a_BezierMap1,1)-2):size(a_BezierMap1,1), :)=(a_devBezierMap1((size(a_BezierMap1,1)-2):end, :)+a_devBezierMap2((end-5):(end-3), :))/2.;
                        a_devBezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=a_devBezierMap2((size(a_BezierMap1,1)-2):end, :);
                    else % size(a_BezierMap,1)==size(a_BezierMap1,1)+6, size(a_BezierMap1,1)==size(a_BezierMap2,1)==size(a_BezierMap3,1)
                        a_BezierMap(7:size(a_BezierMap1,1), :)=(a_BezierMap1(7:end, :)+a_BezierMap2(4:end-3, :)+a_BezierMap3(1:end-6, :))/3.; % size(a_BezierMap1,1)-6 points
                        a_BezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=(a_BezierMap2((size(a_BezierMap1,1)-2):end, :)+a_BezierMap3((end-5):(end-3), :))/2.;
                        a_BezierMap((size(a_BezierMap1,1)+4):(size(a_BezierMap1,1)+6), :)=a_BezierMap3((size(a_BezierMap1,1)-2):end, :);
                        a_devBezierMap(7:size(a_BezierMap1,1), :)=(a_devBezierMap1(7:end, :)+a_devBezierMap2(4:end-3, :)+a_devBezierMap3(1:end-6, :))/3.;
                        a_devBezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=(a_devBezierMap2((size(a_BezierMap1,1)-2):end, :)+a_devBezierMap2((end-5):(end-3), :))/2.;
                        a_devBezierMap((size(a_BezierMap1,1)+4):(size(a_BezierMap1,1)+6), :)=a_devBezierMap3((size(a_BezierMap1,1)-2):end, :);
                    end
                end
                reconstructions_approved=reconstructions_approved+1;
                filament_pool{reconstructions_approved, 1}=a_BezierMap; % Store  filament map in column 1 and derivative info in column two of the cell array. smoothed filament curve position map as [i, j]
                filament_pool{reconstructions_approved, 2}=a_devBezierMap; %derivative of posion map of smoothed curve [di/dt, dj/dt]=[-dy/dt, dx/dt], t=[0, 1] is a parameter for redigitizing smoothed curve, dy/dx=-di/dt/dj/dt
                filament_pool{reconstructions_approved, 3}=[frame, ll]; % Store the frame # and the  filament # in the frame as a single row arry in the third column of cell array
            end
        else
            if strcmp(select_ends_reply, 'Yes')  % Want to manually select ends.
                [location_values,quit_reply] = select_end(a_Original, location_values, frame, reconstructions_approved); % Updated end map |location_values| by hand selection.
            end
            % Reconstruct and analyze the filament shape for all the ends or all filaments selected.
            m=1;
            countn=0; % # of filaments reconstructed in this frame
            a_countf=0; % # of filaments reconstructed and approved in this frame.
            key_button=29; % Set default left arrow  selection for process all filament recontructd.
            while m<=size(location_values,1)
                % Find the pixel coordinates in a position map for the filament with the |m|th end.
                [a_PositionMap,m_new, location_values]=position(a_Skeleton,m, location_values);
                if m_new < m % The mth end didn't result a usable filament. Break!
                    continue;
                end
                countn=countn+1; % The mth end resulteds a single filament reconstruction. m_new==m+1 indicates both the ends of the reconstructed filament are in the |em|, at m and m+1 position. m_new==m indicates only one end of the reconstructed filament is in |em|, while the other end is a banching point with two ignored very short branches.
                m=m_new+1; % m_new is the number of emds in |location_values| that have been processed. Therefore the next one will process m=m_new+1.
                if size(a_PositionMap,1)>= pix_threshold % Reconstruct detected filaments larger than or equal to |pix_threshold| in settings.
                    % Reconstruct filaments using a 3rd order Bezier spline using 10
                    % points per segment using i j coordinates.
                    pmN=size(a_PositionMap,1);
                    a_BezierMap=zeros(1,2);  % initializing
                    a_devBezierMap=zeros(1,2); % initializing
                    % Averaging three 3rd-order bezier spline one shifted toward back from previous one.
                    if  pmN<=4 % minimum of 4 pixel for third order spline.
                        [a_BezierMap,a_devBezierMap]=bezier_reconstruct(a_PositionMap(1:pmN,2:3),10,3);
                    elseif pmN==5
                        [a_BezierMap1,a_devBezierMap1]=bezier_reconstruct(a_PositionMap(1:pmN,2:3),10,3);
                        [a_BezierMap2,a_devBezierMap2]=bezier_reconstruct(a_PositionMap(2:pmN,2:3),10,3);
                        a_BezierMap(1:3, :)=a_BezierMap1(1:3, :);
                        a_BezierMap(4:size(a_BezierMap1,1), :)=(a_BezierMap1(4:end, :)+a_BezierMap2(1:end-3, :))/2.;
                        a_BezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=a_BezierMap2(end-2:end, :);
                        a_devBezierMap(1:3, :)=a_devBezierMap1(1:3, :);
                        a_devBezierMap(4:size(a_BezierMap1,1), :)=(a_devBezierMap1(4:end, :)+a_devBezierMap2(1:end-3, :))/2.;
                        a_devBezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=a_devBezierMap2(end-2:end, :);
                    else
                        [a_BezierMap1,a_devBezierMap1]=bezier_reconstruct(a_PositionMap(1:pmN,2:3),10,3);
                        [a_BezierMap2,a_devBezierMap2]=bezier_reconstruct(a_PositionMap(2:pmN,2:3),10,3);
                        [a_BezierMap3,a_devBezierMap3]=bezier_reconstruct(a_PositionMap(3:pmN,2:3),10,3);
                        a_BezierMap(1:3, :)=a_BezierMap1(1:3, :);
                        a_BezierMap(4:6, :)=(a_BezierMap1(4:6, :)+a_BezierMap2(1:3, :))/2.;
                        a_devBezierMap(1:3, :)=a_devBezierMap1(1:3, :);
                        a_devBezierMap(4:6, :)=(a_devBezierMap1(4:6, :)+a_devBezierMap2(1:3, :))/2.;
                        if size(a_BezierMap2, 1)<size(a_BezierMap1, 1) % size(a_BezierMap,1)== size(a_BezierMap1,1), size(a_BezierMap2,1) == size(a_BezierMap3,1)==size(a_BezierMap1,1)-9.
                            a_BezierMap(7:(size(a_BezierMap1,1)-6), :)=(a_BezierMap1(7:end-6, :)+a_BezierMap2(4:end, :)+a_BezierMap3(1:end-3, :))/3.; % size(a_BezierMap1,1)-12 points
                            a_BezierMap((size(a_BezierMap1,1)-5):(size(a_BezierMap1,1)-3), :)=(a_BezierMap1((size(a_BezierMap1,1)-5):(size(a_BezierMap1,1)-3), :)+a_BezierMap3((end-2):end, :))/2.; % 3 points
                            a_BezierMap((size(a_BezierMap1,1)-2):size(a_BezierMap1,1), :)=a_BezierMap1((size(a_BezierMap1,1)-2):end, :); % 3 points
                            a_devBezierMap(7:(size(a_BezierMap1,1)-6), :)=(a_devBezierMap1(7:end-6, :)+a_devBezierMap2(4:end, :)+a_devBezierMap3(1:end-3, :))/3.;
                            a_devBezierMap((size(a_BezierMap1,1)-5):(size(a_BezierMap1,1)-3), :)=(a_devBezierMap1((size(a_BezierMap1,1)-5):(size(a_BezierMap1,1)-3), :)+a_devBezierMap3((end-2):end, :))/2.;
                            a_devBezierMap((size(a_BezierMap1,1)-2):size(a_BezierMap1,1), :)=a_devBezierMap1((size(a_BezierMap1,1)-2):end, :);
                        elseif size(a_BezierMap3, 1)<size(a_BezierMap2, 1) % size(a_BezierMap,1)==size(a_BezierMap1,1)+3, size(a_BezierMap1,1)==size(a_BezierMap2,1)==size(a_BezierMap3,1)+9.
                            a_BezierMap(7:(size(a_BezierMap1,1)-3), :)=(a_BezierMap1(7:end-3, :)+a_BezierMap2(4:end-6, :)+a_BezierMap3(1:end, :))/3.; % size(a_BezierMap1,1)-9
                            a_BezierMap((size(a_BezierMap1,1)-2):size(a_BezierMap1,1), :)=(a_BezierMap1((size(a_BezierMap1,1)-2):end, :)+a_BezierMap2((end-5):(end-3), :))/2.; % 3 points
                            a_BezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=a_BezierMap2((size(a_BezierMap1,1)-2):end, :); % 3 points
                            a_devBezierMap(7:(size(a_BezierMap1,1)-3), :)=(a_devBezierMap1(7:end-3, :)+a_devBezierMap2(4:end-6, :)+a_devBezierMap3(1:end, :))/3.;
                            a_devBezierMap((size(a_BezierMap1,1)-2):size(a_BezierMap1,1), :)=(a_devBezierMap1((size(a_BezierMap1,1)-2):end, :)+a_devBezierMap2((end-5):(end-3), :))/2.;
                            a_devBezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=a_devBezierMap2((size(a_BezierMap1,1)-2):end, :);
                        else % size(a_BezierMap,1)==size(a_BezierMap1,1)+6, size(a_BezierMap1,1)==size(a_BezierMap2,1)==size(a_BezierMap3,1)
                            a_BezierMap(7:size(a_BezierMap1,1), :)=(a_BezierMap1(7:end, :)+a_BezierMap2(4:end-3, :)+a_BezierMap3(1:end-6, :))/3.; % size(a_BezierMap1,1)-6 points
                            a_BezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=(a_BezierMap2((size(a_BezierMap1,1)-2):end, :)+a_BezierMap3((end-5):(end-3), :))/2.;
                            a_BezierMap((size(a_BezierMap1,1)+4):(size(a_BezierMap1,1)+6), :)=a_BezierMap3((size(a_BezierMap1,1)-2):end, :);
                            a_devBezierMap(7:size(a_BezierMap1,1), :)=(a_devBezierMap1(7:end, :)+a_devBezierMap2(4:end-3, :)+a_devBezierMap3(1:end-6, :))/3.;
                            a_devBezierMap((size(a_BezierMap1,1)+1):(size(a_BezierMap1,1)+3), :)=(a_devBezierMap2((size(a_BezierMap1,1)-2):end, :)+a_devBezierMap2((end-5):(end-3), :))/2.;
                            a_devBezierMap((size(a_BezierMap1,1)+4):(size(a_BezierMap1,1)+6), :)=a_devBezierMap3((size(a_BezierMap1,1)-2):end, :);
                        end
                    end
                    if strcmp(select_reply,'Yes') % Want to confirm filament shape reconstructions
                        % Calculate the end to end length create points using
                        % The position of the first end
                        a_end_A=a_BezierMap(1,:);
                        % The position of the last end
                        a_end_B=a_BezierMap(end,:);
                        % Calculate zoom parameters
                        a_mins=min(a_BezierMap);
                        a_maxs=max(a_BezierMap);
                        a_dim=max((a_maxs-a_mins))+max((a_maxs-a_mins))/2.; % the size a_dim is 1.5 of the filament size.
                        a_center=(a_mins+a_maxs)/2.; % V3.3 changed from this code: a_center=mean(a_BezierMap);
                        a_xmin=a_center(2)-a_dim/2.;
                        a_ymin=a_center(1)-a_dim/2.;
                        a_xmax=a_center(2)+a_dim/2.;
                        a_ymax=a_center(1)+a_dim/2.;
                        % Insert figure displaying reconstructed filaments.
                        % Compare to subfigure 6 and large-scale image to check
                        % for artifacts. Flip to match reconstruction.                        
                       
                        figure(confirm_fig);                        
                        subplot(2, 3, [1 5])
                        imshow(a_Original);
                        axis square
                        subplot(2, 3, [1 5])
                        % Overlay rectangle to show location.
                        drawrectangle(gca, 'Position', [(a_xmin-2) (a_ymin-2) a_dim a_dim], 'Color',[0 1 0]);
                        subplot(2, 3, 3) % Subfigure 1
                        imagesc(a_Original);
                        title(['Frame ',int2str(frame)])
                        %                         axis xy
                        axis square
                        axis([a_xmin-2 a_xmax-2 a_ymin-2 a_ymax-2])
                        figure(confirm_fig)
                        subplot(2, 3, 6) % Subfigure 2
                        % Plot detected and reconstructed filament.
                        imshow(a_Original);
                        hold on
                        plot(a_PositionMap(:,3)-2,a_PositionMap(:,2)-2,'bs','MarkerSize',3,'LineWidth',2)
                        hold on
                        plot(a_BezierMap(:,2)-2,a_BezierMap(:,1)-2,'g-','LineWidth',3)
                        scatter(a_end_A(1,2)-2,a_end_A(1,1)-2,50,'g','filled')
                        scatter(a_end_B(1,2)-2,a_end_B(1,1)-2,50,'c','filled')
                        scatter(a_center(2)-2,a_center(1)-2,50,'b','filled')
                        a_endtoend=[a_end_A-2;a_end_B-2];
                        plot(a_endtoend(:,2),a_endtoend(:,1),'b-','LineWidth',1)
                        shape_selected=round(size(location_values,1)/2);
                        title(['Frame Reconstruction ',int2str(countn), ', ',num2str(shape_selected), ' Shapes Collected, ', num2str(reconstructions_approved), ' Filaments Approved'])
                        axis ij
                        axis square
                        axis([a_xmin-2 a_xmax-2 a_ymin-2 a_ymax-2])
                        hold off
                        %ignore return and function keypresses and waits for proper input.                        
                        while 1
                            [~,~, key_button] = myginput(1, 'arrow');
                            if isequal(key_button, [])
                                key_button = 600; % do nothing.
                                continue;
                            end
                            if (key_button == 28) || (key_button == 29) || (key_button == 52) || (key_button == 54)
                                break;
                            elseif (key_button == 30) || (key_button == 56)
                                quit_reply = questdlg('Want to finish selections for all? (Select "yes" if quit for all and select "No" to continue in the next frame)','Well?',...
                                    'Yes','No','Cancel','No');
                                if strcmp(quit_reply, 'No')
                                    figure('units','pixels',...
                                        'position',[500 200 290 140],...
                                        'menubar','none',...
                                        'name','Look here!',...
                                        'numbertitle','off',...
                                        'resize','off');
                                    uicontrol('Style','text','Position',[20 65 250 60],'FontSize',12,...
                                        'String','Continue to the next frame');
                                    uicontrol('Position',[45 20 200 40],'String','OK',...
                                        'Callback','uiresume(gcbf)');
                                    uiwait(gcf);
                                    close;
                                end
                                break;
                            else
                                figure('units','pixels',...
                                    'position',[500 200 440 180],...
                                    'menubar','none',...
                                    'name','Look here!',...
                                    'numbertitle','off',...
                                    'resize','off');
                                uicontrol('Style','text','Position',[20 65 400 80],'FontSize',14,...
                                    'String','When crosshairs appear on Filament Confirmation Figure: right arrow to accept, left arrow to reject or up arrow to end selections',...
                                    'BackgroundColor',[0.8 0.8 0.8]);
                                uicontrol('Position',[120 20 200 40],'String','OK','FontSize',14,...
                                    'Callback','uiresume(gcbf)');
                                uiwait(gcf);
                                close;
                            end
                        end
                    end
                    if (key_button==29) || (key_button==54)  % Confirm reconstruction in selections table
                        if reconstructions_approved>=filamentN % number of fimaments collected is over the limit.
                            quit_reply='Yes';
                        else
                            % Store confirmed reconstructed a_BezierMap and a_devBezierMap in a cell array for analysis.
                            reconstructions_approved=reconstructions_approved+1;  % Record the filament in total counting.
                            a_countf = a_countf+1; % Record the filament in this frame.
                            if reconstructions_approved>size(filament_pool,1)
                                filament_pool=[filament_pool;cell(reconstructions_approved,3)]; % If run out of cells, reallocate more.
                            end
                            filament_pool{reconstructions_approved, 1}=a_BezierMap; % Store  filament map in column 1 and derivative info in column two of the cell array. smoothed filament curve position map as [i, j]
                            filament_pool{reconstructions_approved, 2}=a_devBezierMap; %derivative of posion map of smoothed curve [di/dt, dj/dt]=[-dy/dt, dx/dt], t=[0, 1] is a parameter for redigitizing smoothed curve, dy/dx=-di/dt/dj/dt
                            filament_pool{reconstructions_approved, 3}=[frame, a_countf]; % Store the frame # and the  filament # in the frame as a single row arry in the third column of cell array
                        end
                    end
                end
                if (key_button == 30) || (key_button == 56)
                    break;
                end
            end
        end
        clear a*
        if strcmp(quit_reply, 'Yes')
            break;
        end
    end
    filament_pool((reconstructions_approved+1):end,:)=[]; % Delete empty cells.
    cd(savepath);
    % Store  filament map in column 1 and derivative info in column two of the
    % cell array filament_pool.    
    save([savefile,'.mat'],'filament_pool')
    cd(home);
    if strcmp(select_reply,'Yes') && strcmp(manual_filament_reply, 'No')
        close (confirm_fig);
    end
    if strcmp(quit_reply, 'Yes')
        quit_contine_later = questdlg('Want to finish selections and continue later? (Select "yes" to quit for now and continue later and select "No" to process selected)','Well?',...
            'Yes','No','Yes');
        if strcmp(quit_contine_later, 'Yes')
            
            figure('units','pixels',...
                'position',[500 200 440 200],...
                'menubar','none',...
                'name','Look here!',...
                'numbertitle','off',...
                'resize','off');
            uicontrol('Style','text','Position',[20 20 400 160],'FontSize',12,...
                'HorizontalAlignment', 'left', 'String',['Filament selections completed on frame #',num2str(frame),'.', ...
                ' Total ', num2str(reconstructions_approved), ' Selections for ',file2, ' saved as ',savefile, '.mat. ' , newline ...
                'to be continue later on as wish. The program now closes. Goodbye! ' ],...
                'BackgroundColor',[0.8 0.8 0.8]);
            uicontrol('Position',[120 20 200 40],'String','OK',...
                'Callback','uiresume(gcbf)');
            uiwait(gcf);
            close;
            
            Summary_file = fopen(Summary_filename, 'a');
            fprintf(Summary_file, '%s\n', ['Filament selections completed on frame #',num2str(frame),' and total ', num2str(reconstructions_approved), ' Selections for ',file2]);
            fprintf(Summary_file, '%s\n\n', ['The selectionssaved are saved as ',savefile, '.mat to be continue later on as wish.']);
            fclose(Summary_file);
            return;
        end
    end        
    set(f_notification, 'Visible', 'on'); % Notification is on.
    uicontrol(f_notification,'Style','text','Position',[20 220 400 260],'FontSize',12,...
        'HorizontalAlignment', 'left', 'String',['Filament selections completed on frame #',num2str(frame),'.', ...
        ' Total ', num2str(reconstructions_approved), ' Selections for ',file2, ' saved as ',savefile, '.mat. ' , newline ...
        'Now processing the collected filaments. Please be patient ... ', newline ...
        'After finishing calculating data, It takes time to output files because of huge data set. ', ...
        'Please wait patiently. Putting music on helps or have a cup of coffee and come back later...' ],...
        'BackgroundColor',[0.8 0.8 0.8]);
end
clear countn filamentN frame frameN key_button location_values m m_new pmN reconstructions_approved scrsz start_frame shape_selected;
% Only reconstructed filament info cell array |filament_pool| and file path
% infomation are needed after this point.

if size(filament_pool,1)==0 % If no filament  was collected, terminate the program.
    figure('units','pixels',...
           'position',[500 200 440 180],...
           'menubar','none',...
           'name','Look here!',...
           'numbertitle','off',...
           'resize','off');
    uicontrol('Style','text','Position',[20 65 400 80],'FontSize',14,...
              'String','No filaments have been selected or all filaments selected have branches. You need at least one filament to do analysis. Goodbye.',...
              'BackgroundColor',[0.8 0.8 0.8]);
    uicontrol('Position',[120 20 200 40],'String','OK','FontSize',14,...
              'Callback','uiresume(gcbf)');
    uiwait(gcf);
    close;
    return;
end

tic
% Collect data Block from selections
WB=waitbar(0,'Calculating.  Progress below.'); % set up waitbar to indicate progress
cd(home);
% This block analyzes the selected shapes 
filament_n=size(filament_pool,1);
tot_data_ppt=0; % Total filament data points. 
tot_Cs_ppt=0; % Total number of correlation data.
for k=1:filament_n
    tot_data_ppt=tot_data_ppt+size(filament_pool{k,1},1); % Total number data points of all filaments.
    tot_Cs_ppt=tot_Cs_ppt+size(filament_pool{k,1},1)*(size(filament_pool{k,1},1)-1)/2; % # of correlation data for every filament is size(filament)*(size(filament)-1)/2.
end
CsDataPool=zeros(tot_Cs_ppt,3);
ModeAmplitudes=zeros(filament_n,17); %end-to-edn and bending modes
% lengths=zeros(filament_n,1);
curvatures=zeros(tot_data_ppt-filament_n,6); %every filament has # of subunits -1 curvatures and delta_angles
Points=zeros(tot_data_ppt,9);
saved_ppt=0;
saved_curvature=0;
saved_Cs=0;
for filament=1:filament_n % for end selection check confirmation
    waitbar(filament/filament_n, WB)
    a_BezierMap=filament_pool{filament,1}; % Load the filament map from first column.
    a_devBezierMap=filament_pool{filament,2}; % Load the filament derivative infomation from second column.
    % Calculate the end to end length with the first and last points
    a_endtoend_length=(((a_BezierMap(1,1)-a_BezierMap(end,1))^2+(a_BezierMap(1,2)-a_BezierMap(end,2))^2)^(1/2))*um_pix;   
    a_L=contourL(a_BezierMap*um_pix); % Contour length. 
    % Calculate the fourier coefficients of first 10 modes of theta(s).
    a_An=fourier_coeff(a_BezierMap*um_pix, a_devBezierMap*um_pix);% |um_pix| convert pixels to um from settings above.  An is mode n Fourier coefficient,   
    a_center=mean(a_BezierMap); % The center  coordinate of the filament.        
   
    % Collect end-to-end length and mode amplitudes    
    ModeAmplitudes(filament,1)=a_L;% first column contour length    
    ModeAmplitudes(filament,2)=a_endtoend_length;% second column end to end length
    ModeAmplitudes(filament,3)=a_endtoend_length^2;% third column end to end length^2
    ModeAmplitudes(filament,4:5)=a_center;%  4th and 5th column filament centroid
    ModeAmplitudes(filament,6:15)=a_An(1:10); % 6--15th column first 10 mode amplitudes
    ModeAmplitudes(filament,16:17)=filament_pool{filament,3}; % 16th column: frame # and 17th column filament # in that frame.
    
    npts=size(a_BezierMap,1); %Number of points in a_BezierMap
    %Collect all filaments and its derivative to parameter t, all in unit of pixel.
    Points((saved_ppt+1):(saved_ppt+npts),1)=filament;
    Points((saved_ppt+1):(saved_ppt+npts),2:3)=a_BezierMap; %coordinates in unit of pixel
    Points((saved_ppt+1):(saved_ppt+npts),4:5)=a_devBezierMap; %derivative to parameter t of coordinates in unit of pixel
    Points((saved_ppt+1):(saved_ppt+npts),6)=tangent_angle(a_devBezierMap*um_pix); % |um_pix| convert pixels to um from settings above. The tangent angle is relative to j (x-axis), angle in unit of rad.
    Points((saved_ppt+1):(saved_ppt+npts),7)=part_contourL(a_BezierMap*um_pix); %contour length from one end to the current position in unit of pixel size
    Points((saved_ppt+1):(saved_ppt+npts),8:9)=ones(npts, 1)*filament_pool{filament,3}; % 6th column: frame # and 7th column filament # in that frame.
    
    % Calculate the cosine correlation data
    CsDataPool((saved_Cs+1):(saved_Cs+npts*(npts-1)/2), :)=cos_autocorr(a_BezierMap*um_pix, a_devBezierMap*um_pix);% |um_pix| convert pixels to um from settings above.
    saved_Cs=saved_Cs+npts*(npts-1)/2;
    
    % Collect tangent angles along contour length in unit of um (contour)
    % and rad (angle)
    curvatures((saved_curvature+1):(saved_curvature+npts-1),1) = segmentL(a_BezierMap*um_pix);   %segment length  
    curvatures((saved_curvature+1):(saved_curvature+npts-1), 2) = Points((saved_ppt+2):(saved_ppt+npts),6)-Points((saved_ppt+1):(saved_ppt+npts-1),6); %tangent angle difference in unit of rad
    curvatures((saved_curvature+1):(saved_curvature+npts-1), 3) = curvatures((saved_curvature+1):(saved_curvature+npts-1), 2)./curvatures((saved_curvature+1):(saved_curvature+npts-1),1); %cureture = delta(angle)/segment_length in unit of 1/pixel size
    curvatures((saved_curvature+1):(saved_curvature+npts-1),4)=filament; %filament iedentifier
    curvatures((saved_curvature+1):(saved_curvature+npts-1),5:6) = ones(npts-1, 1)*filament_pool{filament,3}; % 4th column: frame # and 5th column filament # in that frame.
    saved_ppt=saved_ppt+npts;
    saved_curvature=saved_curvature+npts-1;
    clear a_*
end
close(WB)

% Sort final  data and save.
curvatures(saved_curvature+1:end, :) = []; %delete empty cells
CsDataPool=sortrows(CsDataPool); 
% ModeAmplitudes=sortrows(ModeAmplitudes);
ModeAmplitudes=sortrows(ModeAmplitudes); %sorts the rows of a matrix in ascending order based on the elements in the first column

%save data
cd(savepath);
writematrix(round(CsDataPool, 6), [savefile,'_Cs_raw.txt'], 'Delimiter', '\t'); %raw Cs 
writematrix(round(Points, 6), [savefile,'_fil.txt'], 'Delimiter', '\t'); %coordinates, derivatives, poartial contour, and tangent angles
clear Points
writematrix(round(curvatures, 6), [savefile,'_curv.txt'], 'Delimiter', '\t'); %delta_anlges and curvature at every position
clear curvatures
% Save bending mode amplitudes
writematrix(round(ModeAmplitudes, 6), [savefile,'_bend.txt'], 'Delimiter', '\t'); %filament length, end-to-ten length, bending modes
% save end to end length vs contour length data
cd(home);

Lmean=mean(ModeAmplitudes(:, 1)); % Average contour length.
Lste=std(ModeAmplitudes(:, 1))/filament_n^0.5; % Standard error of contour lengths.
uicontrol(f_notification, 'Style','text','Position',[20 120 400 80],'FontSize',12,...
    'String',['Cosine correlations done for ', num2str(filament),' selected filaments with an average length ',num2str(Lmean),' +/- ',num2str(Lste),' um'],...
    'BackgroundColor',[0.8 0.8 0.8]);

Summary_file = fopen(Summary_filename, 'a');
fprintf(Summary_file,  '%s\n\n', ['1. ', num2str(filament_n), ' filaments have been analyzed.']);
fprintf(Summary_file, '%s\n\n', '2. Please consult file_readme.txt for files exported from the ananlysis');
fprintf(Summary_file, '%s\n\n', ['3. selected filaments has an average length ',num2str(Lmean),' +/- ',num2str(Lste),' um (error of the mean)']);
fclose(Summary_file);

process_timer = toc;
disp(['time used for processing and outputing files ', num2str(process_timer), ' second.'])
tic % time average process

fs=20;%font size for plots
% Setup figure for fitting results
f_Lp=figure('units','pixels',...
    'position',[370 20 440 620],...
    'menubar','none',...
    'name','Lp Values from Fitting the Data',...
    'numbertitle','off',...
    'resize','off');

if strcmp(mean_reply, 'Yes')
    %calculate average  
    bin_n=ceil((CsDataPool(end,1)-CsDataPool(1,1))/um_pix); % Number of bins. Divide the longest-shortest correlation length length by bin size um_pix, and the remainder is also counted as a bin.
    CsDataAve=zeros(bin_n,11);  %file to save averaged data. col 1, segment length, col 2-4 average cos correlation, standard deviation and standard error, col 5-7, one exponential fit of cos correlation and confident intervals, col 8-10, average correlation angles, standard deviation and standard error, col 11, number of data points in a bin for averging.
    c_CsDataAve_size = 0;
    for bin=1:bin_n %contour length at right side of data bin
        c_CsDataAve = CsDataPool((CsDataPool(:,1)>=(bin-1)*um_pix+CsDataPool(1,1))&(CsDataPool(:,1)<bin*um_pix+CsDataPool(1,1)),:); % The first bin starts from shortest correlation length, and bins cover from shorest to longest correlation length
        c_Num_of_data = size(c_CsDataAve,1); % Number of data points in the same bin to calculate average.
        if c_Num_of_data~=0
            c_CsDataAve_size = c_CsDataAve_size + 1;            
            CsDataAve(c_CsDataAve_size,1) = mean(c_CsDataAve(:,1));
            CsDataAve(c_CsDataAve_size,2) = mean(c_CsDataAve(:,2));
            CsDataAve(c_CsDataAve_size,3) = std(c_CsDataAve(:,2));
            CsDataAve(c_CsDataAve_size,4) = CsDataAve(bin,3)/c_Num_of_data^0.5;
            CsDataAve(c_CsDataAve_size,8) = mean(c_CsDataAve(:,3));
            CsDataAve(c_CsDataAve_size,9) = std(c_CsDataAve(:,3));
            CsDataAve(c_CsDataAve_size,10) = CsDataAve(bin,9)/c_Num_of_data^0.5; 
            CsDataAve(c_CsDataAve_size,11) = c_Num_of_data;
        end        
    end
    if c_CsDataAve_size < bin_n
        CsDataAve((c_CsDataAve_size+1):end,:) = []; % delete extra zero rows
    end    
        
%     % Determine where to cut data for exponential fit. If samples are
%     % random and sample size is large enough, the average correlation angle
%     % at every correlation length should be 0. If the average correlation
%     % angle^2 >0.005  (by experince), the samples are either under
%     % sampled or samples are not random.
%     c_fit_limit_index=round(0.9*c_CsDataAve_size);  % Set 0.9 of maxL as limit to look for boundary for the fit    
%     c_left_mean = mean(CsDataAve(1:c_fit_limit_index,8).^2); % CsDataAve(1:c_fit_limit_index,8) correlation angle
%     c_left_stdv = std(CsDataAve(1:c_fit_limit_index,8).^2); 
%     if c_left_mean < 0.005 && c_left_stdv < 0.005  % To find fit boundary at average correlation angle < 0.07 (i.e. 0.022293Pi or 4 degree) and correlation angle stdv < 0.07
%         c_fit_boundary_index = c_fit_limit_index;
%     else %searh 
%         c_fit_boundary_low_limit_index = 1; %Reccord fit boundary low limit  
%         c_fit_boundary_index = c_fit_boundary_low_limit_index;
%         while c_fit_limit_index - c_fit_boundary_low_limit_index > 2
%             c_fit_boundary_index = c_fit_boundary_index  + round(0.382*(c_fit_limit_index - c_fit_boundary_low_limit_index + 1)); % Set attempt fit boundary at 1-0.618 of c_fit_limit_index-c_fit_boundary_low_limit_index
%             c_left_mean = mean(CsDataAve(1:c_fit_boundary_index,8).^2);
%             c_left_stdv = std(CsDataAve(1:c_fit_boundary_index,8).^2);
%             if c_left_mean < 0.005 && c_left_stdv < 0.005
%                 c_fit_boundary_low_limit_index = c_fit_boundary_index;
%             else
%                 c_fit_limit_index = c_fit_boundary_index;
%                 c_fit_boundary_index = c_fit_boundary_low_limit_index;
%             end
%         end
%     end  

    %The above method to determine the cutting point didn't work well for
    %some reason and give up for now. Use the cutting point in V4_2_3_Aug16
    %instead.
    c_fit_limit_index=round(0.9*c_CsDataAve_size);  % Set 0.9 of maxL as limit to look for boundary for the fit 
    [~, c_fit_boundary_index]=max(CsDataAve(1:c_fit_limit_index,9)); % Find the boundary and its index for maximum number of stardard deviation for setting Lp exponential fit limit
    w=ones(c_fit_boundary_index,1); % weight for fit
    for k=1:c_fit_boundary_index
        if CsDataAve(k,4)> 0
            w(k)=1/CsDataAve(k,4)^2; %calculate the weights for the fits as the inverse of the standard error squared
        end
    end
    % Fit data to cosine correlation function.
    Lp_guess=10.0;
    A_guess=0.97;
    s_cos = fitoptions('Method','NonlinearLeastSquares', 'Robust', 'LAR', 'Lower',[0,1e-10],'Upper',[1,Inf],'Startpoint',[A_guess, Lp_guess],'Weights',w);
    f_cos = fittype('A*exp(-x/(Lp*2.0))','options',s_cos); 
    uicontrol(f_Lp, 'Style','text','Position',[20 540 400 60],'FontSize',12,...
        'String',['The calculation was based on pixel size: ', num2str(um_pix), ' um/pixel \n', ...
        ' Method 1: calculating fit of =A*exp(-x/(2*Lp)) to Cosine Correlation Data with standard error as weight'],...
        'BackgroundColor',[0.8 0.8 0.8]);

    [Ac,Agof] = fit(CsDataAve(1:c_fit_boundary_index,1),CsDataAve(1:c_fit_boundary_index,2),f_cos); % Fit only to the maximum data deviation
    Aconf=confint(Ac);
    uicontrol(f_Lp, 'Style','text','Position',[20 400 400 120],'FontSize',12,...
        'String',['The averaged correlation data has been cut off at point ', num2str(c_fit_boundary_index), ', contour length s = ', ...
        num2str(CsDataAve(c_fit_boundary_index,1)), ' and fit to Lp=', num2str(Ac.Lp),' (',num2str(Aconf(1,2)),', ',num2str(Aconf(2, 2)), ...
        ') micrometers, and a scaling factor of A=', num2str(Ac.A),' (',num2str(Aconf(1, 1)),' ',num2str(Aconf(2, 1)), ...
        ') with the adjusted R square=', num2str(Agof.adjrsquare)], 'BackgroundColor',[0.8 0.8 0.8]);
    %disp(Ac)
    %disp(Agof)
    %Generate fit curve and confident interval and store
    CsDataAve(:,5) = Ac.A*exp(-CsDataAve(:,1)/2.0/Ac.Lp); %one exponential fit
    CsDataAve(:,6) = Ac.A*exp(-CsDataAve(:,1)/2.0/Aconf(1,2)); %confident interval upper bound
    CsDataAve(:,7) = Ac.A*exp(-CsDataAve(:,1)/2.0/Aconf(2,2)); %confident interval lower bound
    
    Summary_file = fopen(Summary_filename, 'a'); %save the result
    fprintf(Summary_file,'%s\n', '4. Lp Values from Fitting the Data:');
    fprintf(Summary_file,'%s\n',  'Method 1: calculating fit of =A*exp(x/-2/Lp) to Cosine Correlation Data with standard error as weight');
    fprintf(Summary_file,'%s\n\n', ['The averaged correlation data has been cut off at point ', num2str(c_fit_boundary_index), ...
        ', contour length s = ', num2str(CsDataAve(c_fit_boundary_index,1)), ' and fit to Lp=', num2str(Ac.Lp),' (',num2str(Aconf(1,2)), ...
        ', ',num2str(Aconf(2, 2)),') micrometers, and a scaling factor of A=', num2str(Ac.A),' (',num2str(Aconf(1, 1)),' ',num2str(Aconf(2, 1)), ...
        ') with the adjusted R square=', num2str(Agof.adjrsquare)]);
    fclose(Summary_file);
    clear c_*;
     
    % Display fit to angular correlations data.
    figure();
    plot(downsample(CsDataPool(:,1),3*filament_n),downsample(CsDataPool(:,2),3*filament_n),'k.','MarkerSize',1)
    hold on
    plot(CsDataAve(:,1),CsDataAve(:,2),'go')
    plot(CsDataAve(:,1),CsDataAve(:,5),'r-','LineWidth',2)
    plot(CsDataAve(:,1),CsDataAve(:,6),'r--','LineWidth',2)
    plot(CsDataAve(:,1),CsDataAve(:,7),'r--','LineWidth',2)
    title('Cosine Correlation Data','FontSize',fs,'FontName','Arial')
    ylabel('Angular Cos Correlation, \it C(s)','FontSize',fs,'FontName','Arial')
    xlabel('Contour length, \it s \rm (\mum)','FontSize',fs,'FontName','Arial')
    legend('Data Sample','Average','Fit')
    set(legend,'Location','NorthEastOutside')
    set(gca,'FontSize',fs,'FontName','Arial','LineWidth',2)
    
    figure()
    plot(CsDataAve(:,1),CsDataAve(:,4),'k-','LineWidth',2)
    title('\it <C(s)> Standard Error','FontSize',fs,'FontName','Arial')
    ylabel('Standard Error','FontSize',fs,'FontName','Arial')
    xlabel('Contour length, \it s \rm (\mum)','FontSize',fs,'FontName','Arial')
    set(gca,'FontSize',fs,'FontName','Arial','LineWidth',2)
    
    figure()
    plot(CsDataAve(:,1),CsDataAve(:,2)-CsDataAve(:,5),'kx','LineWidth',2)
    title('Cosine Correlation Function Fit Residuals','FontSize',fs,'FontName','Arial')
    ylabel('Fit Residuals','FontSize',fs,'FontName','Arial')
    xlabel('Contour length, \it s \rm (\mum)','FontSize',fs,'FontName','Arial')
    set(gca,'FontSize',fs,'FontName','Arial','LineWidth',2)
    
    % Save averaged data.
    cd(savepath); 
    writematrix(round(CsDataAve, 6), [savefile,'_ave_Cs.txt'], 'Delimiter', '\t');
    cd(home);    
   
else
    % Display data.   
    uicontrol(f_Lp, 'Style','text','Position',[20 520 400 80],'FontSize',12,...
        'String','Method 1: calculating fit of =A*exp(x/-2/Lp) to Cosine Correlation Data with standard error as weight. Since the average is not calculated, fitting is not done and plot only the raw cosine correlation data',...
        'BackgroundColor',[0.8 0.8 0.8]);    
    figure()
    plot(downsample(CsDataPool(:,1),3*filament_n),downsample(CsDataPool(:,2),3*filament_n),'k.','MarkerSize',1)
    title('Cosine Correlation Data: raw Data (black)')
end
clear CsDataPool CsDataAve;

ModeAmplitudes_square = [ModeAmplitudes(:, 1).^2, ModeAmplitudes(:, 6:15).^2]; %col1: contour length^2, col2-11: first 10 mode amplitue^2
if (ModeAmplitudes(end, 1)-ModeAmplitudes(1, 1)>2.0*um_pix) && (filament_n > 2) %difference btween longest and shortest is lrarger than 2 pixel lengths and there are more than 3 filaments
    % Calculate the persitence length from end to end lengths
    uicontrol(f_Lp, 'Style','text','Position',[20 300 400 80],'FontSize',12,...
        'String','Method 2: Calculating fit of endtoend^2=8*Lp^2*(exp(-x/(2*Lp))-1+x/(2*Lp)) to 2D end-to-end length^2 vs contour length Data',...
        'BackgroundColor',[0.8 0.8 0.8]);

    endtoend_vs_length=zeros(filament_n+1, 2); %use V4_2_3_Aug16 method to include [0, 0] data point in the fitting
    endtoend_vs_length(2:end, 1)= ModeAmplitudes(:,1);
    endtoend_vs_length(2:end, 2)= ModeAmplitudes(:,3);

    Lp_guess=10.0;
    s_endtoend = fitoptions('Method','NonlinearLeastSquares', 'Robust', 'LAR', 'MaxFunEvals',100,'TolFun', 1e-20,'TolX', 1e-20, 'Lower',1e-10,'Upper',Inf,'Startpoint', Lp_guess);
    f_endtoend = fittype('8.0*(Lp^2.0)*(exp(-x/(2.0*Lp))-1.0+x/(2.0*Lp))','options',s_endtoend);
    [Ec,Egof] = fit(endtoend_vs_length(:,1), endtoend_vs_length(:,2), f_endtoend);
    Econf=confint(Ec);
    uicontrol(f_Lp, 'Style','text','Position',[20 210 400 70],'FontSize',12,...
              'String',['The end to end length data fit to Lp=', num2str(Ec.Lp),  ' (',num2str(Econf(1)),' ',num2str(Econf(2)),') micrometers with the adjusted R square=', num2str(Egof.adjrsquare)],...
              'BackgroundColor',[0.8 0.8 0.8]);
%     disp(Ec)
%     disp(Egof)
    
    Summary_file = fopen(Summary_filename, 'a'); %save the result
    fprintf(Summary_file, '%s\n', 'Method 2: Calculating fit of endtoend^2=8*Lp^2*(exp(-x/(2*Lp))-1+x/(2*Lp)) to 2D end-to-end length^2 vs contour length Data');
    fprintf(Summary_file, '%s\n\n', ['The end to end length data fit to Lp=', num2str(Ec.Lp),  ' (',num2str(Econf(1)),' ',num2str(Econf(2)),') micrometers with the adjusted R square=', num2str(Egof.adjrsquare)]);
    fclose(Summary_file); 
    
    l_data_range = (1.0+0.1)*ModeAmplitudes(end,1); %set up range for plotting.  range is longest filament + 10% of it
    l_fit_size = ceil(l_data_range/um_pix) +1; %number of data points for fit
    l_fit=zeros(l_fit_size, 4);
    l_fit(:, 1) = linspace(0, l_data_range, l_fit_size); %set up x-axis for fit plotting [0, l_data_range],interval between 2 points is pixel size
    l_fit(:, 2) = 8.0*Ec.Lp^2*(exp(-l_fit(:, 1)/(2.0*Ec.Lp))-1.0+l_fit(:, 1)/(2.0*Ec.Lp)); %y-axis for fit plot
    l_fit(:, 3) = 8.0*Econf(1)^2*(exp(-l_fit(:, 1)/(2.0*Econf(1)))-1.0+l_fit(:, 1)/(2.0*Econf(1))); %confident interval upper bound of the fit
    l_fit(:, 4) = 8.0*Econf(2)^2*(exp(-l_fit(:, 1)/(2.0*Econf(2)))-1.0+l_fit(:, 1)/(2.0*Econf(2))); %confident interval lower bound of the fit
    
    % save end to end length vs contour length data
    cd(savepath);
    writematrix(round(l_fit, 6), [savefile,'_endtoend_fit.txt'], 'Delimiter', '\t'); %fit to end to end
    cd(home);
    
    %plot
    figure()
    hold on    
    plot(endtoend_vs_length(:,1), endtoend_vs_length(:,2), 'kx',l_fit(:, 1), l_fit(:, 2),'r-','LineWidth',2)
    plot(l_fit(:, 1), l_fit(:, 3), 'r--','LineWidth',1)
    plot(l_fit(:, 1), l_fit(:, 4), 'r--','LineWidth',1)
    title('End-to-End Length Fit','FontSize',fs,'FontName','Arial')
    ylabel('End-to-End Length Squared, \it r\rm^{2} ','FontSize',fs,'FontName','Arial')
    xlabel('Contour length, \it s \rm (\mum)','FontSize',fs,'FontName','Arial')
    set(gca,'FontSize',fs,'FontName','Arial','LineWidth',2)
    
    figure() %fitting residure plot
    plot(ModeAmplitudes(:,1),  ModeAmplitudes(:,3)-(8.0*Ec.Lp^2*(exp(-ModeAmplitudes(:,1)/(2.0*Ec.Lp))-1.0+ ModeAmplitudes(:,1)/(2.0*Ec.Lp))),'kx','LineWidth',2)
    title('End-to-End Function Fit Residuals','FontSize',fs,'FontName','Arial')
    ylabel('Residuals','FontSize',fs,'FontName','Arial')
    xlabel('Contour length, \it s \rm (\mum)','FontSize',fs,'FontName','Arial')
    set(gca,'FontSize',fs,'FontName','Arial','LineWidth',2)  
    % end of Calculate the persitence length from end to end lengths
    
    % Fit data to contour length dependent Fourier coefficients function (Howard book).
    uicontrol(f_Lp, 'Style','text','Position',[20 100 400 90],'FontSize',12,...
              'String','Method 3: Calculating linear fit of an^2=(1/Lp)*((L/n/pi)^2)+an0^2 to Fourier Coefficient Data with Mode Amplitue^2 vs Contour Length^2. an0 is filament at rest and is assumed as 0',...
              'BackgroundColor',[0.8 0.8 0.8]);

    Ac=zeros(10,1); % Store fitting parameters |Ac|=[Lp] for mode 1-10.
    weight=zeros(10,1); % Store weight as 1/standard deviation^2  |weight|=[Lp_weight] for mode 1-10. 
    s_mode = fitoptions('Method','LinearLeastSquares','Lower',0,'Upper',Inf);
    f_mode = fittype({'x'}, 'coefficients', {'c'}, 'options', s_mode);
    for n=1:10
        fit_obj = fit(ModeAmplitudes_square(:,1), ModeAmplitudes_square(:,n+1),f_mode);
        Ac_temp=coeffvalues(fit_obj);
        if Ac_temp(1,1)==0
            Ac(n,1)=Inf;
        else
            Ac(n,1)=1.0/Ac_temp(1,1)/(pi*n)^2;  % Lp=1/(Ac_temp(1,1)*n^2*pi^2)
        end
        Aconf_temp=confint(fit_obj);
        if Aconf_temp(1,1)==0 || isnan(Aconf_temp(1,1))||Aconf_temp(2,1)==0 || isnan(Aconf_temp(2,1))
            weight(n,1)=1;
        else
            weight(n,1)=(Ac(n,1)*pi*n)^4/(Aconf_temp(1,1)-Aconf_temp(2,1))^2; % weight_Lp=1/std_Lp^2, and std_Lp^2=delta(c1)^2/(c1*n*pi)^4
        end
    end 
    Lp_mean=(weight(:,1)')*Ac(:,1)/sum(weight(:,1)); % Average Lp is a weighted average=sum(Lp*weight)/sum(weight)    
    Lp_ste=1.0/sqrt(sum(weight(:,1))); % Average Lp standrd deviation is 1/sum(weight)^(1/2)
    %Plot Lp calculated from the fisr 4 modes
    figure()
    plot(ModeAmplitudes_square(:,1),ModeAmplitudes_square(:,2),'ro',ModeAmplitudes_square(:,1), ModeAmplitudes_square(:,3).^2,'go',...
        ModeAmplitudes_square(:,1),ModeAmplitudes_square(:,4),'bo',ModeAmplitudes_square(:,1),ModeAmplitudes_square(:,5),'co',...
            ModeAmplitudes_square(:,1), ModeAmplitudes_square(:,1)/(pi*1)^2/Ac(1,1),'r-',...
            ModeAmplitudes_square(:,1), ModeAmplitudes_square(:,1)/(pi*2)^2/Ac(2,1),'g-',...
            ModeAmplitudes_square(:,1), ModeAmplitudes_square(:,1)/(pi*3)^2/Ac(3,1),'b-',...
            ModeAmplitudes_square(:,1), ModeAmplitudes_square(:,1)/(pi*4)^2/Ac(4,1),'c-','LineWidth',1)
    xlabel('Contour Length^{2} (um^{2})','FontSize',fs,'FontName','Arial')
    ylabel('\it  A\rm_n^{2}','FontSize',fs,'FontName','Arial')
    title('Mode \it A\rm_n^{2}','FontSize',fs,'FontName','Arial')
    legend('Mode 1','Mode 2','Mode 3','Mode 4')
    set(legend,'Location','NorthEastOutside')
    set(gca,'FontSize',fs,'FontName','Arial','LineWidth',2)
    %end of Fit data to contour length dependent Fourier coefficients function (Howard book).
    uicontrol(f_Lp, 'Style','text','Position',[20 20 400 60],'FontSize',12,...
        'String',['The Fourier coefficient Data fit results: Lp=', num2str(Lp_mean), ' (+-',num2str(Lp_ste), ')'],...
        'BackgroundColor',[0.8 0.8 0.8]);
   
else %only 2 filement lengths or less or the difference between the longest and shortest filaments is less than 2 pixels, so make only 2 points
    if filament_n > 2 %more than 2 filament lengths, but length difference is less than 2 pixels
        endtoend_lengths_avg = zeros(2, 2);
        ModeAmplitudes_avg = zeros(2, 11);
        endtoend_lengths_avg(1, :) = [mean(ModeAmplitudes(ModeAmplitudes(:,1)<=(ModeAmplitudes(1,1)+um_pix), 2)),  ...
            mean(ModeAmplitudes(ModeAmplitudes(:,1)<=(ModeAmplitudes(1,1)+um_pix), 3))]; %col1 average of end-to-end lenght, col2: average end-to-end^2, 2 average lengths
        endtoend_lengths_avg(2, :) = [mean(ModeAmplitudes(ModeAmplitudes(:,1)> (ModeAmplitudes(1,1)+um_pix), 2)), ...
            mean(ModeAmplitudes(ModeAmplitudes(:,1)>(ModeAmplitudes(1,1)+um_pix), 3))]; %col1 average of end-to-end lenght, col2: average end-to-end^2
        ModeAmplitudes_avg(1, 2:11) = mean(ModeAmplitudes_square(ModeAmplitudes(:,1)<=(ModeAmplitudes(1,1)+um_pix), 2:11)); %average mode amplitude^2
        ModeAmplitudes_avg(2, 2:11) = mean(ModeAmplitudes_square(ModeAmplitudes(:,1)>(ModeAmplitudes(1,1)+um_pix), 2:11));
        if endtoend_lengths_avg(2, 1) == 0.0
            endtoend_lengths_avg(2, :) = []; %The second average filament length is 0, There is no filament longer than 1 pixel length, so delete the second average filament data
            ModeAmplitudes_avg(2, :) = [];
        end
    else %only less than 2 filament lenghts, the average  are themselves
        endtoend_lengths_avg(:, 1) = ModeAmplitudes(:, 2);
        endtoend_lengths_avg(:, 2) = ModeAmplitudes(:, 3);
        ModeAmplitudes_avg(:, 2:11) = ModeAmplitudes_square(:, 2:11);
    end
    ModeAmplitudes_avg(:, 1) = endtoend_lengths_avg(:, 1).^2;   
    lengths_n = size(endtoend_lengths_avg, 1);
    % Calculate the persitence length from end to end lengths    
    uicontrol(f_Lp, 'Style','text','Position',[20 300 400 80],'FontSize',12,...
              'String','Method 2: calculating Lp as a root from solving implicit 2D Lp equation endtoend^2=8*Lp^2*(exp(-x/(2*Lp))-1+x/(2*Lp))',...
              'BackgroundColor',[0.8 0.8 0.8]);
    l_Lp = zeros(lengths_n);
    for ii = 1:lengths_n
         l_Lp(ii) = Lp_cal(endtoend_lengths_avg(ii,1), endtoend_lengths_avg(ii,2)); % Call Lp_cal function to solve Lp as root.
    end
    Lp = mean(l_Lp);
    Lp_err = std(l_Lp)/lengths_n^0.5;
    uicontrol(f_Lp, 'Style','text','Position',[20 210 400 70],'FontSize',12,...
              'String',['The calculated mean Lp = ', num2str(Lp),' and error of the mean = ',  num2str(Lp_err), ' micrometers'],...
              'BackgroundColor',[0.8 0.8 0.8]);
          
    Summary_file = fopen(Summary_filename, 'a'); %save the result
    fprintf(Summary_file, '%s\n', 'Method 2: calculating Lp as a root from solving implicit 2D Lp equation endtoend^2=8*Lp^2*(exp(-x/(2*Lp))-1+x/(2*Lp)) 2D Lp equation');
    fprintf(Summary_file, '%s\n\n', ['The calculated Lp = ', num2str(Lp),' and error of the mean = ', num2str(Lp_err),' micrometers']);
    fclose(Summary_file);
    %end of Calculate the persitence length from end to end lengths
    
    % Fit data to contour length dependent Fourier coefficients function (Howard book).
    uicontrol(f_Lp, 'Style','text','Position',[20 100 400 90],'FontSize',12,...
        'String','Method 3: Linear fit Fourier coefficients to an^2-a0^2=(L^2/(Lp*pi^2))*(1/n^2) as an^2 vs 1/n^2 for each filament to calculate Lp and then calculate the average Lp over all filaments.',...
        'BackgroundColor',[0.8 0.8 0.8]);    
    X=(1.0./(1:10).^2)';
    s_mode = fitoptions('Method','LinearLeastSquares', 'Lower',0.0,'Upper',Inf);
    f_mode = fittype({'x'}, 'coefficients', {'c'}, 'options', s_mode); % mode number n as x
    sum_weighted_Lp = 0.0;
    sum_weight = 0.0;
    plot_symbol = ['o', 's'];
    plot_symbo_color = ['k', 'b'];
    plot_line_color = ['r', 'g'];
    figure(), hond on %create a  figure for plot    
    for k=1:lengths_n
        fit_obj = fit(X, (ModeAmplitudes_avg(k,2:11))', f_mode);
        Ac_temp = coeffvalues(fit_obj);
        Aconf_temp =confint(fit_obj);
        plot(X,ModeAmplitudes_avg(k,2:11)', plot_symbo_color(k), plot_symbol(k), X, X*Ac_temp(1, 1), plot_line_color(k), '-',...
             X, X*Aconf_temp(1)+Aconf_temp(1, 2), plot_line_color(k), '--', X, X*Aconf_temp(2, 1), plot_line_color(k), '--','LineWidth',2)
        if Ac_temp(1, 1)~=0.0
            if (Aconf_temp(1, 1)==0.0 && Aconf_temp(2, 1)==0.0 ) || isnan(Aconf_temp(1,1)) || isnan(Aconf_temp(2, 1))
                weight = 1.0; %the weight is 1
            else
                weight = (Ac_temp(1, 1)*pi)^4/(ModeAmplitudes_avg(k,1)*(Aconf_temp(1, 1)-Aconf_temp(2, 1)))^2; % weight=1/deltaLp^2, Lp=(L/Pi)^2/Ac_temp, deltaLP=-(L/Pi)^2*delta(Ac-temp)/(Ac_temp)^2
            end
            Lp = (ModeAmplitudes_avg(k,1)/pi^2)/Ac_temp(1, 1);
            sum_weighted_Lp = sum_weighted_Lp+weight*Lp;
            sum_weight = sum_weight+weight;
        end
    end
    Lp_mean=sum_weighted_Lp/sum_weight; % Average Lp is a weighted average=sum(Lp*weight)/sum(weight)
    Lp_ste=1.0/sqrt(sum_weight);  % Average Lp standrd deviation is 1/sum(weight)^(1/2)
    
    xlabel('Mode Number 1/\it n^{2}','FontSize',fs,'FontName','Arial')
    ylabel('\it A\rm_n^{2}','FontSize',fs,'FontName','Arial')
    title('Mode \it A\rm_n^{2}','FontSize',fs,'FontName','Arial')
    set(gca,'FontSize',fs,'FontName','Arial','LineWidth',2)    
    hold off       
    %end of Fit data to contour length dependent Fourier coefficients function (Howard book).
end

Summary_file = fopen(Summary_filename, 'a'); %save the result
fprintf(Summary_file, '%s\n', 'Method 3: Calculating linear fit of an^2=(1/Lp)*((L/n/pi)^2) to Fourier Coefficient Data with Mode Amplitue^2 vs Contour Length^2, assuming filament at rest is a streight line,i.e. an0=0 for n>=1.');
fprintf(Summary_file, '%s\n\n', ['The Fourier coefficient Data fit results: Lp=', num2str(Lp_mean), ' (+-',num2str(Lp_ste), ')']);
fclose(Summary_file);

process_timer = toc;
disp(['time used for calculating cosine correlation and fits ', num2str(process_timer), ' second.'])
      
figure('units','pixels',...
       'position',[500 200 440 180],...
       'menubar','none',...
       'name','Look here!',...
       'numbertitle','off',...
       'resize','off');
uicontrol('Style','text','Position',[20 65 400 80],'FontSize',12,...
          'String',[num2str(filament_n), ' filaments have been analyzed. Please consult file_readme.txt for files exported from the ananlysis. Goodbye.'],...
          'BackgroundColor',[0.8 0.8 0.8]);
uicontrol('Position',[120 20 200 40],'String','OK',...
          'Callback','uiresume(gcbf)');
uiwait(gcf);
close;
clear
end
% The end of the main function

%% Subfunction param_setting
% %Create fields for variable and option entry. Take a default set of
% setting values as input and then return user imputed setting values
function setting_value=param_setting(iniatial_setting_value)
%setting
scrsz = get(0,'ScreenSize');
S.fh = figure('units','pixels',...
              'position',[(scrsz(3)-1010)/2. (scrsz(4)-320)/2. 1010 320],...
              'menubar','none',...
              'name','Bending Persistence Length Measurement. PersistenceVersion_5_Nov19',...
              'numbertitle','off',...
              'resize','off'); 
          
S.corr = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'ForegroundColor',[1 0.5 0.5],...
                 'string','@Copyright under De La Cruz lab at Yale University. It is free to use and distribute. Please reference the software publication: Plos one, 9 (4), e94766 (2014)',...
                 'Visible', 'on',...
                 'position',[10 295 990 20]);
             
S.tx(1) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackgroundColor',[0.8 0.8 0.8],...
                 'string','Minimum number of pixels in a filament. Must be >= 4.',...
                 'position',[10 250 200 40]);
S.tx(2) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackgroundColor',[0.8 0.8 0.8],...
                 'position',[10 190 200 30],...
                 'string','Size of a pixel (microns).');
S.tx(3) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackgroundColor',[0.8 0.8 0.8],...
                 'position',[10 130 200 40],...
                 'string','Maximum number of filaments to analyze.');
S.tx(4) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackgroundColor',[0.8 0.8 0.8],...
                 'position',[10 80 200 30],...
                 'string','Start frame of image stack.');
S.tx(5) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackgroundColor',[0.8 0.8 0.8],...
                 'position',[10 10 200 50],...
                 'string','Allowed branch length. Value of 0 indicates branches are not allowed');
             
S.ed(1) = uicontrol('style','edit',...
                 'unit','pix',...
                 'position',[220 255 180 30],...
                 'fontsize',14,...
                 'string',num2str(iniatial_setting_value(1)));
S.ed(2) = uicontrol('style','edit',...
                 'unit','pix',...
                 'position',[220 190 180 30],...
                 'fontsize',14,...
                 'string',num2str(iniatial_setting_value(2)));
S.ed(3) = uicontrol('style','edit',...
                 'unit','pix',...
                 'position',[220 135 180 30],...
                 'fontsize',14,...
                 'string',num2str(iniatial_setting_value(3)));
S.ed(4) = uicontrol('style','edit',...
                 'unit','pix',...
                 'position',[220 80 180 30],...
                 'fontsize',14,...
                 'string',num2str(iniatial_setting_value(4)));
S.ed(5) = uicontrol('style','edit',...
                 'unit','pix',...
                 'position',[220 20 180 30],...
                 'fontsize',14,...
                 'string',num2str(iniatial_setting_value(5)));
          
S.pp(1) = uicontrol('style','pop',...
                 'units','pix',...
                 'position',[640 255 60 30],...
                 'FontSize',12,...
                 'String',{'Yes'  'NO'});
             set(S.pp(1),'Value',iniatial_setting_value(6));
S.tx(6) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackGroundColor',[0.8 0.8 0.8],...
                 'string','Calculate the average and fit to data?',...
                 'position',[430 250 200 40]);

S.pp(2) = uicontrol('style','pop',...
                 'units','pix',...
                 'position',[640 200 60 30],...
                 'FontSize',12,...
                 'String',{'Yes' 'NO'});
             set(S.pp(2),'Value',iniatial_setting_value(7));
S.tx(7) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackGroundColor',[0.8 0.8 0.8],...
                 'string','Analyze shapes you have already selected?',...
                 'position',[430 195 200 40]);
             
S.pp(3) = uicontrol('style','pop',...
                 'units','pix',...
                 'position',[640 140 60 30],...
                 'FontSize',12,...
                 'String',{'Yes' 'NO'});
             set(S.pp(3),'Value',iniatial_setting_value(8)); %manually construct by two points
S.tx(8) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackGroundColor',[0.8 0.8 0.8],...
                 'string','want to manually constuct filament between two slected points?',...
                 'position',[430 130 200 50]);  
             
S.pp(4) = uicontrol('style','pop',...
                 'units','pix',...
                 'position',[640 75 60 30],...
                 'FontSize',12,...
                 'String',{'Yes' 'NO'});
             set(S.pp(4),'Value',iniatial_setting_value(9));
S.tx(9) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackGroundColor',[0.8 0.8 0.8],...
                 'string','want to manually constuct filament by stretching from one selected point?',...
                 'position',[430 65 200 50]);  
                                                  
S.pp(5) = uicontrol('style','pop',...
                 'units','pix',...
                 'position',[640 15 60 30],...
                 'FontSize',12,...
                 'String',{'Yes' 'NO'});
             set(S.pp(5),'Value',iniatial_setting_value(10));
S.tx(10) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackGroundColor',[0.8 0.8 0.8],...
                 'string','Want to allow a single pixel gap in filament reconstruction?',...
                 'position',[430 10 200 40]);              
             
S.pp(6) = uicontrol('style','pop',...
                 'units','pix',...
                 'position',[940 240 60 30],...
                 'FontSize',12,...
                 'String',{'Yes' 'NO'});
             set(S.pp(6),'Value',iniatial_setting_value(11));
S.tx(11) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackGroundColor',[0.8 0.8 0.8],...
                 'string','Want to select the ends for filament shape reconstructions? Recommended if a subset of filaments needs to be analyzed.',...
                 'position',[730 210 200 80]); 
             
S.pp(7) = uicontrol('style','pop',...
                 'units','pix',...
                 'position',[940 150 60 30],...
                 'FontSize',12,...
                 'String',{'Yes' 'NO'});
             set(S.pp(7),'Value',iniatial_setting_value(12));
S.tx(12) = uicontrol('style','text',...
                 'FontSize',10,...
                 'FontWeight','bold',...
                 'BackGroundColor',[0.8 0.8 0.8],...
                 'string','Want to confirm filament shape reconstructions? Recommended if data is noisy or crowded.',...
                 'position',[730 120 200 80]); 

S.pb(1) = uicontrol('style','push',...
                 'units','pix',...
                 'position',[770 70 180 40],...
                 'fontsize',14,...
                 'string','Set Values',...
                 'callback',{@ed_call1,S});
S.pb(2) = uicontrol('style','push',...
                 'units','pix',...
                 'position',[770 20 180 40],...
                 'fontsize',14,...
                 'string','Start',...
                 'Callback','uiresume(gcbf)');
             
uiwait(S.fh);
while strcmp(get(S.corr, 'Visible'),'on')
    set(S.corr, 'string', 'Please set parameters or correct invalid parameters before you click "Start"');
    uiwait(S.fh);
end
pp_setting_value = [get(S.pp(1), 'Value'), get(S.pp(2), 'Value'), get(S.pp(3), 'Value'), get(S.pp(4), 'Value'), get(S.pp(5), 'Value'), get(S.pp(6), 'Value'), get(S.pp(7), 'Value')];
setting_value =[str2double(get(S.ed(1:5), 'String')'), pp_setting_value]; 
end
% The end of function
%% Subfunction ed_call1
function []=ed_call1(varargin)
% Callback for pushbutton "Set Value".
S = varargin{3};  % Get the structure.
%Get variable values.
addstr1 = {get(S.ed(1),'string')};
addstr2 = {get(S.ed(2),'string')};
addstr3 = {get(S.ed(3),'string')};
addstr4 = {get(S.ed(4),'string')};
addstr5 = {get(S.ed(5),'string')};

set(S.corr, 'string','Please correct highlighted invalid parameters'); % Reset warning if it excists.
set(S.corr, 'Visible', 'off');
S.set_corr=1; % Defualt all values are good
% Select the minimum pixel length for filaments to analyze |pix_threshold|>4.
if isnan(str2double(addstr1)) 
    set(S.ed(1),'backgroundcolor',[1 .5 .5],'string','Invalid Value!')
    set(S.corr, 'Visible', 'on');
elseif str2double(addstr1)<4
    set(S.ed(1),'backgroundcolor',[1 .5 .5],'string','Must be >=4!')
    set(S.corr, 'Visible', 'on');
else
    set(S.ed(1),'backgroundcolor',[.2 1 0.2])
end

% Pixel size conversion from calibration |um_pix| in microns per pixel.
if isnan(str2double(addstr2))
    set(S.ed(2),'backgroundcolor',[1 .5 .5],'string','Invalid Value!')
    set(S.corr, 'Visible', 'on');
elseif str2double(addstr2)==0
    set(S.ed(2),'backgroundcolor',[1 .5 .5],'string','Must be >=0!')
    set(S.corr, 'Visible', 'on');
else
    set(S.ed(2),'backgroundcolor',[.2 1 0.2])
end

% Limit for total number of filament shapes to be analyzed |filamentN|.
if isnan(str2double(addstr3))
    set(S.ed(3),'backgroundcolor',[1 .5 .5],'string','Invalid Value!')
    set(S.corr, 'Visible', 'on');
elseif str2double(addstr3)<1
    set(S.ed(3),'backgroundcolor',[1 .5 .5],'string','Must be >=1!')
    set(S.corr, 'Visible', 'on');
else
    set(S.ed(3),'backgroundcolor',[.2 1 0.2])
end

%Select initial frame number.
if isnan(str2double(addstr4)) || str2double(addstr4) <= 0
    set(S.ed(4),'backgroundcolor',[1 .5 .5],'string','Invalid Value!')
    set(S.corr, 'Visible', 'on');
elseif str2double(addstr4)<1
    set(S.ed(4),'backgroundcolor',[1 .5 .5],'string','Must be >=1!')
    set(S.corr, 'Visible', 'on');
else
    set(S.ed(4),'backgroundcolor',[.2 1 0.2])
end

%Select branch size limit
if isnan(str2double(addstr5))
    set(S.ed(5),'backgroundcolor',[1 .5 .5],'string','Invalid Value!')
    set(S.corr, 'Visible', 'on');
elseif str2double(addstr5)<0
    set(S.ed(5),'backgroundcolor',[1 .5 .5],'string','Must be >=0!')
    set(S.corr, 'Visible', 'on');
else
    set(S.ed(5),'backgroundcolor',[.2 1 0.2])
end
end
% The end of function ed_call1

%% Subfunction Position
% This filament line tracking algorithm bridges single pixel gaps as wish
% and allows indicated length of artificial branches developed by the
% skeletonization.  It takes input from a skeletonized binary image |img|,
% a starting filament and |endn|, |em| that is the 3 columns matrices of
% end coodinations from the iamge |img| through the function |find_end|,
% |single_gap|='y' or 'n' to include or not to include single pix gap, and
% |brch_size|, a integer number to indecate allowed branch length (in
% number of pixels). Return position map of the tracked filamen |pm|, the
% filament ending point index |endn_new| and updated end coordinate map
% em_new. |endn_new|==endn-1 indicates the filament is branched and the
% starting end has been deleted, pm=[].
% Usage: [pm,endn_new, em_new]=position(img,endn,em)
% Copyright originally 2012 Brannon R McCullough, modified 2014 Wenxiang
% Cao
function [pm,endn_new, em_new]=position(img, endn,em)
global brch_size single_pixel_gap_reply
img=logical(img);% Scale image to binary |img|.
%(['Single gap is:', single_gap])

% Set first filament point coordinates as endn.
i=em(endn,2); % |i| is the i coordinate.
j=em(endn,3); % |j| is the j coordinate.
% If endn==1, search the duplicates in |em| from endn+1 to end and clean them
%disp(['Single gap:',single_gap])
if endn==1
    n=1;
    while n<=size(em, 1)
        k=n+1;
        while k<=size(em,1)
            if (em(k,2)==em(n,2))&&(em(k,3)==em(n,3)) % Found.
                em(k, :)=[]; % This is a repeat. Delete repeat.
            else
                k=k+1;
            end            
        end
        n=n+1;
    end
end
em(:, 1)=1:size(em,1); % Fix index in the first column due to deletion. The first column storing index is a real pain in ass and unnecessary. It is there because it was there and I don't want go through thoubles to change it.
pre_coord=[i,j]; % |pre_coord| is the coordinates of the previous point from [i,j]. If [i, j] is a starting end point, pre_coord==[i,j].
pre_pre_coord=[i,j]; % |pre_Pre_coord| is the coordinates of the previous point from |pre_coord|. If [i, j] is a starting end point, pre_pre_coord==[i,j].
exclude_pnt=[i, j]; % exclude_pnt is the coordinates of position needed to be excluded from filament searching. If [i, j] is a starting end point, exclude_pnt=[i, j].
n=0; % Found data ponit counter.
pm=zeros(500,3); % Position map of filament is preallocated 500 data points.|pm| = [nth point, |i|, |j|], |pm| is a position map for found filament point coordinates.
pm(1:end,1)=1:size(pm,1);
stop_search=0; % Signal when to stop searching.
filament_connector=[0,0,0]; % Store filament end information that will be deleted from end map |em| because it connects other filaments with one pixel gap.
while stop_search==0 % Search the nearest neighbors first for the filament pixels. 
    [neighbor_coor,pm_temp] = branch_trk(img,[i,j], pre_coord, pre_pre_coord, exclude_pnt); % Call function branch_trk(img,starting, old) to search nearest neighbor connected filatment.    
    if (size(pm,1)-n)<=size(pm_temp,1)
        pm=[pm; zeros(n+size(pm_temp,1),3)]; % If the seize of pm is not enough, double the size.
    end
    pm((n+1):(n+size(pm_temp,1)), :)=[((n+1):(n+size(pm_temp,1)))', pm_temp(:,:)]; % Store the found filament.
    n=n+size(pm_temp,1);
    end_stat=size(neighbor_coor,1); % Number of nearest neighbors at the filament ending point.
    Three_conf=0; % Initialize the flag variable for four special three neighbor point configurations.
    if end_stat==3 % When there are three neighbor points, there are foure special cases. Use a variable Three_conf=1, 2, 3, 4 to present the four case. Three_conf=0 means all other except the four special configurations.
        if ((neighbor_coor(1,1)==neighbor_coor(2,1))&&(neighbor_coor(2,1)==neighbor_coor(3,1))) || ((neighbor_coor(1,2)==neighbor_coor(2,2))&&(neighbor_coor(2,2)==neighbor_coor(3,2))) 
            Three_conf=1;  % The three neighbor points are in the same line.
        elseif ((neighbor_coor(1,1)-neighbor_coor(2,1))^2+(neighbor_coor(1,2)-neighbor_coor(2,2))^2<2)&&((neighbor_coor(3,1)-neighbor_coor(1,1))^2+(neighbor_coor(3,2)-neighbor_coor(1,2))^2>=2)&&((neighbor_coor(3,1)-neighbor_coor(2,1))^2+(neighbor_coor(3,2)-neighbor_coor(2,2))^2>=2)
            Three_conf=2; % Neighbor points 1 and 2 are adjacent to each other in the same line, but the two are not adjacent in the same line with  point 3.
        elseif ((neighbor_coor(2,1)-neighbor_coor(3,1))^2+(neighbor_coor(2,2)-neighbor_coor(3,2))^2<2)&&((neighbor_coor(1,1)-neighbor_coor(2,1))^2+(neighbor_coor(1,2)-neighbor_coor(2,2))^2>=2)&&((neighbor_coor(1,1)-neighbor_coor(3,1))^2+(neighbor_coor(1,2)-neighbor_coor(3,2))^2>=2)
            Three_conf=3; % Neighbor points 2 and 3 are adjacent to each other in the same line, but the two are not adjacent in the same line with  point 1.
        elseif ((neighbor_coor(1,1)-neighbor_coor(3,1))^2+(neighbor_coor(1,2)-neighbor_coor(3,2))^2<2)&&((neighbor_coor(2,1)-neighbor_coor(1,1))^2+(neighbor_coor(2,2)-neighbor_coor(1,2))^2>=2)&&((neighbor_coor(2,1)-neighbor_coor(3,1))^2+(neighbor_coor(2,2)-neighbor_coor(3,2))^2>=2) 
            Three_conf=4; % Neighbor points 1 and 3 are adjacent to each other in the same line, but the two are not adjacent in the same line with  point 2.
        end
    end
    % Check the new found neighbors.
    if  end_stat==0 % the nth point has no new neareat neighbor pixel. Therefore it is an end point and the last point of nearest neighbor connectd filament. Check if it connects to other unsearched ends (endn+2, endn=3, ..).
        if size(em, 1)>endn
            for k=endn+1:size(em, 1)
                if (em(k,2)==pm(n,2))&&(em(k,3)==pm(n,3)) % Found.
                    if k>endn+1
                        em(k,2:3)=em(endn+1,2:3); % Found pm(n,2:3) in end map at em(k,2:3), k>endn+1. Switch em(k,2:3) with em(endn+1,2:3).
                        em(endn+1,2:3)=pm(n,2:3);
                    end
                    break;                
                end
            end
        end
        if (size(em, 1)<=endn) || (em(endn+1,2)~=pm(n,2))||(em(endn+1,3)~=pm(n,3)) % Did not find pm(n,2:3) in end map em(endn+1:end). Insert pm(n,2:3) into em as endn+1 point.
            emt=zeros(size(em,1)+1,3);
            emt(1:(end-1),:)=em;
            if size(emt, 1)>endn+1 % Insert branch 1 end in endn+1 position, but keep all elements after endn+2 at their position.Therefore, exchange position of endn+1 with last one.
                emt(end, :)=[size(emt, 1), emt(endn+1, 2:3)];
                emt(endn+1,2:3)=pm(n,2:3);
            elseif size(emt, 1)==endn+1
                emt(end, :)=[endn+1, pm(n,2:3)];
            end
            em=emt;
        end
        % Now the initial starting end is at endn position, while
        % ending end of the filament is at endn+1 position.
        if (strcmp(single_pixel_gap_reply, 'Yes')) && ((endn+1)<size(em,1)) % The guy wants a single pixel gap to connec two filament at their ends. Therefore from both filament ends, search other ends within two pixels away.
            end_to_end=[0,0]; % |end_to_end| counts number of ends in |em| that are the second nearest neighbor (two points away) to em(endn) (first column) or em(endn+1) (second column) except endn and endn+1 themselves.
            connect_endn_coordinate=zeros(size(em,1)-2,3); % Store the coordinates of the second nearest neighbors to endn. The first column is the index in end map |em|.
            connect_endn_plus_coordinate=zeros(size(em,1)-2,3); % Store the coordinates of the second nearest neighbors to endn+1. The first column is the index in end map |em|.
            for k=1:size(em,1)
                if (k~=endn)&&(k~=endn+1)
                    if ((em(endn,2)==pm(1,2)) && (em(endn,3)==pm(1,3))) && ((em(k,2)-em(endn,2))^2<=4)&&((em(k,3)-em(endn,3))^2<=4) % Make sure the filament starting point is at position endn in |em|.
                        end_to_end(1,1)=end_to_end(1,1)+1;
                        connect_endn_coordinate(end_to_end(1,1), :)=em(k,:);
                    elseif ((em(k,2)-em(endn+1,2))^2<=4)&&((em(k,3)-em(endn+1,3))^2<=4)
                        end_to_end(1,2)=end_to_end(1,2)+1;
                        connect_endn_plus_coordinate(end_to_end(1,2), :)=em(k,:);
                    end
                end
            end
            if (end_to_end(1,1)==1) && (connect_endn_coordinate(1,1)>endn+1) % The endn-th end has a second nearest neighbor that has not processed for search filament yet.
                pmt=pm(1:n,2:3); %Create a temp nX2 matrix to copy n points of filament.
                pmt=flipud(pmt); 
                pm(1:n, 2:3)=pmt; % Change filament search direction.
                filament_connector=pm(n,:); % Preserve the endn information for later on if it need s to delete the new filament.
                em(connect_endn_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                em(endn,:)=[]; % Since endn is not considered as an end anymore, delete it from the end map.
                em(endn:end,1)=endn:size(em,1); % Fix the first column values due to deletion.
                i=connect_endn_coordinate(1,2);
                j=connect_endn_coordinate(1,3);
                pre_coord=[i,j]; % pre_coord is the same as [i,j].
                pre_pre_coord=[i, j];
                exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
            elseif end_to_end(1,2)==1 && (connect_endn_plus_coordinate(1,1)>endn+1) % The endn+1-th end has a second nearest neighbor that has not processed for search filament yet.
                filament_connector=pm(n,:); % Preserve the endn+1 information for later on if it need s to delete the new filament.
                em(connect_endn_plus_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                em(endn+1,:)=[]; % Since endn+1 is not considered as an end anymore, delete it from the end map.
                em(endn+1:end,1)=(endn+1):size(em,1); % Fix the first column values due to deletion..
                i=connect_endn_plus_coordinate(1,2);
                j=connect_endn_plus_coordinate(1,3);
                pre_coord=[i,j]; % pre_coord is the same as [i,j].
                pre_pre_coord=[i,j]; % pre_pre_coord is the same as [i,j].
                exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
            else % Either both ends have no other ends two pixel away or either or both ends have more than two other ends two pixel away. Either case, stop search and reture the filament.
                em_new=em;
                endn_new=endn+1;
                pm((n+1):end,:)=[]; % Delete empty elements.
                stop_search=1;
            end
        else % The guy doesn't want one pixel gap between the ends of two filaments or end+1 is the last one in |em|. Stop searching and return the filament.
            em_new=em;
            endn_new=endn+1; % Since endn+1 is the other end of the filament, next search endn+2.
            pm((n+1):end,:)=[]; % Delete empty elements.
            stop_search=1;
        end
    elseif (end_stat==2) || (Three_conf==1)|| (Three_conf==2)|| (Three_conf==3)|| (Three_conf==4) % the nth point connects to two neighbor points.
        if (end_stat==2) && ((neighbor_coor(1,1)-neighbor_coor(2,1))^2+(neighbor_coor(1,2)-neighbor_coor(2,2))^2<2) % The two neighbor are in the same horizontal or vertical line.
            if (pm(n, 2)==neighbor_coor(1,1)) || (pm(n, 3)==neighbor_coor(1,2)) %The first neighbor point and the last filament point are in the same line.
                i=neighbor_coor(1,1); % The first neighbor point becomes the next searching start point.
                j=neighbor_coor(1,2);
            else  %The second neighbor point and the last filament point are in the same line.
                i=neighbor_coor(2,1); % The second neighbor point becomes the next searching start point.
                j=neighbor_coor(2,2);
            end
            if n==1
                pre_pre_coord=pre_coord; % pre_coord becomes pre_pre_coord.
            else
                pre_pre_coord=pm(n-1, 2:3); %The second last filament point becomes pre_pre_coord.
            end
            pre_coord=pm(n, 2:3); % The last filament point becomes pre_coord
            exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
        elseif Three_conf==1 % The three neighbor points are in the same line.
            if (pm(n, 2)==neighbor_coor(1,1)) || (pm(n, 3)==neighbor_coor(1,2)) %The first neighbor point and the last filament point are in the same line.
                i=neighbor_coor(1,1); % The first neighbor point becomes the next searching start point.
                j=neighbor_coor(1,2);
            elseif (pm(n, 2)==neighbor_coor(2,1)) || (pm(n, 3)==neighbor_coor(2,2)) %The second neighbor point and the last filament point are in the same line.
                i=neighbor_coor(2,1); % The second neighbor point becomes the next searching start point.
                j=neighbor_coor(2,2);
            else % The third neighbor point and the last filament point are in the same line.
                i=neighbor_coor(3,1); % The third neighbor point becomes the next searching start point.
                j=neighbor_coor(3,2);
            end
            if n==1
                pre_pre_coord=pre_coord; % pre_coord becomes pre_pre_coord.
            else
                pre_pre_coord=pm(n-1, 2:3); %The second last filament point becomes pre_pre_coord.
            end
            pre_coord=pm(n, 2:3); % The last filament point becomes pre_coord
            exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
        elseif (end_stat==2) || (Three_conf==2)|| (Three_conf==3)|| (Three_conf==4)
            if end_stat==2 % The two neighbor points are not in the same line. The last filament point cannot be filament end point, i.e. n >1.
                [brch_end_neighbor1,branch_mp1] = branch_trk(img, neighbor_coor(1,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(2,:)); % Using the first neighbor point to track the branch and exclude the second neighbor point from the traching.
                [brch_end_neighbor2,branch_mp2] = branch_trk(img, neighbor_coor(2,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(1,:)); % Using the second neighbor point to track the branch and exclude the first neighbor point from the traching.
            elseif Three_conf==2 % The first two of the three neighbor points are adjacent to each other in the same line, but they are not adjacent in the same line with neighbor pint 3. 
                if (pm(n, 2)==neighbor_coor(1,1)) || (pm(n, 3)==neighbor_coor(1,2)) % The first neighbor point is at the same line with the n point and the second neighbor point must be diagonal to the n point.
                    [brch_end_neighbor1,branch_mp1] = branch_trk(img, neighbor_coor(1,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(3,:)); % Using the first neighbor point to track the branch and exclude the third neighbor point from the traching.
                    [brch_end_neighbor2,branch_mp2] = branch_trk(img, neighbor_coor(3,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(1,:)); % Using the third neighbor point to track the branch and exclude the first neighbor point from the traching.
                else % The neighbor point 2 is at the same line with the n point and the neighbor point 1 must be diagonal to the n point.
                    [brch_end_neighbor1,branch_mp1] = branch_trk(img, neighbor_coor(2,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(3,:)); % Using the neighbor point 2 to track the branch and exclude the neighbor point 3 from the traching.
                    [brch_end_neighbor2,branch_mp2] = branch_trk(img, neighbor_coor(3,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(2,:)); % Using the neighbor point 3 to track the branch and exclude the neighbor point 2 from the traching.
                end
            elseif Three_conf==3 % The last two of the three neighbor points are adjacent to each other in the same line, but they are not adjacent in the same line with neighbor pint 1. 
                if (pm(n, 2)==neighbor_coor(2,1)) || (pm(n, 3)==neighbor_coor(2,2)) % The neighbor point 2 is at the same line with the n point and the neighbor point 3 must be diagonal to the n point.
                    [brch_end_neighbor1,branch_mp1] = branch_trk(img, neighbor_coor(1,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(2,:)); % Using the neighbor point 1 to track the branch and exclude the neighbor point 2 from the traching.
                    [brch_end_neighbor2,branch_mp2] = branch_trk(img, neighbor_coor(2,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(1,:)); % Using the neighbor point 2 to track the branch and exclude the neighbor point 1 from the traching.
                else % The neighbor point 3 is at the same line with the n point and the neighbor point 2 must be diagonal to the n point.
                    [brch_end_neighbor1,branch_mp1] = branch_trk(img, neighbor_coor(1,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(3,:)); % Using the neighbor point 1 to track the branch and exclude the neighbor point 3 from the traching.
                    [brch_end_neighbor2,branch_mp2] = branch_trk(img, neighbor_coor(3,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(1,:)); % Using the neighbor point 3 to track the branch and exclude the neighbor point 1 from the traching.
                end
            elseif Three_conf==4 % The neighbor point 1 and 3 are adjacent to each other in the same line, but they are not adjacent in the same line with neighbor pint 2. 
                if (pm(n, 2)==neighbor_coor(1,1)) || (pm(n, 3)==neighbor_coor(1,2)) % The neighbor point 1 is at the same line with the n point and the neighbor point 3 must be diagonal to the n point.
                    [brch_end_neighbor1,branch_mp1] = branch_trk(img, neighbor_coor(1,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(2,:)); % Using the neighbor point 1 to track the branch and exclude the neighbor point 2 from the traching.
                    [brch_end_neighbor2,branch_mp2] = branch_trk(img, neighbor_coor(2,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(1,:)); % Using the neighbor point 2 to track the branch and exclude the neighbor point 1 from the traching.
                else % The neighbor point 3 is at the same line with the n point and the neighbor point 1 must be diagonal to the n point.
                    [brch_end_neighbor1,branch_mp1] = branch_trk(img, neighbor_coor(2,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(3,:)); % Using the neighbor point 2 to track the branch and exclude the neighbor point 3 from the traching.
                    [brch_end_neighbor2,branch_mp2] = branch_trk(img, neighbor_coor(3,:), pm(n, 2:3), pm(n-1, 2:3), neighbor_coor(2,:)); % Using the neighbor point 3 to track the branch and exclude the neighbor point 2 from the traching.
                end
            end
            if (size(brch_end_neighbor1,1)==0) && (size(brch_end_neighbor2,1)==0) && (size(branch_mp1, 1)<=brch_size) && (size(branch_mp2, 1)<=brch_size) % Both branches end at a non-branching point and both are shorter than brch_size and will be ignored.
                % Find the two branch ends in the end map |em| and store branch 1 end to endn+1 and branch 2 end to endn+2.
                found=0; % Scaler found==0, not found the two neighbor points in end map yet; found==1 or 2 found the neighbor point 1 or 2 in the end map; found==3 found both neighbor points in end map.
                if size(em,1)>endn                    
                    for k=endn+1:size(em, 1)
                        if (found~=1)&&((em(k,2)==branch_mp1(end,1))&&(em(k,3)==branch_mp1(end,2))) % Not Found before and put the found end to endn+1 or endn+2 position.
                            if (found==0) % Not one found before and put at endn+1
                                found=1;
                                if k~=endn+1
                                    em(k,2:3)=em(endn+1,2:3); % If k is not at endn+1 position, exchange endn+1 and k
                                    em(endn+1,2:3)=branch_mp1(end,:);
                                end
                            elseif found==2 % Found neighbor point 2 before and it is already at endn+1, put branch1 end at endn+2
                                found=3; % found==3 means branch2 end is at endn+1 while branch1 end is at endn+2.
                                if k~=endn+2
                                    em(k,2:3)=em(endn+2,2:3); % If k is not at endn+2 position, exchange endn+2 and k
                                    em(endn+2,2:3)=branch_mp1(end,:);
                                end
                                break;
                            end
                        elseif (found~=2)&&(em(k,2)==branch_mp2(end,1))&&(em(k,3)==branch_mp2(end,2)) % Not Found before and put the found end to endn+1 or endn+2 position.
                            if found==0 % Not one found before and put at endn+1
                                found=2;
                                if k~=endn+1
                                    em(k,2:3)=em(endn+1,2:3); % If k is not at endn+1 position, exchange endn+1 and k
                                    em(endn+1,2:3)=branch_mp2(end,:);
                                end
                            elseif found==1 % Found neighbor point 1 before and it is already in endn+1 position. Put at endn+2
                                if k~=endn+2
                                    em(k,2:3)=em(endn+2,2:3); % If k is not at endn+2 position, exchange endn+2 and k
                                    em(endn+2,2:3)=branch_mp2(end,:);
                                end
                                found=4; % found==4 means branch1 end is at endn+1 while branch2 end is at endn+2.
                                break;
                            end
                        end
                    end
                end
                if found==2 % Found branch 2 end and it is at endn+1, but did not find branch 1 end in end map |em|. Insert it at endn+2 in |em|.
                    emt=zeros(size(em,1)+1, 3);
                    emt(1:(end-1), :)=em;
                    if size(emt, 1)>endn+2 % Insert branch 1 end in endn+2 position, but keep all elements after endn+3 at their position.Therefore, exchange position of endn+1 with last one.
                        emt(end, :)=[size(emt,1),emt(endn+2, 2:3)];
                        emt(endn+2, 2:3)=branch_mp1(end,:);
                    elseif size(emt, 1)==endn+2
                        emt(end, :)=[endn+2,branch_mp1(end,:)];
                    end
                    em=emt;
                    found=3; % found==3 means branch2 end is at endn+1 while branch1 end is at endn+2.
                elseif found==1 % Found branch 1 end and it is at endn+1, but did not find branch 2 end in end map |em|. Insert it endn+2 in |em|.
                    emt=zeros(size(em,1)+1, 3);
                    emt(1:(end-1), :)=em;
                    if size(emt, 1)>endn+2 % Insert branch 2 end in endn+2 position, but keep all elements after endn+3 at their position.Therefore, exchange position of endn+2 with last one.
                        emt(end, :)=[size(emt,1),emt(endn+2, 2:3)];
                        emt(endn+2, 2:3)=branch_mp2(end,:);
                    elseif size(emt, 1)==endn+2
                        emt(end, :)=[endn+2,branch_mp2(end,:)];
                    end
                    em=emt;
                    found=4; % found==4 means branch1 end is at endn+1 while branch2 end is at endn+2.
                elseif found==0 % Did not find both branch ends in end map. Insert both at endn+1 and endn+2 position.
                    emt=zeros(size(em,1)+2, 3);
                    emt(1:endn, :)=em(1:endn, :);
                    emt(endn+1, :)=[endn+1,branch_mp1(end,:)]; % Insert branch1 end at endn+1
                    emt(endn+2, :)=[endn+2,branch_mp2(end,:)]; % Insert branch2 end at endn+2
                    if size(emt, 1)>endn+2 % Append the rest of end in end map.
                        emt((endn+3):end, :)=[((endn+3):size(emt,1))',em((endn+1):end, 2:3)];
                    end
                    em=emt;
                    found=4; % found==4 means branch1 end is at endn+1 while branch2 end is at endn+2.
                end
                % Now the initial starting end is at endn position, while
                % the two branch ends are at endn+1 and endn+2 position respectively.
                if (strcmp(single_pixel_gap_reply, 'Yes')) && ((endn+2)<size(em,1)) % The guy wants a single pixel gap to connec two filament at their ends and there are ends in end map that haven't processed yet.
                    end_to_end=[0,0,0]; % |end_to_end| counts number of ends in |em| that are the second nearest neighbor (two points away) to em(endn) (first column) or em(endn+1) (second column) except endn and endn+1 themselves.
                    connect_endn_coordinate=zeros(size(em,1)-3,3); % Store the coordinates of the second nearest neighbors to endn. The first column is the index in end map |em|.
                    connect_endn_plus_coordinate=zeros(size(em,1)-3,3); % Store the coordinates of the second nearest neighbors to endn+1. The first column is the index in end map |em|.
                    connect_endn_plus_two_coordinate=zeros(size(em,1)-3,3); % Store the coordinates of the second nearest neighbors to endn+2. The first column is the index in end map |em|.
                    for k=1:size(em,1) % From filament starting end and both branch ends, search other ends within two pixels away.
                        if (k~=endn)&&(k~=endn+1)&&(k~=endn+2)
                            if ((em(endn,2)==pm(1,2))&&(em(endn, 3)==pm(1,3)))&&(((em(k,2)-em(endn,2))^2<=4)&&((em(k,3)-em(endn,3))^2<=4)) % Make sure the endn-th end is the starting of the filament. If not, the filament starts from two ingnored ends. Therefore don't search for the secondary nearest neighbors.
                                end_to_end(1,1)=end_to_end(1,1)+1;
                                connect_endn_coordinate(end_to_end(1,1), :)=em(k,:);
                            elseif ((em(k,2)-em(endn+1,2))^2<=4)&&((em(k,3)-em(endn+1,3))^2<=4)
                                end_to_end(1,2)=end_to_end(1,2)+1;
                                connect_endn_plus_coordinate(end_to_end(1,2), :)=em(k,:);
                            elseif ((em(k,2)-em(endn+2,2))^2<=4)&&((em(k,3)-em(endn+2,3))^2<=4)
                                end_to_end(1,3)=end_to_end(1,3)+1;
                                connect_endn_plus_two_coordinate(end_to_end(1,3), :)=em(k,:);
                            end
                        end
                    end
                    if (end_to_end(1,2)==1) && (connect_endn_plus_coordinate(1,1)>endn+2)&& (end_to_end(1,3)~=1)% The  first branch end has a second nearest neighbor that has not processed for search filament yet.  Also the second branch end has no or more than 2 secondary nearest neighbor.
                        if (n<size(branch_mp2, 1)) && (end_to_end(1,1)==0) % |pm| is shorter than branch2 and its starting point has no secondary nearest neighbor. ignor the current filament
                            pmt=[flipud(branch_mp2);pm(n,2:3)]; % Delete currnet filament store branch2
                            n=size(branch_mp2, 1)+1; % Reassign n
                            pm(1:n, :)=[(1:n)', pmt];
                            em(connect_endn_plus_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                            if found==3 % Branch2 end is at endn+1 and branch1 end is at endn+2
                                em(endn+2,:)=[]; % Since branch1 end (endn+2) is not considered as an end anymore, delete it from the end map.
                            elseif found==4 % Branch1 end is at endn+1 and branch2 end is at endn+2
                                em(endn+1,:)=[]; % Since branch1 end (endn+1) is not considered as an end anymore, delete it from the end map.
                            end
                            em(endn, :)=[]; % Since the current filament is ignored and endn is not considered as an end anymore, delete it from the end map.
                            if size(em, 1)>=endn
                                em(endn:end,1)=endn:size(em,1); % Fix the first column values due to deletion.
                            end
                            % After this branch2 end is at endn
                            % position and it is the starting of the
                            % new filament
                        else % Ignor the branch2.
                            em(connect_endn_plus_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                            em(endn+2,:)=[]; % Since the second branch is ignored and endn+2 is not considered as an end anymore, delete it from the end map.
                            em(endn+1,:)=[]; % Since endn+1 is not considered as an end anymore, delete it from the end map.
                            if size(em, 1)>=endn+1
                                em((endn+1):end,1)=(endn+1):size(em,1); % Fix the first column values due to deletion.
                            end
                        end % Current filament starting is at endn position
                        pm((n+1):(n+size(branch_mp1, 1)),:)=[((n+1):(n+size(branch_mp1, 1)))', branch_mp1(:,:)]; % Store the first branch and connect to it.
                        n=n+size(branch_mp1, 1);
                        filament_connector=pm(n,:); % Preserve the endn+1 information for later on if it need s to delete the new filament.
                        i=connect_endn_plus_coordinate(1,2); % Start searching from the new end.
                        j=connect_endn_plus_coordinate(1,3);
                        pre_coord=[i,j]; % pre_coord is the same as [i,j].
                        pre_pre_coord=[i,j]; % pre_pre_coord is the same as [i,j].
                        exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                    elseif (end_to_end(1,3)==1) && (connect_endn_plus_two_coordinate(1,1)>endn+2)&& (end_to_end(1,2)~=1)% The second branch end has a second nearest neighbor that has not processed for search filament yet.  Also the first branch end has no or more than 2 secondary nearest neighbor.
                        if (n<size(branch_mp1, 1)) && (end_to_end(1,1)==0) % |pm| is shorter than branch1 size and its starting point has no secondary nearest neighbor. ignor the current filament also
                            pmt=[flipud(branch_mp1);pm(n,2:3)]; % Delete currnet filament store branch1
                            n=size(branch_mp1, 1)+1; % Reassign n
                            pm(1:n, :)=[(1:n)', pmt];
                            em(connect_endn_plus_two_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                            if found==3 % Branch2 end is at endn+1 and branch1 end is at endn+2
                                em(endn+1,:)=[]; % Since branch2 end (endn+1) is not considered as an end anymore, delete it from the end map.
                            elseif found==4 % Branch1 end is at endn+1 and branch2 end is at endn+2
                                em(endn+2,:)=[]; % Since branch2 end (endn+2) is not considered as an end anymore, delete it from the end map.
                            end
                            em(endn, :)=[]; % Since the current filament is ignored and endn is not considered as an end anymore, delete it from the end map.
                            if size(em, 1)>=endn
                                em(endn:end,1)=endn:size(em,1); % Fix the first column values due to deletion.
                            end
                            % After this, branch1 end is at endn
                            % position and it is the starting of the
                            % new filament
                        else % Ignor the first branchs.
                            em(connect_endn_plus_two_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                            em(endn+2,:)=[]; % Since endn+2 is not considered as an end anymore, delete it from the end map.
                            em(endn+1,:)=[]; % Since the firstd branch is ignored and endn+1 is not considered as an end anymore, delete it from the end map.
                            if size(em, 1)>=endn+1
                                em((endn+1):end,1)=(endn+1):size(em,1); % Fix the first column values due to deletion.
                            end
                        end
                        pm((n+1):(n+size(branch_mp2, 1)),:)=[((n+1):(n+size(branch_mp2, 1)))', branch_mp2(:,:)]; % Store the second branch and connect to it.
                        n=n+size(branch_mp2, 1);
                        filament_connector=pm(n,:); % Preserve the endn+2 information for later on if it need s to delete the new filament.
                        i=connect_endn_plus_two_coordinate(1,2); % Start searching from the new end.
                        j=connect_endn_plus_two_coordinate(1,3);
                        pre_coord=[i,j]; % pre_coord is the same as [i,j].
                        pre_pre_coord=[i,j]; % pre_pre_coord is the same as [i,j].
                        exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                    elseif (end_to_end(1,1)==1) && (connect_endn_coordinate(1,1)>endn+2) % The endn-th end has a second nearest neighbor that has not processed for search filament yet.
                        pmt=pm(1:n, 2:3);
                        pmt=flipud(pmt);
                        if size(branch_mp1, 1)>=size(branch_mp2, 1) %Delete branch2 and connect branch1 to pm
                            n=n+size(branch_mp1, 1);
                            pm(1:n,2:3)=[flipud(branch_mp1);pmt];
                            if found==3 % branch2 end is at endn+1 position, delelte it because branch2 is ignored
                                em(endn+1,:)=[]; % Since the branch2 is ignored and endn+1 is not considered as an end anymore, delete it from the end map
                            elseif found==4 % branch2 end is at endn+2 position, delelte it because branch2 is ignored
                                em(endn+2,:)=[]; % Since the branch2 is ignored and endn+2 is not considered as an end anymore, delete it from the end map
                            end
                            %Now the branch1 end is at endn+1 position
                        else %Delete branch1 and connect branch2 to pm
                            n=n+size(branch_mp2, 1);
                            pm(1:n,2:3)=[flipud(branch_mp2);pmt];
                            if found==3 % branch1 end is at endn+2 position, delelte it because branch1 is ignored
                                em(endn+2,:)=[]; % Since the branch2 is ignored and endn+1 is not considered as an end anymore, delete it from the end map
                            elseif found==4 % branch1 end is at endn+1 position, delelte it because branch1 is ignored
                                em(endn+1,:)=[]; % Since the branch2 is ignored and endn+2 is not considered as an end anymore, delete it from the end map
                            end
                            %Now the branch2 end is at endn+1 position
                        end
                        filament_connector=pm(n,:); % Preserve the endn information for later on if it need s to delete the new filament.
                        em(connect_endn_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                        em(endn,:)=[]; % Since endn is not considered as an end anymore, delete it from the end map. Now branch1 or branch2 end is at endn position
                        if size(em, 1)~=0
                            em(:,1)=1:size(em,1); % Fix index due to deletion.
                        end
                        i=connect_endn_coordinate(1,2);
                        j=connect_endn_coordinate(1,3);
                        pre_coord=[i,j]; % pre_coord is the same as [i,j].
                        pre_pre_coord=[i,j]; % pre_pre_coord is the same as [i,j].
                        exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                    else% Either all the ends have no other ends two pixel away or have more than two other ends two pixel away. Stop search and reture the filament.
                        if size(branch_mp1, 1)>=size(branch_mp2, 1) %Delete branch2 and connect branch1 to pm
                            pm((n+1):(n+size(branch_mp1, 1)), :)=[((n+1):(n+size(branch_mp1, 1)))',branch_mp1];
                            n=n+size(branch_mp1, 1);
                            if found==3 % Delete branch2 end at endn+1
                                em(endn+1,:)=[];
                            elseif found==4 % Delete branch2 end at endn+2
                                em(endn+2,:)=[];
                            end
                        else %Delete branch1 and connect branch2 to pm
                            pm((n+1):(n+size(branch_mp2, 1)), :)=[((n+1):(n+size(branch_mp2, 1)))',branch_mp2];
                            n=n+size(branch_mp2, 1);
                            if found==3 % Delete branch1 end at endn+2
                                em(endn+2,:)=[];
                            elseif found==4 % Delete branch1 end at endn+1
                                em(endn+1,:)=[];
                            end
                        end
                        em(1:end,1)=1:size(em,1); % Fix the first column values due to deletion.
                        em_new=em; % Return end map.
                        endn_new=endn+1; % Return the number of processed ends. Next search will starts at endn+2.
                        pm((n+1):end,:)=[]; % Delete empty elements.
                        stop_search=1;
                    end
                else % The guy doesn't want one pixel gap between the ends of two filaments or end+2 is the last one in |em|. Stop searching and return the filament.
                    if size(branch_mp1, 1)>=size(branch_mp2, 1) %Delete branch2 and connect branch1 to pm
                        pm((n+1):(n+size(branch_mp1, 1)), :)=[((n+1):(n+size(branch_mp1, 1)))',branch_mp1];
                        n=n+size(branch_mp1, 1);
                        if found==3 % Delete branch2 end at endn+1
                            em(endn+1,:)=[];
                        elseif found==4 % Delete branch2 end at endn+2
                            em(endn+2,:)=[];
                        end
                    else %Delete branch1 and connect branch2 to pm
                        pm((n+1):(n+size(branch_mp2, 1)), :)=[((n+1):(n+size(branch_mp2, 1)))',branch_mp2];
                        n=n+size(branch_mp2, 1);
                        if found==3 % Delete branch1 end at endn+2
                            em(endn+2,:)=[];
                        elseif found==4 % Delete branch1 end at endn+1
                            em(endn+1,:)=[];
                        end
                    end
                    em(1:end,1)=1:size(em,1); % Fix the first column values due to deletion.
                    em_new=em; % Return end map.
                    endn_new=endn+1; % Return the number of processed ends. Next search will starts at endn+2.
                    pm((n+1):end,:)=[]; % Delete empty elements.
                    stop_search=1;
                end
            elseif ((size(brch_end_neighbor1,1)==0) && (size(brch_end_neighbor2,1)==0)) && ((size(branch_mp1, 1)>brch_size) && (size(branch_mp2, 1)<=brch_size)) % Branch 1 is longer than brch_size, while branch 2 is shorter. The both branche ends have no neighbor
                if n<brch_size % Current filament is shorter than brch_size. Hard to determine what to do. Therefore, delete filament and put the ending point of branch 1 into end position map at endn position and start new search from it.
                    if size(em, 1)>endn
                        for k=(endn+1):size(em, 1)
                            if (em(k,2)==branch_mp1(end,1)) && (em(k,3)==branch_mp1(end,2)) % Found.
                                em(k, 2:3)=em(endn, 2:3); % Exchange
                                em(endn, 2:3)=branch_mp1(end,:);
                                break;
                            end
                        end
                    end
                    if (em(endn, 2)~=branch_mp1(end,1)) || (em(endn, 3)~=branch_mp1(end,2)) % Did not find in branch_mp1(end, :) in em. Insert it.
                        emt=zeros(size(em, 1)+1, 3);
                        emt(1:(end-1),:)=em;
                        emt(end, :)=[size(emt, 1), emt(endn, 2:3)]; % Insert branch 1 end in endn position, but keep all elements after endn+1 at their position. Therefore, exchange position of endn with last one.
                        emt(endn,2:3)=branch_mp1(end,:);
                        em=emt;
                    end
                    n=size(branch_mp1, 1)-1; % Store branch 1 starting from branch 1 ending and overwrite current filament.
                    pmt=flipud(branch_mp1);
                    pm(1:n, :)=[(1:n)', pmt(1:n, :)];
                    i=branch_mp1(1, 1); % Starting point of branch 1 as next search.
                    j=branch_mp1(1, 2);
                    pre_coord=pm(n, 2:3);
                    pre_pre_coord=pm(n-1, 2:3);
                    exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                else % Find the branch 2 end in the end map |em| and delete it. Ignor branch 2 and connect to branch 1.
                    if size(em, 1)>endn
                        for k=(endn+1):size(em, 1)
                            if (em(k,2)==branch_mp2(end,1)) && (em(k,3)==branch_mp2(end,2)) % Found.
                                em(k,:)=[]; % Delete the end.
                                em(:,1)=1:size(em,1); % Fix index duo to deletion.
                                break;
                            end
                        end
                    end
                    pm((n+1):(n+size(branch_mp1, 1)-1), :)=[((n+1):(n+size(branch_mp1, 1)-1))',branch_mp1(1:(end-1),:)]; % Store branch 1 to the filament map |pm| and ignor branch 2.
                    n=n+size(branch_mp1, 1)-1; % This already is a complete filament. Take the last data point off and let it go through another iteration. It will end up at no-nearest neighbor case and will search the secondary nearest neighbors.
                    i=branch_mp1(end,1);
                    j=branch_mp1(end,2);
                    pre_coord=pm(n, 2:3);
                    if n<2
                        pre_pre_coord=pre_coord;
                    else
                        pre_pre_coord=pm(n-1, 2:3);
                    end
                    exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                end
            elseif ((size(brch_end_neighbor1,1)==0) && (size(brch_end_neighbor2,1)==0)) && ((size(branch_mp1, 1)<=brch_size) && (size(branch_mp2, 1)>brch_size)) % Branch 2 is longer than brch_size, while branch 1 is shorter.
                if n<brch_size % Current filament is shorter than brch_size. Hard to determine what to do. Therefore, delete filament and put the ending point of branch 2 into end position map at endn position and start new search from it.
                    if size(em, 1)>endn %Find branch2 end and put it at endn position
                        for k=(endn+1):size(em, 1)
                            if (em(k,2)==branch_mp2(end,1)) && (em(k,3)==branch_mp2(end,2)) % Found.
                                em(k, 2:3)=em(endn, 2:3); % Exchange
                                em(endn, 2:3)=branch_mp2(end,:);
                            end
                        end
                    end
                    if (em(endn, 2)~=branch_mp2(end,1)) || (em(endn, 3)~=branch_mp2(end,2)) % Did not find in branch_mp2(end, :) in em. Insert it.
                        emt=zeros(size(em, 1)+1, 3);
                        emt(1:(end-1),:)=em;
                        emt(end, :)=[size(emt, 1), emt(endn, 2:3)]; % Insert branch 2 end in endn position, but keep all elements after endn+1 at their position. Therefore, exchange position of endn with last one.
                        emt(endn,2:3)=branch_mp2(end,:);
                        em=emt;
                    end
                    n=size(branch_mp2, 1)-1; % Store branch 2 starting from branch 2 ending and overwrite current filament.
                    pmt=flipud(branch_mp2);
                    pm(1:n, :)=[(1:n)', pmt(1:n, :)];
                    i=branch_mp2(1, 1); % Starting point of branch 1 as next search.
                    j=branch_mp2(1, 2);
                    pre_coord=pm(n, 2:3);
                    pre_pre_coord=pm(n-1, 2:3);
                    exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                else % Find the branch 1 end in the end map |em| and delete it. Ignor branch 1 and connect to branch 2.
                    if size(em, 1)>endn
                        for k=(endn+1):size(em, 1)
                            if (em(k,2)==branch_mp1(end,1)) && (em(k,3)==branch_mp1(end,2)) % Found.
                                em(k,:)=[]; % Delete the end.
                                em(:,1)=1:size(em,1); % Fix index due to deletion.
                                break;
                            end
                        end
                    end
                    pm((n+1):(n+size(branch_mp2, 1)-1), :)=[((n+1):(n+size(branch_mp2, 1)-1))',branch_mp2(1:(end-1),:)]; % Store branch 2 to the filament map |pm| and ignor branch 1.
                    n=n+size(branch_mp2, 1)-1; % This already is a complete filament. Take the last data point off and let it go through another iteration. It will end up at no-nearest neighbor case and will search the secondary nearest neighbors.
                    i=branch_mp2(end,1);
                    j=branch_mp2(end,2);
                    pre_coord=pm(n, 2:3);
                    if n<2
                        pre_pre_coord=pre_coord;
                    else
                        pre_pre_coord=pm(n-1, 2:3);
                    end
                    exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                end
            elseif ((size(brch_end_neighbor1,1)==0) && (size(brch_end_neighbor2,1)==0)) && ((size(branch_mp1, 1)>brch_size) && (size(branch_mp2, 1)>brch_size)) && (n<=brch_size) % The two branches are longer than brch_size, but the current filament is shorter than brch_size.
                em(endn,:)=[]; % Ignor the cuurent fliament and delete the currnet filament starting end from |em|.
                em(:,1)=1:size(em,1); % Fix index duo to deletion. Real Pain in butt.
                % Find the two branch ends in the end map |em| and store branch 1 end to endn and branch 2 end to endn+1.
                found=0; % No branch end is found yet.
                if size(em, 1)>endn-1
                    for k=endn:size(em, 1)
                        if (found~=1)&&((em(k,2)==branch_mp1(end,1))&&(em(k,3)==branch_mp1(end,2))) % If branch1 end was not found before and found now, put the found the end to endn or endn+1 position.
                            if found==0 % No branch ends found before. put branch1 end to endn position
                                found=1;
                                if k~=endn
                                    em(k,2:3)=em(endn,2:3); % If k is not at endn position, exchange endn and k
                                    em(endn,2:3)=branch_mp1(end,:);
                                end
                            elseif found==2 % Found branch 2 end before and it is already in endn potsition, Put branch1 end to endn+1 position
                                found=3; % branch2 end is at endn while branch1 end is at endn+1
                                if k~=endn+1
                                    em(k,2:3)=em(endn+1,2:3); % If k is not at endn position, exchange endn+1 and k
                                    em(endn+1,2:3)=branch_mp1(end,:);
                                end
                                break; % stop searching.
                            end
                        elseif (found~=2)&&(em(k,2)==branch_mp2(end,1))&&(em(k,3)==branch_mp2(end,2)) % Foundand and put the found end to endn+1 position.
                            if found==0 % No branch ends found before. put branch2 end to endn position
                                found=2;
                                if k~=endn
                                    em(k,2:3)=em(endn,2:3); % If k is not at endn position, exchange endn and k
                                    em(endn,2:3)=branch_mp2(end,:);
                                end
                            elseif found==1 % Found branch 1 end before and it is already in endn potsition, Put branch2 end to endn+1 position
                                found=4; % branch1 end is at endn while branch2 end is at endn+1
                                if k~=endn+1
                                    em(k,2:3)=em(endn+1,2:3); % If k is not at endn position, exchange endn+1 and k
                                    em(endn+1,2:3)=branch_mp2(end,:);
                                end
                                break; % stop searching.
                            end
                        end
                    end
                end
                if found==2 % Found branch 2 end it is at endn position, but did not find branch 1 end in end map |em|. Insert it at endn+1 in |em|.
                    emt=zeros(size(em,1)+1, 3);
                    emt(1:(end-1), :)=em;
                    if size(emt, 1)>endn+1 % Insert branch 1 end in endn+1 position, but keep all elements after endn+1 at their position. Therefore, exchange position of endn with last one.
                        emt(end, :)=[size(emt,1),emt(endn+1, 2:3)];
                        emt(endn+1, 2:3)=branch_mp1(end,:);
                    elseif size(emt, 1)==endn+1
                        emt(end, :)=[endn+1, branch_mp1(end,:)];
                    end
                    em=emt;
                    found=3;
                elseif found==1 % Found branch1 end and it is at endn position, but did not find branch 2 end in end map |em|. Insert it in |em|.
                    emt=zeros(size(em,1)+1, 3);
                    emt(1:(end-1), :)=em;
                    if size(emt, 1)>endn+1 % Insert branch 2 end in endn+1 position, but keep all elements after endn+2 at their position. Therefore, exchange position of endn with last one.
                        emt(end, :)=[size(emt,1),emt(endn+1, 2:3)];
                        emt(endn+1, 2:3)=branch_mp2(end,:);
                    elseif size(emt, 1)==endn+1
                        emt(end, :)=[endn+1, branch_mp2(end,:)];
                    end
                    em=emt;
                    found=4;
                elseif found==0 % Did not find both branch ends in end map. Insert both at endn1 and endn+1 position.
                    emt=zeros(size(em,1)+2, 3);
                    emt(1:endn-1, :)=em(1:endn-1, :);
                    emt(endn, :)=[endn,branch_mp1(end,:)]; % Insert branch1 end at endn+1
                    emt(endn+1, :)=[endn+1,branch_mp2(end,:)]; % Insert branch2 end at endn+2
                    if size(emt, 1)>endn+1 % Append the rest of end in end map.
                        emt((endn+2):end, :)=[((endn+2):size(emt,1))',em(endn:end, 2:3)];
                    end
                    em=emt;
                    found=4;
                end
                if found==3 % Branch2 end at endn while branch2 end at branch1 end at endn+1
                    % Start from branch 2 and end at branch1, This already is a complete filament.
                    % Take the last data point off and let it go through another iteration.
                    % It will end up at no-nearest neighbor case and will search the secondary nearest neighbors.
                    pmt=[flipud(branch_mp2);pm(n,2:3);branch_mp1]; % Construct the new filament by branch 2, starting from the ending point of branch2, the n point of current filament and the whole branch 1, ending at the ending point of branch 1.
                    n=size(branch_mp2,1)+1+size(branch_mp1,1)-1;
                    i=branch_mp1(end,1); %Branch1 end as the next search
                    j=branch_mp1(end,2);
                elseif found==4 % Branch2 end at endn while branch2 end at branch1 end at endn+1
                    pmt=[flipud(branch_mp1);pm(n,2:3);branch_mp2]; % Construct the new filament by branch 1, starting from the ending point of branch1, the n point of current filament and the whole branch 2, ending at the ending point of branch 2.
                    n=size(branch_mp1,1)+1+size(branch_mp2,1)-1;
                    i=branch_mp2(end,1); %Branch2 end as the next search
                    j=branch_mp2(end,2);
                end
                pm(1:n,1)=1:n;
                pm(1:n,2:3)=pmt(1:(end-1), :);
                pre_coord=pm(n, 2:3);
                pre_pre_coord=pm(n-1, 2:3);
                exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
            elseif ((size(brch_end_neighbor1,1)==0) && (size(brch_end_neighbor2,1)>=2)) && (size(branch_mp1, 1)<=brch_size)  % Branch 1 ends at a non-branching point and its length is shorter than brch_size, while branch 2 ends at a branching point.
                % Find the branch 1 end in the end map |em|. If found, put it to endn+1 position
                if size(em, 1)>endn
                    for k=(endn+1):size(em, 1)
                        if (em(k,2)==branch_mp1(end,1)) && (em(k,3)==branch_mp1(end,2)) % Found.
                            if k>endn+1
                                em(k, 2:3)=em(endn+1, 2:3);
                                em(endn+1, 2:3)=branch_mp1(end,:);
                            end
                            break;
                        end
                    end
                end
                if (size(em,1)<endn+1) || (em(endn+1,2)~=branch_mp1(end,1))|| (em(endn+1,3)~=branch_mp1(end,2)) % Did not find branch 1 end in end map |em|. Insert it in |em|.
                    emt=zeros(size(em,1)+1, 3);
                    emt(1:(end-1), :)=em;
                    if size(emt, 1)>endn+1 % Insert branch 1 end in endn+1 position, but keep all elements after endn+1 at their position. Therefore, exchange position of endn+1 with last one.
                        emt(end, :)=[size(emt,1),emt(endn+1, 2:3)];
                        emt(endn+1, 2:3)=branch_mp1(end,:);
                    elseif size(emt, 1)==endn+1
                        emt(endn+1, :)=[endn+1, branch_mp1(end,:)];
                    end
                    em=emt;
                end
                if n<brch_size % Current filament is shorter than brch_size.
                    if (strcmp(single_pixel_gap_reply, 'Yes')) && (endn+1<size(em,1))
                        % Search the starting point of current filament and
                        % branch 1 ending for the secondary nearest
                        % neighbor.  If there is one, connect that branch with branch 2.
                        end_to_end=[0,0]; % |end_to_end| counts number of ends in |em| that are the second nearest neighbor (two points away) to em(endn) (first column) or em(endn+1) (branch_mp1(end,:)) (second column) except endn and endn+1 themselves.
                        connect_endn_coordinate=zeros(size(em,1)-2,3); % Store the coordinates of the second nearest neighbors to endn. The first column is the index in end map |em|.
                        connect_endn_plus_coordinate=zeros(size(em,1)-2,3); % Store the coordinates of the second nearest neighbors to branch 1 end. The first column is the index in end map |em|.
                        for k=1:size(em,1) % From filament starting end and both branch ends, search other ends within two pixels away.
                            if (k~=endn)&&(k~=endn+1)
                                if ((em(endn,2)==pm(1,2))&&(em(endn, 3)==pm(1,3)))&&(((em(k,2)-em(endn,2))^2<=4)&&((em(k,3)-em(endn,3))^2<=4)) % Make sure the endn-th end is the starting of the filament. If not, the filament starts from two ingnored ends. Therefore don't search for the secondary nearest neighbors.
                                    end_to_end(1,1)=end_to_end(1,1)+1;
                                    connect_endn_coordinate(end_to_end(1,1), :)=em(k,:);
                                elseif ((em(k,2)-em(endn+1,2))^2<=4)&&((em(k,3)-em(endn+1,3))^2<=4)
                                    end_to_end(1,2)=end_to_end(1,2)+1;
                                    connect_endn_plus_coordinate(end_to_end(1,2), :)=em(k,:);
                                end
                            end
                        end
                        if (end_to_end(1,1)==1) && (connect_endn_coordinate(1,1)>endn+1) && (end_to_end(1,2)~=1) % Starting end of current filament has a secondary nearest neighbor and branch 1 has no or more than 2 secondary neighbors.
                            em(endn+1, :)=[]; % Ignor first branch and delete the end. Connect current filament to branch 2.
                            em(:,1)=1:size(em,1); % Fix index duo to deletion.
                            pm((n+1):(n+size(branch_mp2, 1)-1), :)=[((n+1):(n+size(branch_mp2, 1)-1))',branch_mp2(1:(end-1),:)]; % Store branch 2 to the filament map |pm| and ignor branch 1.
                            n=n+size(branch_mp2, 1)-1; % Take the last data point off the filament and let it go through another iteration. It will end up at two-nearest neighbor case and will deal with it then.
                        elseif (end_to_end(1,2)==1) && (connect_endn_plus_coordinate(1,1)>endn+1) && (end_to_end(1,1)~=1)  % branch 1 end has a secondary nearest neighbor and the starting of current filament has no or more than 2 secondary neighbors.
                            em(endn, :)=[]; % Ignor the current filament and delete the end. Connect branch 1 to branch 2.
                            em(:,1)=1:size(em,1); % Fix index due to deletion.
                            pmt=[flipud(branch_mp1); pm(n, 2:3); branch_mp2];
                            n=size(branch_mp1, 1)+1+size(branch_mp2, 1)-1; % Take the last data point off the filament and let it go through another iteration.
                            pm(1:n, :)=[(1:n)',pmt(1:n,:)]; % Store branch 1 and branch 2 to the filament map |pm| and overwrite current filament.
                        else % Current starting and branch1 end have no suitable secondary neighbors. Delete either one of them and connect another to branch2
                            if size(branch_mp1,1)>n
                                em(endn, :)=[]; % Ignor the current filament and delete the end. Connect branch 1 to branch 2.
                                em(:,1)=1:size(em,1); % Fix index due to deletion.
                                pmt=[flipud(branch_mp1); pm(n, 2:3); branch_mp2];
                                n=size(branch_mp1, 1)+1+size(branch_mp2, 1)-1; % Take the last data point off the filament and let it go through another iteration.
                                pm(1:n, :)=[(1:n)',pmt(1:n,:)]; % Store branch 1 and branch 2 to the filament map |pm| and overwrite current filament.
                            else
                                em(endn+1, :)=[]; % Ignor first branch and delete the end. Connect current filament to branch 2.
                                em(:,1)=1:size(em,1); % Fix index duo to deletion.
                                pm((n+1):(n+size(branch_mp2, 1)-1), :)=[((n+1):(n+size(branch_mp2, 1)-1))',branch_mp2(1:(end-1),:)]; % Store branch 2 to the filament map |pm| and ignor branch 1.
                                n=n+size(branch_mp2, 1)-1; % Take the last data point off the filament and let it go through another iteration. It will end up at two-nearest neighbor case and will deal with it then.
                            end
                        end
                        i=branch_mp2(end,1);
                        j=branch_mp2(end,2);
                        pre_coord=pm(n, 2:3);
                        pre_pre_coord=pm(n-1, 2:3);
                        exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                    else % Keep the longer one between current filament and branch 1, and delete the short one.
                        if size(branch_mp1, 1) > n % Branch 1 is longer. Replace the current with branch 1.
                            em(endn, :)=[]; % Ignor the current filament and delete the end.
                            if size(em, 1)~=0
                                em(:,1)=1:size(em,1); % Fix index due to deletion.
                            end
                            pmt=[flipud(branch_mp1); pm(n, 2:3); branch_mp2]; % Connect branch 1 to branch 2.
                        else % Branch 1 is not longer than current. Igor branch 1 and delete the branch 1 end.
                            em(endn+1, :)=[];
                            if size(em, 1)~=0
                                em(:,1)=1:size(em,1); % Fix index due to deletion.
                            end
                            pmt=[pm(:, 2:3); branch_mp2]; % Connect current filament to branch 2.
                        end
                        n=size(pmt,1)-1; % Take the last point off the filament and let it go through another iteration.
                        pm(1:n, 1)=1:n;
                        pm(1:n, 2:3)=pmt(1:(end-1), :);
                        i=branch_mp2(end,1);
                        j=branch_mp2(end,2);
                        pre_coord=pm(n, 2:3);
                        pre_pre_coord=pm(n-1, 2:3);
                        exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                    end
                else % Current filament is longer than branch allowed. Ignor branch 1 and connect current filament to branch 2.
                    em(endn+1, :)=[];
                    if size(em, 1)~=0
                        em(:,1)=1:size(em,1); % Fix index due to deletion.
                    end
                    pm((n+1):(n+size(branch_mp2, 1)-1), :)=[((n+1):(n+size(branch_mp2, 1)-1))',branch_mp2(1:(end-1),:)]; % Store branch 2 to the filament map |pm| and ignor branch 1.
                    n=n+size(branch_mp2, 1)-1; % Take the last data point off the filament and let it go through another iteration. It will end up at two-nearest neighbor case and will deal with it then.
                    i=branch_mp2(end,1);
                    j=branch_mp2(end,2);
                    pre_coord=pm(n, 2:3);
                    pre_pre_coord=pm(n-1, 2:3);
                    exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                end
            elseif ((size(brch_end_neighbor1,1)>=2) && (size(brch_end_neighbor2,1)==0)) && (size(branch_mp2, 1)<=brch_size) % Branch 2 ends at a non-branching point and its length is less than brch_size, while branch 1 ends at a branching point.
                % Find the branch 2 end in the end map |em|. If found, put it to endn+1 position
                if size(em, 1)>endn
                    for k=(endn+1):size(em, 1)
                        if (em(k,2)==branch_mp2(end,1)) && (em(k,3)==branch_mp2(end,2)) % Found.
                            if k>endn+1
                                em(k, 2:3)=em(endn+1, 2:3);
                                em(endn+1, 2:3)=branch_mp2(end,:);
                            end
                            break;
                        end
                    end
                end
                if (size(em,1)<endn+1) || (em(endn+1,2)~=branch_mp2(end,1))|| (em(endn+1,3)~=branch_mp2(end,2)) % Did not find branch 2 end in end map |em|. Insert it in |em|.
                    emt=zeros(size(em,1)+1, 3);
                    emt(1:(end-1), :)=em;
                    if size(emt, 1)>endn+1 % Insert branch 2 end in endn+1 position, but keep all elements after endn+1 at their position. Therefore, exchange position of endn+1 with last one.
                        emt(end, :)=[size(emt,1),emt(endn+1, 2:3)];
                        emt(endn+1, 2:3)=branch_mp2(end,:);
                    elseif size(emt, 1)==endn+1
                        emt(end, :)=[endn+1, branch_mp2(end,:)];
                    end
                    em=emt;
                end
                if n<brch_size % Current filament is shorter than brch_size.
                    if (strcmp(single_pixel_gap_reply, 'Yes')) && ((endn+1)<size(em,1))
                        % Search the starting point of current filament and
                        % branch 2 ending for the secondary nearest
                        % neighbor.  If there is one, connect that branch with branch 1.
                        end_to_end=[0,0]; % |end_to_end| counts number of ends in |em| that are the second nearest neighbor (two points away) to em(endn) (first column) or em(endn+1) (branch_mp2(end,:)) (second column) except endn and endn+1 themselves.
                        connect_endn_coordinate=zeros(size(em,1)-2,3); % Store the coordinates of the second nearest neighbors to endn. The first column is the index in end map |em|.
                        connect_endn_plus_coordinate=zeros(size(em,1)-2,3); % Store the coordinates of the second nearest neighbors to endn+1. The first column is the index in end map |em|.
                        for k=1:size(em,1) % From filament starting end and both branch ends, search other ends within two pixels away.
                            if (k~=endn)&&(k~=endn+1)
                                if ((em(endn,2)==pm(1,2))&&(em(endn, 3)==pm(1,3)))&&(((em(k,2)-em(endn,2))^2<=4)&&((em(k,3)-em(endn,3))^2<=4)) % Make sure the endn-th end is the starting of the filament. If not, the filament starts from two ingnored ends. Therefore don't search for the secondary nearest neighbors.
                                    end_to_end(1,1)=end_to_end(1,1)+1;
                                    connect_endn_coordinate(end_to_end(1,1), :)=em(k,:);
                                elseif ((em(k,2)-em(endn+1,2))^2<=4)&&((em(k,3)-em(endn+1,3))^2<=4)
                                    end_to_end(1,2)=end_to_end(1,2)+1;
                                    connect_endn_plus_coordinate(end_to_end(1,2), :)=em(k,:);
                                end
                            end
                        end
                        if (end_to_end(1,1)==1) && (connect_endn_coordinate(1,1)>endn+1) && (end_to_end(1,2)~=1) % Starting end of current filament has a secondary nearest neighbor and branch 2 has no or more tahn 2.
                            em(endn+1, :)=[]; % Ignor branch 2 and delete the end. Connect current filament to branch 1.
                            em(:,1)=1:size(em,1); % Fix index duo to deletion.
                            pm((n+1):(n+size(branch_mp1, 1)-1), :)=[((n+1):(n+size(branch_mp1, 1)-1))',branch_mp1(1:(end-1),:)]; % Store branch 2 to the filament map |pm| and ignor branch 1.
                            n=n+size(branch_mp1, 1)-1; % Take the last data point off the filament and let it go through another iteration. It will end up at two-nearest neighbor case and will deal with it then.
                        elseif (end_to_end(1,2)==1) &&  (connect_endn_plus_coordinate(1,1)>endn+1) && (end_to_end(1,1)~=1) % branch 2 end has a secondary nearest neighbor and the starting of current filament has no or more tahn 2.
                            em(endn, :)=[]; % Ignor the current filament and delete the end. Connect branch 2 to branch 1.
                            em(:,1)=1:size(em,1); % Fix index due to deletion.
                            pmt=[flipud(branch_mp2); pm(n, 2:3); branch_mp1];
                            n=size(branch_mp2, 1)+1+size(branch_mp1, 1)-1; % Take the last data point off the filament and let it go through another iteration.
                            pm(1:n, :)=[(1:n)',pmt(1:n,:)]; % Store branch 1 and branch 2 to the filament map |pm| and overwrite current filament.
                        else % Current starting and branch1 end have no suitable secondary neighbors. Delete either one of them and connect another to branch1
                            if size(branch_mp2,1)>n
                                em(endn, :)=[]; % Ignor the current filament and delete the end. Connect branch 2 to branch 1.
                                em(:,1)=1:size(em,1); % Fix index due to deletion.
                                pmt=[flipud(branch_mp2); pm(n, 2:3); branch_mp1];
                                n=size(branch_mp2, 1)+1+size(branch_mp1, 1)-1; % Take the last data point off the filament and let it go through another iteration.
                                pm(1:n, :)=[(1:n)',pmt(1:n,:)]; % Store branch 1 and branch 2 to the filament map |pm| and overwrite current filament.
                            else
                                em(endn+1, :)=[]; % Ignor second branch and delete the end. Connect current filament to branch 1.
                                em(:,1)=1:size(em,1); % Fix index duo to deletion.
                                pm((n+1):(n+size(branch_mp1, 1)-1), :)=[((n+1):(n+size(branch_mp1, 1)-1))',branch_mp1(1:(end-1),:)]; % Store branch 2 to the filament map |pm| and ignor branch 1.
                                n=n+size(branch_mp1, 1)-1; % Take the last data point off the filament and let it go through another iteration. It will end up at two-nearest neighbor case and will deal with it then.
                            end
                        end
                        i=branch_mp1(end,1);
                        j=branch_mp1(end,2);
                        pre_coord=pm(n, 2:3);
                        pre_pre_coord=pm(n-1, 2:3);
                        exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                    else % Keep the longer one between current filament and branch 2, and delete the short one.
                        if size(branch_mp2, 1) > n % Branch 2 is longer. Replace the current with branch 2.
                            em(endn, :)=[]; % Ignor the current filament and delete the end.
                            if size(em, 1)~=0
                                em(:,1)=1:size(em,1); % Fix index due to deletion.
                            end
                            pmt=[flipud(branch_mp2); pm(n, 2:3); branch_mp1]; % Connect branch 2 to branch 1.
                        else % Branch 2 is not longer than current. Igor branch 2 and delete the branch 2 end.
                            em(endn+1, :)=[];
                            if size(em, 1)~=0
                                em(:,1)=1:size(em,1); % Fix index due to deletion.
                            end
                            pmt=[pm(:, 2:3); branch_mp1]; % Connect current filament to branch 1.
                        end
                        n=size(pmt,1)-1; % Take the last point off the filament and let it go through another iteration.
                        pm(1:n, 1)=1:n;
                        pm(1:n, 2:3)=pmt(1:(end-1), :);
                        i=branch_mp1(end,1);
                        j=branch_mp1(end,2);
                        pre_coord=pm(n, 2:3);
                        pre_pre_coord=pm(n-1, 2:3);
                        exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                    end
                else % Ignor branch 2 and connect current filament to branch 1.
                    em(endn+1, :)=[];
                    if size(em, 1)~=0
                        em(:,1)=1:size(em,1); % Fix index due to deletion.
                    end
                    pm((n+1):(n+size(branch_mp1, 1)-1), :)=[((n+1):(n+size(branch_mp1, 1)-1))',branch_mp1(1:(end-1),:)]; % Store branch 2 to the filament map |pm| and ignor branch 1.
                    n=n+size(branch_mp1, 1)-1; % Take the last data point off the filament and let it go through another iteration. It will end up at two-nearest neighbor case and will deal with it then.
                    i=branch_mp1(end,1);
                    j=branch_mp1(end,2);
                    pre_coord=pm(n, 2:3);
                    pre_pre_coord=pm(n-1, 2:3);
                    exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                end
            elseif ((size(brch_end_neighbor1,1)==0) && (size(brch_end_neighbor2,1)>=2)) && (size(branch_mp1, 1)>brch_size) && (n<=brch_size) % Branch 1 ends at a non-branching point and its length is longer than brch_size, while branch 2 ends at a branching point. However, the current filament length is shorter than brch_size.
                em(endn,:)=[]; % Ignor the cuurent fliament and delete the currnet filament starting end from |em|.
                % Find branch 1 ending in the end map |em| and store branch 1 end to endn.
                if size(em, 1)>endn-1
                    for k=endn:size(em, 1)
                        if ((em(k,2)==branch_mp1(end,1))&&(em(k,3)==branch_mp1(end,2))) % Found and put the found end to end position.
                            if k~=endn
                                em(k,2:3)=em(endn,2:3); % If k is not endn, exchange endn and k
                                em(endn,2:3)=branch_mp1(end,:);
                            end
                            break;
                        end
                    end
                end
                if (size(em, 1)<endn) || (em(endn,2)~=branch_mp1(end,1))|| (em(endn,3)~=branch_mp1(end,2)) % Did not find branch 1 end in end map |em|. Insert it in |em|.
                    emt=zeros(size(em,1)+1, 3);
                    emt(1:(end-1), :)=em;
                    if size(emt, 1)>endn % Insert branch 1 end in endn position, but keep all elements after endn+1 at their position. Therefore, exchange position of endn1 with last one.
                        emt(end, :)=[size(emt,1),emt(endn, 2:3)];
                        emt(endn, 2:3)=branch_mp1(end,:);
                    elseif size(emt, 1)==endn
                        emt(end, :)=[endn, branch_mp1(end,:)];
                    end
                    em=emt;
                end
                em(:,1)=1:size(em,1); % Fix index duo to deletion.
                pmt=[flipud(branch_mp1);pm(n,2:3);branch_mp2]; % Construct the new filament by branch 1, starting from the ending point of branch1, the n point of current filament and the whole branch 2, ending at the ending point of branch 2.
                n=size(branch_mp1, 1)+1+size(branch_mp2, 1)-1; % This already is a complete filament. Take the last data point off and let it go through another iteration. It will end up at no-nearest neighbor case and will search the secondary nearest neighbors.
                pm(1:n,1)=1:n;
                pm(1:n,2:3)=pmt(1:n, :);
                i=branch_mp2(end,1);
                j=branch_mp2(end,2);
                pre_coord=pm(n, 2:3);
                pre_pre_coord=pm(n-1, 2:3);
                exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
            elseif ((size(brch_end_neighbor1,1)>=2) && (size(brch_end_neighbor2,1)==0)) && (size(branch_mp2, 1)>brch_size) && (n<=brch_size) % Branch 2 ends at a non-branching point and its length is longer than brch_size, while branch 1 ends at a branching point. However, the current filament length is shorter than brch_size
                em(endn,:)=[]; % Ignor the cuurent fliament and delete the currnet filament starting end from |em|.
                % Find branch 2 ending in the end map |em| and store branch 2 end to endn.
                if size(em, 1)>endn-1
                    for k=endn:size(em, 1)
                        if ((em(k,2)==branch_mp2(end,1))&&(em(k,3)==branch_mp2(end,2))) % Found and put the found end to endn position.
                            if k~=endn
                                em(k,2:3)=em(endn,2:3); % If k is not endn, exchange endn and k
                                em(endn,2:3)=branch_mp2(end,:);
                            end
                            break;
                        end
                    end
                end
                if (size(em, 1)<endn) || (em(endn,2)~=branch_mp2(end,1))|| (em(endn,3)~=branch_mp2(end,2)) % Did not find branch 2 end in end map |em|. Insert it in |em|.
                    emt=zeros(size(em,1)+1, 3);
                    emt(1:(end-1), :)=em;
                    if size(emt, 1)>endn % Insert branch 2 end in endn position, but keep all elements after endn+1 at their position. Therefore, exchange position of endn1 with last one.
                        emt(end, :)=[size(emt,1),emt(endn, 2:3)];
                        emt(endn, 2:3)=branch_mp2(end,:);
                    elseif size(emt, 1)==endn
                        emt(end, :)=[endn, branch_mp2(end,:)];
                    end
                    em=emt;
                end
                em(:,1)=1:size(em,1); % Fix index duo to deletion.
                pmt=[flipud(branch_mp2);pm(n,2:3);branch_mp1]; % Construct the new filament by branch 1, starting from the ending point of branch1, the n point of current filament and the whole branch 2, ending at the ending point of branch 2.
                n=size(branch_mp1, 1)+1+size(branch_mp2, 1)-1; % This already is a complete filament. Take the last data point off and let it go through another iteration. It will end up at no-nearest neighbor case and will search the secondary nearest neighbors.
                pm(1:n,1)=1:n;
                pm(1:n,2:3)=pmt(1:n,:);
                i=branch_mp1(end,1);
                j=branch_mp1(end,2);
                pre_coord=pm(n, 2:3);
                pre_pre_coord=pm(n-1, 2:3);
                exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
            else % Both Branch 1 and branch 2 are branching points or one of branches has more than three new nearest neighbor. Therefore the n point is a branching point and delete the filament to one pixel gap if any.
                % If either branch1 or and branch2 or both ends at no
                % neighbor, find it or them and delete
                if (size(brch_end_neighbor1,1)==0) && (size(em,1)>endn) % Branch1 end has no neighbor. Delete the end from end map |em|
                    for k=(endn+1):size(em,1)
                        if ((em(k,2)==branch_mp1(end,1))&&(em(k,3)==branch_mp1(end,2))) % Found and delelte.
                            em(k,:)=[];
                            em(1:end, 1)=1:size(em,1); % fix the index due to deletion
                            break;
                        end
                    end
                end
                if (size(brch_end_neighbor2,1)==0) && (size(em,1)>endn) % Branch2 end has no neighbor. Delete the end from end map |em|
                    for k=(endn+1):size(em,1)
                        if ((em(k,2)==branch_mp2(end,1))&&(em(k,3)==branch_mp2(end,2))) % Found and delelte.
                            em(k,:)=[];
                            em(1:end, 1)=1:size(em,1); % fix the index due to deletion
                            break;
                        end
                    end
                end
                if filament_connector(1,1)~=0 % The current filament connects the previous filament through one pixel gap. Therefore, needs to put the end point of last filament back to end map |em| that was previously deleted from there.
                    emt=zeros(size(em,1)+1,3);
                    emt(:,1)=1:size(emt,1);
                    emt(1:endn,2:3)=em(1:endn, 2:3); % It is the end of last filament, so put at endn+1 position
                    emt(endn+1,2:3)=filament_connector(1, 2:3);
                    if size(em, 1)>=endn+1
                        emt((endn+2):end,2:3)=em((endn+1):end,2:3);
                    end
                    em=emt;
                    pm((filament_connector(1,1)+1):end,:)=[]; % Delete the nearest neighbor connected points from the last filament connector to the end.
                    n = filament_connector(1,1); %this is V4.2.3 correction
                    if ((pm(1, 2)==em(endn,2) )&& (pm(1, 3)==em(endn,3))) && (strcmp(single_pixel_gap_reply, 'Yes')) && ((endn+1)<size(em,1)) % The starting point of the filament is the end at endn position in end map |em|. Also the guy wants one pixel gap and there are un-processed ends in |em|.
                        end_to_end=0; % |end_to_end| counts number of ends in |em| that are the second nearest neighbor (two points away) to em(endn) except endn and endn+1 themselves.
                        connect_endn_coordinate=zeros(size(em,1)-2,3); % Store the coordinates of the second nearest neighbors to endn. The first column is the index in end map |em|.
                        for k=1:size(em,1)
                            if (k~=endn) && (k~=endn+1) && ((em(k,2)-em(endn,2))^2<=4)&&((em(k,3)-em(endn,3))^2<=4) % Search secondary nearest neighbors to the starting point of the filament.
                                end_to_end=end_to_end+1;
                                connect_endn_coordinate(end_to_end, :)=em(k,:);
                            end
                        end
                        if (end_to_end==1) && (connect_endn_coordinate(1,1)>endn+1) % The endn-th end has a second nearest neighbor that has not processed for search filament yet.
                            pmt=pm(:, 2:3);
                            pmt=flipud(pmt);
                            pm(:, 2:3)=pmt; % Change filament search direction.
                            filament_connector=pm(end,:); % Preserve the endn information for later on if it needs to delete the new filament.
                            em(connect_endn_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                            em(endn,:)=[]; % Since endn is not considered as an end anymore, delete it from the end map.
                            em(endn:end,1)=endn:size(em,1); % Fix the first column values due to deletion.
                            i=connect_endn_coordinate(1,2);
                            j=connect_endn_coordinate(1,3);
                            pre_coord=[i,j]; % pre_coord is the same as [i,j].
                            pre_pre_coord=[i,j]; % pre_pre_coord is the same as [i,j].
                            exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                        else % Either the end have no other ends two pixel away or the end has more than two other ends two pixel away. Either case, stop search and reture the filament.
                            em_new=em;
                            endn_new=endn+1;
                            % The empty elements in pm have been
                            % deleted and pm is ready to return.
                            stop_search=1;
                        end
                    else % No search secondary nearest neighbors. Return data.
                        em_new=em;
                        endn_new=endn+1;
                        % The empty elements in pm have been deleted and pm is ready to return.
                        stop_search=1;
                    end
                else % The branched filament is a nearest neighbor-connected filament. Delete the whole branched filament and return.
                    stop_search=1;
                    em(endn,:)=[]; % Delete the end.
                    if size(em,1)~=0
                        em(:,1)=1:size(em,1);
                    end
                    em_new=em;
                    endn_new=endn-1;
                    pm(1:end,:)=[];
                end
            end
        end
    else   % the nth point is a branch or crossing point.
        if filament_connector(1,1)~=0 % The current filament connects the previous filament through one pixel gap. Therefore, needs to put the starting point of this filament back to end map |em| that was previously deleted from there.
            emt=zeros(size(em,1)+1,3);
            emt(:,1)=1:size(emt,1);
            emt(1:endn,2:3)=em(1:endn, 2:3);
            emt(endn+1,2:3)=filament_connector(1, 2:3);
            if size(em, 1)>=endn+1
                emt((endn+2):end,2:3)=em((endn+1):end,2:3);
            end
            em=emt;
            pm((filament_connector(1,1)+1):end,:)=[]; % Delete the nearest neighbor connected points from the last filament connector to the end.
            n = filament_connector(1,1); %this is V4.2.3 correction
            if ((pm(1, 2)==em(endn,2) )&& (pm(1, 3)==em(endn,3))) && (strcmp(single_pixel_gap_reply, 'Yes')) && ((endn+1)<size(em,1)) % The starting point of the filament is the end at endn position in end map |em|. Also the guy wants one pixel gap and there are un-processed ends in |em|.
                end_to_end=0; % |end_to_end| counts number of ends in |em| that are the second nearest neighbor (two points away) to em(endn) except endn and endn+1 themselves.
                connect_endn_coordinate=zeros(size(em,1)-2,3); % Store the coordinates of the second nearest neighbors to endn. The first column is the index in end map |em|.
                for k=1:size(em,1)
                    if (k~=endn) && (k~=endn+1) && ((em(k,2)-em(endn,2))^2<=4)&&((em(k,3)-em(endn,3))^2<=4) % Search secondary nearest neighbors to the starting point of the filament.
                        end_to_end=end_to_end+1;
                        connect_endn_coordinate(end_to_end, :)=em(k,:);
                    end
                end
                if (end_to_end==1) && (connect_endn_coordinate(1,1)>endn+1) % The endn-th end has a second nearest neighbor that has not processed for search filament yet.
                    pmt=pm(:,2:3);
                    pmt=flipud(pmt);
                    pm(:, 2:3)=pmt; % Change filament search direction.
                    filament_connector=pm(end,:); % Preserve the endn information for later on if it need s to delete the new filament.
                    em(connect_endn_coordinate(1,1),:)=[]; % The found end in end map is no longer end. Delete it.
                    em(endn,:)=[]; % Since endn is not considered as an end anymore, delete it from the end map.
                    em(endn:end,1)=endn:size(em,1); % Fix the first column values due to deletion.
                    i=connect_endn_coordinate(1,2);
                    j=connect_endn_coordinate(1,3);
                    pre_coord=[i,j]; % pre_coord is the same as [i,j].
                    pre_pre_coord=[i,j]; % pre_pre_coord is the same as [i,j].
                    exclude_pnt=[i,j]; % exclude_pnt is the same as [i, j].
                else % Either the filament starting end has no other ends two pixel away or it has more than two other ends two pixel away. Either case, stop search and reture the filament.
                    em_new=em;
                    endn_new=endn+1;
                    % The empty elements in pm have been
                    % deleted and pm is ready to return.
                    stop_search=1;
                end
            else % Either the starting filament end is not an end (actually starts with two short ignored branches) or there is no un-processed ends in |em|. Stop search and reture the filament.
                em_new=em;
                endn_new=endn+1;
                % The empty elements in pm have been
                % deleted and pm is ready to return.
                stop_search=1;
            end
        else % There is no previouse filament.
            em(endn,:)=[]; % Delete the starting point, i.e the end, from the end map |em|.
            if size(em,1)~=0
                em(endn:end,1)=endn:size(em, 1); % Fix the first column.
            end
            em_new=em;
            endn_new=endn-1; % Since deleted endn point from the end map, next one becomes endn to be processed.
            pm(1:end,:)=[]; % Delete the nearest neighbor connected points from the last filament connector to the end.
            stop_search=1;
        end
    end
%     clear empty_script;
%     empty_script;
end
end
% The end of function position.      
       
%% Subfunction find_end
% This filament end finding algorithm searches for pixels that have 1 
% nearest neighbor in a binary skeletonized image |img|.  It returns the 
% (|i|,|j|) coordinates for each |n|th end found in an array.
% Usage: find_end(img) 
% Copyright 2012 Brannon R McCullough
function em=find_end(img)
[imx,jmx]=size(img);
% eimg=zeros(imx,jmx); %WC comment: eimg is the end position map, but never been used.
em=zeros(0,3); % WC comment: sum(sum(eimg)) is 0. It is just silly to use sum(sum(eimg))
n=1;
for i=1:imx
    for j=1:jmx
        if img(i,j)==1
            horver=img(i-1,j)+img(i+1,j)+img(i,j-1)+img(i,j+1);
            diag=img(i-1,j-1)+img(i-1,j+1)+img(i+1,j-1)+img(i+1,j+1);
            up=sum(img(i-1,j-1:j+1));
            down=sum(img(i+1,j-1:j+1));
            left=sum(img(i-1:i+1,j-1));
            right=sum(img(i-1:i+1,j+1));
            if horver+diag==1 || ((horver==1&&diag==1) && (up==2||down==2||left==2||right==2))
                % eimg(i,j)=1;
                em(n,1:3)=[n i j];% accounts for the one pixel padding
                n=n+1;
            end
        end
    end
end
end
%% Subfunction pad
% Pads an image |img| with zeros by one pixel to allow a 3x3 window to pass 
% over it.
% Usage: pad(img)
function pim=pad(img)
[imx, jmx]=size(img);
pim=zeros(imx+2,jmx+2);
pim(2:imx+1,2:jmx+1)=img;
end
%The end of function pad
%% Subfunction bezier reconstruct
% This function reconstructs a line using a bezier reconstruction 
% without incorporating the trailing tail end of the filament. It calls
% subsequent functions |bezier|,|bernstein| and |n_choose_i|.  The input is
% a position coordinate map |Map|, reconstruction points |k| (redigitizing
% points), , and order of Bzier spline
% |bn| (bn=3 is the 3rd order Bzier spline), and returns the coordinate map for the reconstruction (redeigitized).
% Usage: bezier_reconstruct(Map,k,bn)
function [BezierMap,devBezierMap]=bezier_reconstruct(Map,k,bn)
NPoints=size(Map,1);
tot_sn=floor((NPoints-1)/bn); %number of bn segemnt group (Bezier segment)  and disregard the end segments less than bn
BezierMap=zeros((k-1)*tot_sn+1,2); % Constructed filament by Bezier curves, store Bezier segments from every bn original segments
devBezierMap=zeros((k-1)*tot_sn+1,2); % Derivative of the constructed filament by Bezier curves
for sn=1:tot_sn %for every Bezier segment, i.e. bn original segments 
    [BezierMap_temp,devBezierMap_temp]=bezier(Map((bn*(sn-1)+1):(bn*sn+1),:),k); %bezier is only 3rd order (4 points) spline curve.
    if (sn==1)
        BezierMap(((k-1)*(sn-1)+1):((k-1)*sn+1),:) = BezierMap_temp;
        devBezierMap(((k-1)*(sn-1)+1):((k-1)*sn+1),:) = devBezierMap_temp;
    else
        BezierMap(((k-1)*(sn-1)+2):((k-1)*sn+1),:) = BezierMap_temp(2:end, :);
        devBezierMap(((k-1)*(sn-1)+1),:) = (devBezierMap(((k-1)*(sn-1)+1),:)+ devBezierMap_temp(1, :))/2.0; %the derivative of the first point in this Bezier segment is average with the derivative of the last point in the previouse bezeir segment.
        devBezierMap(((k-1)*(sn-1)+2):((k-1)*sn+1),:) = devBezierMap_temp(2:end, :);
    end
end
end
%The end of function bezier_reconstruct.
%% Subfunction bezier
% This is a third order Bezeir spline curve. It takes in a 4 point Pn (4 row) array and redigitizing number of points k. 
% Then, it returns k points |P(tn,2)=[i(tn), j(tn)]|, tn=1:k, where i(tn)
% is row index and j(tn) is column index at tn. and its derivative to tn,
% |devP(tn,2)=[devi(tn), devj(tn)]|, tn=1:k, where devi(tn)=-dy/dt is row index derivative and j(tn)=dx/dt is column index at tn.
% Usage: [P, devP]=bezier(Pn,k)
function [P, devP]=bezier(Pn,k)
t=(0:1/(k-1):1);
P=((1-t).^3)'*Pn(1,:)+3.0*(((1-t).^2)'.*t')*Pn(2,:)+3*((1-t)'.*(t.^2)')*Pn(3,:)+(t.^3)'*Pn(4,:); % Bezier curve.
devP=3.0*((1-t).^2)'*(Pn(2,:)-Pn(1,:))+6.0*((1-t)'.*t')*(Pn(3,:)-Pn(2,:))+3.0*(t.^2)'*(Pn(4,:)-Pn(3,:)); % Derivative of Bezier curve.
end
% The end of funtion bezier
%% Subfunction contourL
% Returns the length between points of a line defined by coordinates in |Map|.
% Usage: segmentL(Map)
function SegmentLength=segmentL(Map)
N=size(Map,1);
SegmentLength=zeros(N-1,1);
for n=1:N-1
    SegmentLength(n,1)=((Map(n+1,2)-Map(n,2))^2+(Map(n+1,1)-Map(n,1))^2)^0.5;
end
end
%The end of function segmentL.
%% Subfunction ContourLength
% Returns the contour length of a line defined by coordinates in |Map|.
% Usage: contourL(Map)
function ContourLength=contourL(Map)
SegmentLength=segmentL(Map);
ContourLength=sum(SegmentLength);
end
%The end of function contourL
%% Subfunction midptL
% Usage: midptL(Map)
function Mid_pt=midptL(Map)
SegmentLength=segmentL(Map);
N=size(SegmentLength,1);
Mid_pt=zeros(N,1);
for n=1:N
    Mid_pt(n,1)=sum(SegmentLength(1:n-1))+SegmentLength(n)/2;
end
end
%The end of function midptL
%% Subfunction part_contourL
% TAke in position map |Map| and call fuction segmentL(Map). Return one
% column of partial contour length beteewn first to n points.
% Usage: s=part_contourL(Map)
function s=part_contourL(Map)
SegmentLength=segmentL(Map);
N=size(Map,1);
s=zeros(N,1);
for n=2:N
    s(n,1)=sum(SegmentLength(1:n-1));
end
end
% The end of function part_contourL
%% Subfunction cos_autocorr
% Returns the cosine autocorrelation data of a line from a coordinate,
% Retuen [contour length, cos(theta(s)-theta(0)),  theta(s)-theta(0)]. Take
% in  positions map |pm|=[i, j] and derivative |dev|=[di/dt, dj/dt]=[-dy/dt, dx/dt] to t varariable of Bezier curve.  This function calculates the autocorrelation data for
% all points along a filament in both directions.  
% Calls function |contourL|.
% Usage: [contour length, cos(theta(s)-theta(0)),  theta(s)-theta(0)]=cos_autocorr(pm, dev)
function Data=cos_autocorr(pm,dev)
% Define first point of |DataPool| as autocorrelated origin.
% Alan% Flip method was found to double-count the filaments. Not desirable in
% gelsolin calculations where it is necessary to look at only one side of
% the filament.
% Alan% Calculate data in both directions along the filament.

%delete flip block
% for flip=1:2% use flip=1:2 for homogeneous use flip=1 for allostery
% if flip==2
% pm=flipud(pm);
% end

r=size(pm,1);% The number of points along filaments
angle_value=tangent_angle(dev); % Call tangent_angle function array of tangent angles. The tangent angles are angles relative to j (x) in radius unit and confined in (-Pi, Pi].
% |dev|=[di/dt, dj/dt]=[-dy/dt,dx/dt]. Therefore, dy/dx=-(-dy/dt/dx/dt)=-dev(:,1)./dev(:,2).

% Calculate data for each point along a filament.  
Data=zeros(r*(r-1)/2,3);% Removed at the end. 
data_count=0;       
for corr_length=1:(r-1)  %define correlation distance by the number of data points in between+1
    for ii=1:(r-corr_length) % starting position
        Data(ii+data_count,1)=contourL(pm(ii:(ii+corr_length),:)); %contour lenth    
        Data(ii+data_count,3)=angle_value(ii+corr_length)-angle_value(ii); %angle between two correlated tangent angles
        if Data(ii+data_count,3)> pi % if a difference of between two tangent angles is larger than Pi, the difference in other direction is less than Pi, therefore confine the angle between the two points less than or equal to Pi, i.e. the smaller on of the two difference, add -2*Pi to original angle will not change any value of trigonometry function
            Data(ii+data_count,3)=-(2*pi-Data(ii+data_count,3)); %other direction of angle difference is negative
        elseif -Data(ii+data_count,3)> pi
            Data(ii+data_count,3)=2*pi+Data(ii+data_count,3); %the angle difference is smaller than -Pi, there for taking the angle difference in other direction. add 2*Pi to original angle will not change any value of trigonometry function.
        end         
        Data(ii+data_count,2)=cos(Data(ii+data_count,3));% cos correlation    
    end
    data_count=data_count+ii;
end           
Data=sortrows(Data);
end
% The end of function cos_corr.
    
%% Subfunction tangent_angle
% Take in the [i, j] derivative to varariable t of Bezier curve.
% Returns one column of  tangent angles to x-axis along a line, i.e. Return=[theta(s)]. theta (s) is confined in (-pi, pi]
% from a derivative of Bezier curve reconstructed position map |dev|=[di/dt, dj/dt]=[-dy/dt,dx/dt] 
% Usage: [theta(s)]=tangent_angle(dev)
function Tangent=tangent_angle(dev)
N=size(dev,1); % Define the number of points
Tangent=zeros(N, 1); % array of tangent angles.
% |dev|=[-dy/dt,dx/dt]. Therefore, dy/dx=-(-dy/dt)/dx/dt=-dev(:,1)./dev(:,2).
for ii=1:N
    if dev(ii,2)==0
        if dev(ii,1)>=0 % dy<=0
            Tangent(ii)=-pi/2.0;
        else
            Tangent(ii)=pi/2.0; %dy>0
        end
    elseif dev(ii,2)>0
        Tangent(ii)=atan(-dev(ii,1)/dev(ii,2)); % dev(ii,2) can be positive (dy/dx<0) or negative (dy/dx>0). The angle is in Phase IV and Phase I area.
    else  %  dev(ii,2)<0   
        if dev(ii,1)>=0            
            Tangent(ii)=-pi+atan(-dev(ii,1)/dev(ii,2)); % dev(ii,2)<0 and dev(ii,1)>=0 =>dy<=0, the angle is in Phase III area. atan>0 (give phase I angle), therefore  angle_value(ii)=-pi+atan or pi+atan.
        else
            Tangent(ii)=pi+atan(-dev(ii,2)/dev(ii,1)); % dev(ii,2)<0 and dev(ii,1)<0 => dy>0, the angle is in Phase II area. atan<0 (give phase IV angle), therefore  angle_value(ii)=pi+atan=Pi-|atan|.
        end
    end
end
end
% The end of tangent_angle function.
%% Subfunction fourier_coeff
% calculates first 10 Fourier Coefficients an for a filament presented by a tangent angle to x-axis as a function of 
% segment length, according to Joe Howard's book
% Take in position map |pm| and derivative to t varaviable of Bezier curve and filament contour length cl.
% return one row of the first 10 fourier coefficients
% Usage: fourier_coeff(pm, dev)
function an=fourier_coeff(pm, dev)
sl=segmentL(pm); %each segment length between n to n+1 points
cl=sum(sl); %total contour length
st=tangent_angle(dev); %tangent angle to x-axis
N=size(pm,1);
modeN=10; % number of bending modes
an = zeros(1,modeN); % fourier coefficient for nth bending mode
x = 1:modeN;
amp = (2.0/cl)^(0.5);
for k=1:N-1
    theta = (st(k)+st(k+1))/2.0;    
    s=sum(sl(1:k)); % The segment length between the first  and k points.
    an = an+amp*theta*sl(k)*cos(x*pi*s/cl);
end
end
% The end of function fourier_coeff
%% Subfunction analyze_length
% DensityLength,F,XI]=analyze_length(lengths) determines the most 
% abundant length |DensityLength| from a vector fo density values |F| from a
% set of 100 points |XI| using a kernal smothing method from a list of 
% known contour lengths.  The input is a list of contour lengths |lengths|.
% Usage: analyze_length(lengths)
function [DensityLength,f,xi]=analyze_length(lengths)
[f,xi] = ksdensity(lengths); % Compute density estimate using kernel-smoothing method
MaxY=max(f);   % Find the peak of the lengths
index= f==MaxY;
DensityLength=xi(index);  % The final length is computed as the length that exists the most in the stack
end
% The end of function analyze_length
%% Subfunction filter_data
% Filters data |DataPool| by averaging each point by a filter width |Fwidth|
% and downsamples the data to a number of |Bins| points.  This function is 
% used to average cosine correlation filament fluctuation data and 
% downsample it to the number of points per filament.
% Usage:filter_data(DataPool,Fwidth,Bins)
function Filtered=filter_data(DataPool,Fwidth,Bins)
% Make averaging window size Fwidth odd to make it symmetric.
Fwidth=2*ceil(Fwidth/2)-1;
% Make averaging window of size Fwidth.
Window=ones(1,Fwidth)/Fwidth;
% Sort data in ascending data.
DataPool=sortrows(DataPool);
% Define downsample rate.
Pts=size(DataPool,1);
SampleRate=round(Pts/Bins); 
% Remove first point which is 0,0 (the origin) from DataPool. 
DataTemp=DataPool;
Pts=Pts-1;
DataPool=zeros(Pts,2);
DataPool(1:Pts,:)=DataTemp(2:Pts+1,:);
clear DataTemp
% Pad Data. 
DataTemp=zeros(Pts+2*(Fwidth-1),2);
DataTemp(1:Fwidth-1,2)=ones(Fwidth-1,1);
DataTemp(1:Fwidth-1,1)=zeros(Fwidth-1,1);
DataTemp(Fwidth:Pts+Fwidth-1,:)=DataPool(:,:);
DataPad=DataTemp;
clear DataTemp
% Zero mode average filter data.
DataAve=DataPad;
DataAve(:,2)=filtfilt(Window,1,DataPad(:,2));
% Remove pad.
DataTrim=DataAve(Fwidth:Pts+Fwidth-1,:);
% Downsample data to average number of points per filament line.
Filtered=downsample(DataTrim,SampleRate);
end
% the end of function filter_data

%% Subfunction select_end
% Requires the original image (img); complete end map
% (end_map=[endn,endy,endx]); frame number (frame); total number of ends
% selected (tot_select_end) as in put. When crosshairs appear on Figure 1:
% select ends with left mouse button, finish selection in the frame with
% right mouse button (will continue selection in next frame), and end
% selection for all with middle mouse button. The function returns
% colection of selected ends from the frame (end_map_selected) and
% quit_reply indicating if wants to quick end  selection for all.
% Usage: select_end(loc_select,frame, end_map, tot_select_end)
function [end_map_selected,quit_reply] = select_end(img, end_map, frame, tot_select_end)
f_end=figure('name', 'Manually Selecting Filament Ends'); % Create fiure handler f_end
scrsz = get(0,'ScreenSize');
set(f_end,'Position',[0 0 scrsz(3) scrsz(4)*.93])% To view figure full screen (may need to adjust)
uicontrol(f_end,'style','text',...
    'FontSize',12,....
    'string', ['Instruction for select ends: ' ...
    'Select ends with left mouse button and delete the end just selected with left arrow. Up arrow key has a choice between quiting selection for all and continuing selection in the next frame.'],...
    'position',[10. scrsz(4)*.93/2.-150 (scrsz(3)-scrsz(4)*.93)/2. 300]);
warning_msg=uicontrol(f_end,'style','text',...
    'FontSize',12,...
    'FontWeight','bold',...
    'ForegroundColor',[1 0.5 0.5],...
    'string','',...
    'position',[(scrsz(3)-900)/2. scrsz(4)*.93-50 900 20]);
image(img)
hold on
plot(end_map(:,3)-2,end_map(:,2)-2,'o') % Since the search image is padded by added 0s all around edges, i.e. increase both i and j by +1, -2 to align the ends with original image
% axis xy
axis square
end_select_n=0;% Initialize filament end selection parameters
end_select=1; % |end_select| is mouse button, 1 for left, 2 for middle and 3 for right mouse button pressed.
end_map_selected=zeros(400,3);  % Initiallly allocation of 200 filaments.
quit_reply='No'; % Initializing
title(['Frame ',int2str(frame),', ',  num2str(end_select_n), ' Ends Selected. Total number of ends selected so far ', num2str(end_select_n+tot_select_end)])
while end_select==1
    if size(end_map, 1)==0 % If the remaining unselected ends are empty, quit the selection.
        figure('units','pixels',...
            'position',[500 200 290 140],...
            'menubar','none',...
            'name','Look here!',...
            'numbertitle','off',...
            'resize','off');
        uicontrol('Style','text','Position',[20 65 250 60],'FontSize',12,...
            'String','There is no more end to be select in on this frame. Continue selections in the next frame.',...
            'BackgroundColor',[0.8 0.8 0.8]);
        uicontrol('Position',[45 20 200 40],'String','OK',...
            'Callback','uiresume(gcbf)');
        uiwait(gcf);
        close;
        end_select=3;
        continue;
    end
    % Ignore return and function keypresses
    [x_select,y_select,end_select] = myginput(1, 'arrow');
    if isequal(end_select, [])
        end_select=1; % Reset the key
        continue;
    end
    if (end_select==30) || (end_select==56) % Up arrow key was pressed. Exit selection in this frame and continue in the next frame.
        quit_reply = questdlg('Want to finish selections? (Select "yes" if quit for all and select "No" to continue in the next frame)','Eh?',...
            'Yes','No','Cancel','Yes');
        if strcmp(quit_reply, 'Cancel')
            end_select=1; % Reset the key to continue selection in this frame.
        else
            if strcmp(quit_reply, 'No')
                figure('units','pixels',...
                    'position',[500 200 290 140],...
                    'menubar','none',...
                    'name','Look here!',...
                    'numbertitle','off',...
                    'resize','off');
                uicontrol('Style','text','Position',[20 65 250 60],'FontSize',12,...
                    'String','Continue selections in the next frame.',...
                    'BackgroundColor',[0.8 0.8 0.8]);
                uicontrol('Position',[45 20 200 40],'String','OK',...
                    'Callback','uiresume(gcbf)');
                uiwait(gcf);
                close;
            end
            break;
        end
    elseif  end_select==1 % Left mouse was clicked. find the closest end to store.
        if (x_select<1) || (y_select<1) || (x_select>size(img, 2) || (y_select>size(img, 1)))
            set(warning_msg, 'String', 'Mouse click is out of the area');
        else
            dist = ((x_select+2 - end_map(:,3)).^2+(y_select+2 - end_map(:,2)).^2).^(1/2); % +2 since the search image is padded by added 0s all around edges twice, i.e. increase both i and j by +2.
            end_selection=end_map(dist(:)==min(dist),1); % Index of the selected end
            if (x_select+2 - end_map(end_selection,3))^2+(y_select+2 - end_map(end_selection,2))^2>32
                set(warning_msg, 'String', 'Mouse click is more than 4 pixel away from any end. Try click on an end');
            else
                set(warning_msg, 'String', ''); % Reset the warning message as none
                end_select_n=end_select_n+1;
                if end_select_n>=size(end_map_selected,1)
                    end_map_selected=[end_map_selected; zeros(end_select_n,3)];
                end
                end_map_selected(end_select_n,:)=[end_select_n,end_map(end_selection,2:3)];
                end_map(end_selection,:)=[]; %Delete the selected end from the remaining collection of unselected ends.
                end_map(:,1)=1:size(end_map, 1); % Fix the numbers in the first column due to deletion.
                %         figure(f_end)
                %         hold on
                plot(end_map_selected(end_select_n, 3)-2, end_map_selected(end_select_n, 2)-2, 'g+', 'MarkerSize',12, 'LineWidth',2)
                % axis xy
                %         axis square
                title(['Frame ',int2str(frame),', ',  num2str(end_select_n), ' Ends Selected. Total number of ends selected so far ', num2str(end_select_n+tot_select_end)])
            end
        end
    elseif (end_select==28) || (end_select==52)
        end_select=1; % Reset key.
        if end_select_n==0
            set(warning_msg, 'String', 'There is no pre-selected end to delete in this frame.');
            continue;
        else
            dlete_end_confirm = questdlg('Want to delete the last selected end?','End delettion confirmation',...
                'Yes','No','Cancel','No');
            if strcmp(dlete_end_confirm, 'Yes')
                plot(end_map_selected(end_select_n, 3)-2, end_map_selected(end_select_n, 2)-2, 'k+', 'MarkerFaceColor', 'none', 'MarkerSize',12, 'LineWidth',2)
                end_map=[end_map; [end_map(end, 1)+1, end_map_selected(end_select_n, 2:3)]]; % Insert the end back
                end_select_n=end_select_n-1;
                title(['Frame ',int2str(frame),', ',  num2str(end_select_n), ' Ends Selected. Total number of ends selected so far ', num2str(end_select_n+tot_select_end)])
                set(warning_msg, 'String', 'The last selected end is removed from the selectd end pool.');
                continue;
            else
                set(warning_msg, 'String', 'No end was removed from the selectd end pool. Continue to select...');
                continue;
            end
        end
    else
        set(warning_msg, 'String', 'Select ends with left mouse click and discard the selected end with left arrow key. To quit, press up arrow key');
        end_select=1;
    end
end
end_map_selected((end_select_n+1):end,:)=[];
close (f_end) % Close figure f_end
end
% The end of function select_end.

%% Subfunction Lp_cal
%This function solve Lp as a root of non-linear and implicit equation of
%end_to_end square vs. contour length for a single filament. Requires only
%end_to_end length and contour length square of a single filament, Output Lp value. Usage:
% Lp = Lp_cal(contour, end_to__end)
function Lp = Lp_cal(contour, end_to__end_squre)
max_loop=100; % Define maximal number of iterations.
accuracy=1.0e-12; % Define tolerence.
Lp=10.0; % initial guess of Lp value.
while max_loop>0    
    f=8.0*(Lp^2)*(exp(-contour/(2.0*Lp))-1.0+contour/(2.0*Lp))-end_to__end_squre; % End_to_end square vs contour length from Howard book.
    fdiv=16.0*Lp*(exp(-contour/(2.0*Lp))-1.0+contour/(2.0*Lp))+4.0*contour*(exp(-contour/(2.0*Lp))-1.0); % Deviation to Lp
    if fdiv~=0
        term=f/fdiv;
    else        
        term=f/0.0001;
    end    
    if term^2<accuracy
        break;
    end
    temp_Lp=Lp - term; 
    k=2;
    while temp_Lp<0.0 % Since Lp has to larger than 0, this loop guarantee Lp positive.        
        temp_Lp=Lp - term/k;
        k=k+1;
    end    
    Lp =temp_Lp;
    max_loop=max_loop-1;
end
if max_loop==0
     disp(['Lp value was not found after 100 iterations. The difference between the new and old Lp guesses is ',num2str(term)])
end
end
% The end of function Lp_cal.

%% Subfunction branch_trk
% Function branch_trk tracks a single pixel width filament branch formed by
% only the neareat neighbor connectionto the end or to a new branching
% point.
% Input:
% filament starting coordinate starting=[i,j], one position before
% the the branch starting position old=[i, j], one position before the old,
% old_old=[i,j], i.e. two position before the starting point, excluded point position 
% exclude_pnt=[i, j], skeliton image map (img), and a indicator stretch = 0 (not stretch) or 1
% to indicate whether or not stretching the searched filament from both ends. 
% Output:
% neighbor_coor, a two column array stored the nearseat neighbor coordinates of branch
% end point. Also return position map of the branch (branch_mp). 
%  Usage: [neighbor_coor,branch_mp] = branch_trk(img,starting, old, exclude_pnt, stretch)
function [neighbor_coor,branch_mp] = branch_trk(img, starting, old, old_old, exclude_pnt)
% |old| is the coordinates of the point before  [i,j]. If [i, j] is a starting end point, old==[i,j] is the same as the starting point.
% |old_old| is the coordinates of a data point before |old| or two data points before [i, j]. If [i, j] is a starting end point, old_old==[i,j] is the same as the starting point.
% |exclude_pnt| is the coordinates of a data point needed to be excluded
% from nearest neighbor search. Does this only at the starting of filament.
img=logical(img); % Make sure it is a binary.
i=starting(1,1); % Starting coordinate.
j=starting(1,2); % Starting coordinate.
n=1; % Iniatializing ound data ponit counter.
branch_mp=zeros(500,2); % Position map of filament is preallocated 500 data points.
branch_mp(1,:)=starting; % |branch_mp| = [|i|, |j|], |branch_mp| is a position map for found filament point coordinates.
while (1) % Search the nearest neighbors for the filament pixels. 
    if size(branch_mp,1)<=n
        branch_mp=[branch_mp; zeros(n,2)]; % If the seize of pm is not enough, double the size.
    end
    neighbor_num=0; %reset the number
    neighbor_coor=zeros(7,2); % Reallocate 7 possible nearest neighbors.
    if n==1
        for ii=1:3 % Search [i, j] nearest neighbors and itself also for non-zero pixels.
            for jj=1:3
                if img(i+2-ii, j+2-jj)==1 && ((i+2-ii~=i)||(j+2-jj~=j))&&((i+2-ii~=old(1,1))||(j+2-jj~=old(1,2))) && ((i+2-ii~=old_old(1,1)) || (j+2-jj~=old_old(1,2))) && ((i+2-ii~=exclude_pnt(1,1)) || (j+2-jj~=exclude_pnt(1,2))) % If it is non-zero and not current, previous point, old_old and excluded point.
                    neighbor_num=neighbor_num+1;
                    neighbor_coor(neighbor_num,:)=[i+2-ii,j+2-jj];
                end
            end
        end
    else
        for ii=1:3 % Search [i, j] nearest neighbors and itself also for non-zero pixels.
            for jj=1:3
                if img(i+2-ii, j+2-jj)==1 && ((i+2-ii~=i)||(j+2-jj~=j))&&((i+2-ii~=old(1,1))||(j+2-jj~=old(1,2)))&&((i+2-ii~=old_old(1,1)) || (j+2-jj~=old_old(1,2))) % If it is non-zero and not current, previous points and old_old.
                    neighbor_num=neighbor_num+1;
                    neighbor_coor(neighbor_num,:)=[i+2-ii,j+2-jj];
                end
            end
        end
    end
    % Check the new found neighbors.    
    switch neighbor_num
        case 0 % the nth point has no new neareat neighbor pixel. Therefore it is an end point of the branch. 
            branch_mp(n+1:end,:)=[]; % Delete the empty points.
            neighbor_coor((neighbor_num+1):end, :)=[]; % Delete 0 elements.
            break;            
        case 1 % the nth point has 1 new neareat neighbor pixel.
            old_old=old; % |old| becomes |old_old|
            old=[i,j]; % [i,j] becomes the previous point and the new found point becomes [i, j].
            n=n+1; % Add the new neighbor to the position map |pm|
            branch_mp(n, :)=neighbor_coor(1,:); % store new point.
            i=neighbor_coor(1, 1);
            j=neighbor_coor(1, 2);
        case 2 % the nth point has 2 new neareat neighbor pixel. Check if the two new found neareat neighbor are neareat neighbor are to each other and also either at the same i or the same j
            % If not, the nth point is a branching point.
            if ((neighbor_coor(1, 1)-neighbor_coor(2, 1))^2+(neighbor_coor(1, 2)-neighbor_coor(2, 2))^2)==1 % % two new nearest pixels are adjacent to each other, Add the closest neighbor to the position map |pm|., i.e. the one in the same i or j coordinate.
                n=n+1; % store new point.
                old_old=old; % |old| becomes |old_old|
                old=[i,j]; % [i,j] becomes the previous point and the new found point becomes [i, j].
                if (neighbor_coor(1, 1)==i) || (neighbor_coor(1, 2)==j)  
                    branch_mp(n, :)=neighbor_coor(1, :); 
                    i=neighbor_coor(1, 1);
                    j=neighbor_coor(1, 2);
                else
                    branch_mp(n, :)=neighbor_coor(2, :);
                    i=neighbor_coor(2, 1);
                    j=neighbor_coor(2, 2);
                end
            else % The two new found nearest pixels are not adjacent to each other. The nth point a branch point. Stop search.
                neighbor_coor((neighbor_num+1):end, :)=[]; % Delete 0 elements.
                branch_mp(n+1:end,:)=[]; % Delete the empty points.
                break;                
            end 
        case 3 % the nth point has 3 new neareat neighbor pixel. Check if all three new found neareat neighbor are on the same line and the line is perpendicular to the current filament.
            if (( neighbor_coor(1, 1)==neighbor_coor(2, 1)) && (neighbor_coor(2, 1)==neighbor_coor(3, 1))) || (( neighbor_coor(1, 2)==neighbor_coor(2, 2)) && ( neighbor_coor(2, 2)==neighbor_coor(3, 2))) % Three new found at the same line.
                n=n+1; % store new point.
                old_old=old; % |old| becomes |old_old|
                old=[i,j]; % [i,j] becomes the previous point and the new found point becomes [i, j].
                if (neighbor_coor(1, 1)==i) || (neighbor_coor(1, 2)==j)  % Store the new found that is in the same i or j with nth point.
                    branch_mp(n, :)=neighbor_coor(1, :); 
                    i=neighbor_coor(1, 1);
                    j=neighbor_coor(1, 2);
                elseif (neighbor_coor(2, 1)==i) || (neighbor_coor(2, 2)==j)
                    branch_mp(n, :)=neighbor_coor(2, :);
                    i=neighbor_coor(2, 1);
                    j=neighbor_coor(2, 2);
                else
                    branch_mp(n, :)=neighbor_coor(3, :);
                    i=neighbor_coor(3, 1);
                    j=neighbor_coor(3, 2);
                end
            else                
                neighbor_coor((neighbor_num+1):end, :)=[]; % Delete 0 elements.               
                branch_mp(n+1:end,:)=[]; % Delete the empty points.
                break; % the nth point has more than 3 new nearest neighbor pixels and it is a branch or crossing point. stop search.
            end
        otherwise            
            neighbor_coor((neighbor_num+1):end, :)=[]; % Delete 0 elements.               
            branch_mp(n+1:end,:)=[]; % Delete the empty points..
            break; % the nth point has more than 3 new nearest neighbor pixels and it is a branch or crossing point. stop search.
    end
end
end
% The end of function branch_trk.   

%% Subfunction filament_reconsruction_from_two_points
% Function filament_reconsruction_from_two_poins reconstructs a filament
% from two specified points on the filament. The function takes
% the two specified points as input to reconstruct an continuous filament
% (no gap) between the two points. If the two points doesn't 
% connect to each other, the function return a filament containing just the
% two specified points. The function output constructed filamnet position map (pm).
% Usage: [pm]=filament_reconsruction_from_two_points(point_1, point_2)
function pm=filament_reconsruction_from_two_points(img, point_1, point_2)    
% Start search from point_1 and set first filament point coordinates as point_1.
i=point_1(1); % |i| is the i coordinate.
j=point_1(2); % |j| is the j coordinate.
pre_coord=point_1; % |pre_coord| is the coordinates of the previous point from [i,j]. If [i, j] is a starting end point, pre_coord==[i,j].
pre_pre_coord=point_1; % |pre_Pre_coord| is the coordinates of the previous point from |pre_coord|. If [i, j] is a starting end point, pre_pre_coord==[i,j].
exclude_pnt=point_1; % exclude_pnt is the coordinates of position needed to be excluded from filament searching. If [i, j] is a starting end point, exclude_pnt=[i, j].
n=0; % Found data ponit counter.
pm=zeros(500,2); % Position map of filament is preallocated 500 data points.|pm| = [|i|, |j|], |pm| is a position map for found filament point coordinates.
branch_starting_coor=cell(100, 2); % The first column stores all branches in arrays at defferent level and second column stores the ending point position (n, just one number) in pm(1:n, :) of the higher level. The higher a cell column number, the lower its level is.  Pre-allocate 100 branch levels.
branch_starting_num = 0; % Count saved branch levels.
while 1 % Search the nearest neighbors for the filament pixels. 
    if branch_starting_num>100 % If more than 100 branches, stop the search.
        disp('The search is stuck in an infinite loop, so abandon the search and simply connect the two points')
        pm(1:2, :) = [point_1; point_2]; % Store only point_1 and point_2
        pm(3:end, :)=[];             
        return;
    end
    [neighbor_coor,pm_temp] = branch_trk(img,[i,j], pre_coord, pre_pre_coord, exclude_pnt, 0); % Call function branch_trk(img,starting, old, old_old, stretch) to search nearest neighbor connected filatment.   stretch =0 search will not stretch from both end or 1 will strech from both ends.  
    if (size(pm,1)-n)<=size(pm_temp,1) % If not enough space, increase it.
        pm=[pm; zeros(n+size(pm_temp,1),2)]; % If the seize of pm is not enough, double the size.
    end
    for k=1:size(pm_temp, 1) % Check if the point_2 is on the new found branch
        if (pm_temp(k, 1) == point_2(1)) && (pm_temp(k, 2)== point_2(2)) % point_2 is pm_temp(k,:).
            pm((n+1):(n+k), :)= pm_temp(1:k, :); % Append the found filament data point from 1 to k.
            n = n+k; % Update n.
            pm((n+1):end, :)=[]; % Delelte the empty points.            
            return;
        end
    end    
    abandon_segment=0; % flag for the new found segment    
    if n > 0 % There is previous found filament segment
        abandon_segment = two_filaments_cross(pm, pm_temp); %abandon_segment = 0, 1, 2 or 3. 0 means not crossing, 1 means two filaments contact each other at one place,  2 means two filaments contact each other at two places, and 3 means crossing by shared points.        
    end        
    if size(neighbor_coor, 1) == 0 || (abandon_segment>1) % pm_temp ends at an end (non branch point) and the point_2 is not on the found branch or the new found segment crosses previous found segment at two contact places or shared point      
        while branch_starting_num > 0 % There are un-searched branches. Therefore. abandon the found branch pm_temp and restart another branch search at the last banch point. 
            if size(branch_starting_coor{branch_starting_num, 1},  1)> 0 % There are unsearched branches in this level                
                i = branch_starting_coor{branch_starting_num, 1}(end, 1); % Set the last branch starting point in this level  as new branch search point
                j = branch_starting_coor{branch_starting_num, 1}(end, 2);
                if n>0
                    pre_coord = pm(n, :); 
                else
                    pre_coord=[i, j];
                end
                if n >1
                    pre_pre_coord = pm(n-1, :);
                else
                    pre_pre_coord = pre_coord;
                end
                branch_starting_coor{branch_starting_num, 1}(end, :)=[]; % Delete the used branch starting ploint.
                if size(branch_starting_coor{branch_starting_num, 1},  1)> 0 % Has more branching points
                    exclude_pnt = pm(end, :); % Exclude another branching pont
                else
                    exclude_pnt = [i, j]; % Same as [i, j]
                end                    
                break;
            else % There are no more unsearched branches in this level. Abandon the new found and the higher branch segment also. Start new search from the higher level
                branch_starting_num = branch_starting_num-1; % Up date branch_starting_num
                if branch_starting_num>0 % If there is a higher level banch, delete it.
                    n=branch_starting_coor{branch_starting_num, 2}; % Delete the higher level filament segment 
                end
            end   
        end            
        if branch_starting_num==0 % There is no more un-searched branches. point_2 is not connected to point_1 through neareat neighbor connected pixels.
            pm(1:2, :) = [point_1; point_2]; % Store only point_1 and point_2
            pm(3:end, :)=[];             
            return;
        end
    else % pm_temp ends at a branch point,  the point_2 is not on the found branch, and the new found segment doesn't cross the previous found segment. 
        branch_starting_num=branch_starting_num+1;
        branch_starting_coor{branch_starting_num, 2}=n; % Store the last position before the starting position of the appended new found segment.
        pm((n+1):(n+size(pm_temp, 1)), :)= pm_temp; % Append the found filament data point pm.
        n = n+size(pm_temp, 1);
        branch_starting_coor{branch_starting_num, 1}= neighbor_coor(1:(end-1), :); % Save new found branch starting points except the last on.       
        pre_coord = pm(n, :); 
        if n > 1
            pre_pre_coord = pm(n-1, :);
        else
            pre_pre_coord = pre_coord;
        end
        i = neighbor_coor(end, 1); % Start an new search from the last branch starting point in neighbor_coor.
        j = neighbor_coor(end, 2);
        exclude_pnt = neighbor_coor(end-1, :); % Exclude one of the unsearched branching points
        continue;
    end
end
end
% The end of function filament_reconsruction_from_two_points(img,point_1, point_2)

%% Subfunction filament_reconsruction_one_point_stretch
% Function filament_reconsruction_one_point_stretch reconstructs a filament
% from one specified starting point on the filament. The function takes
% the one selected point as input to reconstruct an continuous filament by
% stretch both directions to ends or joints.
% The function output constructed filamnet position map (pm).
% Usage: [pm]=filament_reconsruction_one_point_stretch(img, point)
function pm=filament_reconsruction_one_point_stretch(img, point)    
% Start search from point_1 and set first filament point coordinates as point_1.
i=point(1); % |i| is the i coordinate.
j=point(2); % |j| is the j coordinate.
pre_coord=point; % |pre_coord| is the coordinates of the previous point from [i,j]. If [i, j] is a starting end point, pre_coord==[i,j].
pre_pre_coord=point; % |pre_Pre_coord| is the coordinates of the previous point from |pre_coord|. If [i, j] is a starting end point, pre_pre_coord==[i,j].
exclude_pnt=point; % exclude_pnt is the coordinates of position needed to be excluded from filament searching. If [i, j] is a starting end point, exclude_pnt=[i, j].
% Search the nearest neighbors for the filament pixels.
[neighbor_coor, pm_temp] = branch_trk(img,[i,j], pre_coord, pre_pre_coord, exclude_pnt); % Call function branch_trk(img,starting, old, old_old, exclude_pnt, stretch) to search nearest neighbor connected filatment.
if size(pm_temp,1) > 1 %the starting point is an end point and from it a filament was constructed.
    pm = pm_temp; %store the filament
    return; %return the filament.
else %size(pm_temp,1) == 1, filament has only one point, the starting point.
    if size(neighbor_coor,1) == 2 % the starting point is in the middle of filament. Stretch from both ends
        % will store the starting point as the middle point and strech from
        % its both ends
        pre_pre_coord = neighbor_coor(1, :);
        pre_coord = [i,j];
        i = neighbor_coor(2, 1);   %search from the second neighbor
        j = neighbor_coor(2, 2);   %search from the second neighbor
        [~,pm_temp] = branch_trk(img, [i,j], pre_coord, pre_pre_coord, exclude_pnt); %stretch one side
        pm_temp_1 = flipud(pm_temp); %temperarily save the branch
        i = pre_pre_coord(1, 1); %search the other end, and pre_coord remains the same
        j = pre_pre_coord(1, 2); %search the other end, and pre_coord remains the same
        pre_pre_coord = pm_temp_1(end, :);
        if size(pm_temp_1, 1) > 1
            exclude_pnt = pm_temp_1(end-1, :);
        else
            exclude_pnt = pre_pre_coord;
        end
        [~,pm_temp] = branch_trk(img, [i,j], pre_coord, pre_pre_coord, exclude_pnt);    %stretch another side.
        %check if the branch from the second streching cross the the
        %branching from the first streching
        filament_crossing = two_filaments_cross(pm_temp_1, pm_temp); %cross_yes_no = 0, 1, 2 or 3. 0 means not crossing, 1 means two filaments contact each other at one place,  2 means two filaments contact each other at two places, and 3 means crossing by shared points.        
        if filament_crossing > 0 %two filaments contact each other at one place is the second place the two filaments contact becasue the two filaments is bridged at original starting point. therefore two places connection of the two filaments makes them a circle
            pm = []; %filaments from stretching both sides crossing each other,so abandon the segment and retern an empty array
            disp('The filament segment crosses itself. Therefore it was desregarded.')
        else
            pm = zeros(size(pm_temp, 1)+size(pm_temp_1, 1)+1, 2);
            pm(1:size(pm_temp_1, 1), :) = pm_temp_1;
            pm (size(pm_temp_1, 1)+1, :) = pre_coord;
            pm (size(pm_temp_1, 1)+2:end, :) = pm_temp; %return a filament from streching both sides
        end
    else %pm_temp has only one point (starting pint) and that point has no neighbor (isolated) or has 3 or more neighbors (3 or more branching points)
        pm = pm_temp; % return the filament with only one point, the starting points.
        disp('The starting point is an isolated or a 3 or more branching point')
    end
end
end
% The end of function filament_reconsruction_from_two_points(img,point_1, point_2)

%% Subfunction select_filament
% This function is for reconstucting filament manually. 
% Requires the original image (img) and skelitonized image (Ske_img); complete end map
% (end_map=[endn,endy,endx]); frame number (frame); total number of
% selected filaments (tot_select_filament) as in put. When crosshairs
% appear on Figure 1: 
% select ends with left mouse button, finish selection in the frame with
% right mouse button (will continue selection in next frame), and end
% selection for all with middle mouse button. The function returns
% colection of selected filaments (filament_select) and
% quit_reply indicating if wants to quick filament selection for all.
% Usage: [filament_select,quit_reply] = select_filament(Ori_img, Ske_img, end_map, frame, tot_select_filament)
function [filament_select,quit_reply] = select_filament(Ori_img, Ske_img, end_map, frame, tot_select_filament, pix_threshold, stretch_reply)
quit_reply='No';
filament_select_n=0;% Initialize filament selection number
filament_select=cell(200,1); % 1D cell array, Allocate 200 filaments
point_select=cell(200,1); %1D cell array, Store selected ponts for replotting. 200 filaments.
f_man=figure('name', 'Manually Construct Filaments'); % Create a figure handler f_man
scrsz = get(0,'ScreenSize');
set(f_man,'Position',[0 0 scrsz(3) scrsz(4)*.93])% To view figure full screen (may need to adjust)
image(Ori_img)
hold on
plot(end_map(:,3)-2,end_map(:,2)-2,'ob') % Since the search image is padded twice by added 0s all around edges, i.e. increase both i and j by +2, -2 to align the ends with original image
if strcmp(stretch_reply, 'Yes')  %stretch_reply == 'Yes' or 'No, Yes means for reconstructing filament from one point by stretching form both sides, while No means no stretching and reconstructing segment from two selected points.
    uicontrol(f_man,'style','text',...
        'FontSize',12,....
        'string',['Instruction for manually constructing filament segment from one point stretching: ',  ...
        'click left mouse button to select an filament segment, and click next adjecent segment to append, and so on. Then "right arrow" key to save filament. Press "left arrow" key anytime to abort selection or delete last filament saved. To finish in this frame or to quit selection for all use "up arrow" key.'],...
        'position',[10. scrsz(4)*.93/2.-150 (scrsz(3)-scrsz(4)*.93)/2. 300]);
    % axis xy
    axis square
else
    uicontrol(f_man,'style','text',...
        'FontSize',12,....
        'string',['Instruction for manually constructiing filament segment between two points: ',  ...
        'Guide selection of a filament by a series of clicking left mouse button on the filament and "right arrow" key to save filament. Press "left arrow" key anytime to abort selection or delete last filament saved. To finish in this frame or to quit selection for all use "up arrow" key.'],...
        'position',[10. scrsz(4)*.93/2.-150 (scrsz(3)-scrsz(4)*.93)/2. 300]);
    % axis xy
    axis square
end
warning_msg=uicontrol(f_man,'style','text',...
                 'FontSize',12,...
                 'FontWeight','bold',...
                 'ForegroundColor',[1 0.5 0.5],...
                 'string','',...
                 'position',[(scrsz(3)-900)/2. scrsz(4)*.93-50 900 20]);
end_select=1; % |end_select| record key pressed, 1 for left mouse key, 2 for middle and 3 for right mouse button pressed. 29 or 54 (right arrow), 28 or 52 (left arrow), and 30 or 56 (up arrow)
while 1
    title(['Frame ',int2str(frame),', ',  num2str(filament_select_n), ' filaments Selected. Total number of filaments selected so far ', num2str(filament_select_n+tot_select_filament)])    
    if (end_select~=28) && (end_select~=52) %if not selected left arrow key
        [x_select,y_select,end_select] = myginput(1, 'arrow');   % Capture the first mouse or keypad input. end_select=1 (left mouse button), 29 or 54 (right arrow), 28 or 52 (left arrow), and 30 or 56 (up arrow). Ignore the other keyentering.   
        if isequal(end_select, [])
            end_select=600; % Give a random number to go the next round of while loop.
            continue;
        end
    end
    % Insert early termination from selections option.
    if (end_select==30) || (end_select==56) % Up arrow key pressed. Quit selection in this frame and continue to the next frame or finish selection for all.
        quit_reply = questdlg('Want to finish selections for all? (Select "Yes" if you want to quit for all and select "No" to continue selection in the next frame)','Eh?',...
            'Yes','No','Cancel','No');
        if strcmp(quit_reply, 'Cancel')
            end_select=27;  % random assign Esc key and Continue selections        
        else
            if strcmp(quit_reply, 'No')
                figure('units','pixels',...
                    'position',[500 200 290 140],...
                    'menubar','none',...
                    'name','Look here!',...
                    'numbertitle','off',...
                    'resize','off');
                uicontrol('Style','text','Position',[20 65 250 60],'FontSize',12,...
                    'String','Continue selections in the next frame.',...
                    'BackgroundColor',[0.8 0.8 0.8]);
                uicontrol('Position',[45 20 200 40],'String','OK',...
                    'Callback','uiresume(gcbf)');
                uiwait(gcf);
                close;
            end
            break;
        end
    elseif (end_select==28) || (end_select==52) % left arrow key pressed. Discard the filament just selected if any.        
        if filament_select_n >0  % If there are filaments selected, discard the last one. If no filament selected, do nothing.
            delete_confirm = questdlg('Want to delete the last constructed or partially constructed filament?','Deletion confirmation',...
                'Yes','No','Cancel','No');
            if strcmp(delete_confirm, 'Yes')
                set(warning_msg, 'string','The last constructed or partially constructed filament has been deleted.');
                %                 plot(point_select{filament_select_n, 1}(:, 1), point_select{filament_select_n, 1}(:, 2), 'kx', 'MarkerSize',12, 'LineWidth',2 )
                %                 plot(filament_select{filament_select_n, 1}(:, 2)-2, filament_select{filament_select_n, 1}(:, 1)-2, '-k', 'LineWidth',2)
                filament_select_n=filament_select_n-1;
                image(Ori_img) % Update (replot) the figure
                hold on
                plot(end_map(:,3)-2,end_map(:,2)-2,'ob')
                axis square
                %         title( ['Frame ',int2str(frame),', ',  num2str(filament_select_n), ' filaments Selected. Total number of filaments selected so far ', num2str(filament_select_n+tot_select_filament)])
                for kkk=1:filament_select_n
                    hold on
                    plot(point_select{kkk, 1}(:, 1), point_select{kkk, 1}(:, 2), 'rx', 'MarkerSize',12, 'LineWidth',2 )
                    hold on
                    plot(filament_select{kkk, 1}(:, 2)-2, filament_select{kkk, 1}(:, 1)-2, '-g', 'LineWidth',2)
                end
            end
        else
            set(warning_msg, 'string','There is no pre-constructed filament to be deleted in this frame. Please continue to construct or quit.');
        end
        end_select=27; % Reset key to random Esc key. This will take another key
        continue;
    elseif (end_select==1) % Left mouse key clicked. Trace the select point.        
        % Check if the mouse clicked point is a filament point
        select_temp=point_on_filament(Ske_img, [y_select+2,x_select+2]); % Since the search image is padded twice by added 0s all around edges, i.e. increase both i and j by +2, -2 to align the ends with original image
        if size(select_temp, 1) < 1  % The selected point is not on a filament.
            set(warning_msg, 'string','The selected point is more than 4 pixel away from any posible filament! Please try again to click on a filament.'); 
        elseif strcmp(stretch_reply, 'Yes') %the selected point is on a filament            
            filament_temp = filament_reconsruction_one_point_stretch(Ske_img, select_temp); % reconstruct filament from one point by stretching both both sides.
            filament_temp_n=size(filament_temp, 1);  %number of points in a filament
            hold on
            plot(select_temp(1, 2)-2,select_temp(1, 1)-2,'rx', 'MarkerSize',12, 'LineWidth',2) % plot the corect point as red x.            
            if filament_temp_n < 1 %filament crosses itsef
                set(warning_msg, 'string','The constructed filament acrosses itself. Please try again!'); % abandon the selected point.
                continue;
            elseif filament_temp_n == 1 
                set(warning_msg, 'string','The clicked point is an isolated or branching point with more than 3 branches. Please try again!'); % abandon the selected point.
                continue;
            else %the constructed filament is good
                set(warning_msg, 'string',''); % Reset warning if it exists. 
                hold on
                plot(filament_temp(:, 2)-2,filament_temp(:, 1)-2, '-g', 'LineWidth',2) %plot the just reconstucted filament as green
                filament_select_n=filament_select_n+1; % Count the new filament
                point_select{filament_select_n, 1}=[select_temp(1 ,2)-2,select_temp(1, 1)-2]; % Store the first selected point for replot figure later.
                filament_select{filament_select_n, 1} = filament_temp; % Store the first segment.                
            end
            while 1  % Loop to select more segments to construct filament until user presses "right" (to store) or "left" (to abored) arrow key to finish the selection.
                [x_select,y_select,end_select_temp] = myginput(1, 'arrow');   % Capture mouse or keypad input. end_select=1 (left mouse button), 28 or 52 (left arrow) and 29 or 54 (right arrow). Ignore the other keyentering.
                if isequal(end_select_temp, [])
                    continue;
                end
                if end_select_temp==1 % Left mouse clicked
                    % Check if clicked on a filament point
                    select_temp=point_on_filament(Ske_img, [y_select+2,x_select+2]);
                    if size(select_temp, 1) <1 % The selescted point is not on a filament.
                        set(warning_msg, 'string','The selected point is more than 4 pixel away from any posible filament! Please try again to click on a filament.');
                        continue;
                    else
                        filament_temp = filament_reconsruction_one_point_stretch(Ske_img, select_temp); % reconstruct filament from one point by stretching both both sides.
                        sub_filament_temp_n = size(filament_temp, 1); %number of points in newly constructed filament segment. temperorily store before added into existing
                        hold on
                        plot(select_temp(1, 2)-2,select_temp(1, 1)-2,'rx', 'MarkerSize',12, 'LineWidth',2) % plot the corect point as red x.
                        if sub_filament_temp_n < 1 %filament crosses itsef
                            set(warning_msg, 'string','The constructed filament acrosses itself. Please try again!'); % abandon the selected point.
                            continue;
                        elseif sub_filament_temp_n == 1 
                            set(warning_msg, 'string','The clicked point is an isolated or branching point with more than 3 branches. Please try again!'); % abandon the selected point.
                            continue;
                        else %the constructed filament is good
                            set(warning_msg, 'string',''); % Reset warning if it exists.
                            hold on   
                            plot(filament_temp(:, 2)-2,filament_temp(:, 1)-2, '-g', 'LineWidth',2) %plot the just reconstucted filament as green
                            %check if newly found segment can be connected
                            %to the existing one at the end and they don't
                            %cross each other after connection
                            [combind_filament, bridge_status] = append_one_filament_to_another(Ske_img, filament_select{filament_select_n, 1}, filament_temp);
                            switch bridge_status
                                case 1 % The end-toend distance of two filaments are more than two pixel away and they cannot be combined.
                                    save_two_separate_filament_confirm = questdlg('The last constructed segment cannot be appended to the previous one becasue they are two pixel away. Do you want to save them as two separated filaments? If No, the last reconstructed segment will be discarded','Save separatedly confirmation',...
                                        'Yes','No','Yes');
                                    if strcmp(save_two_separate_filament_confirm, 'Yes')
                                        set(warning_msg, 'string','The two last partially constructed filaments have saved as separate filaments.');
                                        %                 plot(point_select{filament_select_n, 1}(:, 1), point_select{filament_select_n, 1}(:, 2), 'kx', 'MarkerSize',12, 'LineWidth',2 )
                                        %                 plot(filament_select{filament_select_n, 1}(:, 2)-2, filament_select{filament_select_n, 1}(:, 1)-2, '-k', 'LineWidth',2)
                                        filament_select_n=filament_select_n+1;
                                        point_select{filament_select_n, 1}=[point_select{filament_select_n, 1};  [select_temp(1, 2)-2,select_temp(1, 1)-2]]; % Store the selected points for replot figure later.
                                        filament_select{filament_select_n, 1} = filament_temp; % Store the last segment.
                                        %                                         image(Ori_img) % Update (replot) the figure
                                        %                                         hold on
                                        %                                         plot(end_map(:,3)-2,end_map(:,2)-2,'ob')
                                        %                                         axis square
                                        %                                         title( ['Frame ',int2str(frame),', ',  num2str(filament_select_n), ' filaments Selected. Total number of filaments selected so far ', num2str(filament_select_n+tot_select_filament)])
                                        %                                         for kkk=1:filament_select_n
                                        %                                             hold on
                                        %                                             plot(point_select{kkk, 1}(:, 1), point_select{kkk, 1}(:, 2), 'rx', 'MarkerSize',12, 'LineWidth',2 )
                                        %                                             hold on
                                        %                                             plot(filament_select{kkk, 1}(:, 2)-2, filament_select{kkk, 1}(:, 1)-2, '-g', 'LineWidth',2)
                                        %                                         end
                                        break;
                                    end
                                case 2 %the segment crosses the previous one.
                                    set(warning_msg, 'string','The last constructed segment acrosses the previous one and thus has been discarded. Please try again!'); % abandon the segment.
                                    image(Ori_img) % Update (replot) the figure
                                    hold on
                                    plot(end_map(:,3)-2,end_map(:,2)-2,'ob')
                                    axis square
                                    for kkk=1:filament_select_n %refresh the plots
                                        hold on
                                        plot(point_select{kkk, 1}(:, 1), point_select{kkk, 1}(:, 2), 'rx', 'MarkerSize',12, 'LineWidth',2 )
                                        plot(filament_select{kkk, 1}(:, 2)-2, filament_select{kkk, 1}(:, 1)-2, '-g', 'LineWidth',2)
                                    end
                                case 0 %the segment is good and appended to the previos one.
                                    point_select{filament_select_n, 1}=[point_select{filament_select_n, 1};  [select_temp(1, 2)-2,select_temp(1, 1)-2]]; % Store the selected points for replot figure later.
                                    filament_select{filament_select_n, 1} = combind_filament; % Store the last segment.
                                    filament_temp_n = size(combind_filament, 1);
                                    set(warning_msg, 'string',''); % Reset warning if it exists.
                                    hold on
                                    plot(combind_filament(:, 2)-2, combind_filament(:, 1)-2, '-g', 'LineWidth',2) %plot the just reconstucted filament as green
                            end
                        end  
                    end
                elseif (end_select_temp==29) || (end_select_temp==54) % "right arrow" key pressed.  Save the filament and quit the while loop
                    if  filament_temp_n < pix_threshold
                        set(warning_msg, 'string', ['The filament is shorter than the pre-defined minimum length ', num2str(pix_threshold), ' pixels. Please discard it using left arrow key or continue to construct.']);
                        continue;
                    else
                        filament_select{filament_select_n, 1}((filament_temp_n+1):end, :)=[]; % Delete the empty space and move to the next filament selection
                        break;
                    end
                elseif (end_select_temp==28) || (end_select_temp==52) % left arrow key pressed. Quit the while loop and delete the filament segment just entered in the upper level while loop
                    end_select=28;                    
                    break;
                end
            end
        else % The selected point is on a filament, and stretch_reply == 'No', i.e. reconstruct filament between two points.
            set(warning_msg, 'string',''); % Reset warning if it exists.
            hold on
            plot(select_temp(1, 2)-2,select_temp(1, 1)-2,'rx', 'MarkerSize',12, 'LineWidth',2) % plot the corect point as red x.
            filament_select_n=filament_select_n+1; % Count the new filament
            point_select{filament_select_n, 1}=[select_temp(1 ,2)-2,select_temp(1, 1)-2]; % Store the first selected point for replot figure later.
            filament_select{filament_select_n, 1} = zeros(500, 2); % Initiallly allocation of 500 point for the filament.
            filament_temp_n=1;  % Store the selected point as the first point of the filament.
            filament_select{filament_select_n, 1}(1, :)=select_temp; % Store the selected point as the first point of the filament.
            while 1  % Loop to select more points to construct filament until user presses "right" (to store) or "left" (to abored) arrow key to finish the selection.
                [x_select,y_select,end_select_temp] = myginput(1, 'arrow');   % Capture mouse or keypad input. end_select=1 (left mouse button), 28 or 52 (left arrow) and 29 or 54 (right arrow). Ignore the other keyentering.
                if isequal(end_select_temp, [])
                    continue;
                end
                if end_select_temp==1 % Left mouse clicked
                    % Check if clicked on a filament point
                    select_temp=point_on_filament(Ske_img, [y_select+2,x_select+2]);
                    if size(select_temp, 1) <1 % The selescted point is not on a filament.
                        set(warning_msg, 'string','The selected point is more than 4 pixel away from any posible filament! Please try again to click on a filament.');
                        continue;
                    else
                        set(warning_msg, 'string',''); % Reset warning if it excists.
                        hold on
                        plot(select_temp(1,2)-2,select_temp(1, 1)-2,'rx', 'MarkerSize',12, 'LineWidth',2) % plot the correct point as red x.
                        point_select{filament_select_n, 1}=[point_select{filament_select_n, 1};  [select_temp(1, 2)-2,select_temp(1, 1)-2]]; % Store the selected points for replot figure later.
                        if filament_temp_n > 0 % The selected point is not the first point,
                            filament_temp=filament_reconsruction_from_two_points(Ske_img, filament_select{filament_select_n, 1}(filament_temp_n, :), select_temp); % then send the last point in found filament as the first point and the just selected point as the second point to trace the filament in between
                            if size(filament_select{filament_select_n, 1},1)-filament_temp_n < size(filament_temp, 1) %If not enough space, increase it for storing.
                                filament_select{filament_select_n, 1}=[filament_select{filament_select_n, 1}; zeros((filament_temp_n+size(filament_temp, 1)),2)]; % If there is not enough space in filament_temp, increase it.
                            end
                            filament_select{filament_select_n, 1}((filament_temp_n+1):(filament_temp_n+size(filament_temp, 1)-1), :)=filament_temp(2:end, :); % Store the found filament between the two points from second point to the end since the starting point is already there.
                            filament_temp_n=filament_temp_n+size(filament_temp, 1)-1;
                        else % The selected point is the first point,
                            filament_temp_n=1;  % Store the selected point as the first point of the filament.
                            filament_temp=select_temp; %For plot out.
                            filament_select{filament_select_n, 1}(1, :)=filament_temp;
                        end
                        hold on
                        plot(filament_temp(:, 2)-2,filament_temp(:, 1)-2,'-g', 'LineWidth',2) % plot the contructed filament as green line.
                    end
                elseif (end_select_temp==29) || (end_select_temp==54) % "right arrow" key pressed.  Save the filament and quit the while loop
                    if  filament_temp_n<pix_threshold
                        set(warning_msg, 'string', ['The filament is shorter than the pre-defined minimum length ', num2str(pix_threshold), ' pixels. Please it or continue to construct.']);
                        continue;
                    else
                        filament_select{filament_select_n, 1}((filament_temp_n+1):end, :)=[]; % Delete the empty space and move to the next filament selection
                        break;
                    end
                elseif (end_select_temp==28) || (end_select_temp==52) % left arrow key pressed. Quit the while loop and delete the filament segment just entered in the upper level while loop
                    end_select=28;                    
                    break;
                end
            end
        end
    end
end     
filament_select((filament_select_n+1):end, :)=[]; % Delete the empty cells and return.
close (f_man) % Close figure f_man
end
% The end of function select_filament.

%% Subfunction point_on_filament
% The function check if the input point is on a filament according to
% global skeleton image file (Skeleton). If yes, return the point. If not,
% find a nearby point that is on a filament to replace the input point and
% return the corrected point.
function correct_point=point_on_filament(img, point)
correct_point=[]; % Return correct_point = null
if (point(1)<=2) || (point(2)<=2)|| (point(1)>size(img, 1)-2) || (point(2)>size(img, 2)-2) %coordinate index in point is original coordinate+2 and skeletal img is paddle twice around 4 sides, i.e. size(img, 1) and size(img, 2) are 4 plus original size.
    disp('The selected point is out of area. Please try again to click on a filament.')    
    return;
end
point=uint16(point); % convert to 16 bit unsigned integer
if img(point(1), point(2)) == 1 
    correct_point=point;
else % The  position is not a filament point. Try to find a filament point nearby to replace the selected.
    m_pixel_away=4; % m_pixel_away is maximum of pixels away from the selected point. Use it to define search area. =2 means search only 2 pixel away    
    for kk=1:m_pixel_away         
        for ii=0:kk
            if (point(1)-kk>0)&&(point(2)-ii>0)&&(img(point(1)-kk, point(2)-ii) == 1)
                correct_point=[point(1)-kk, point(2)-ii];
                break;
            elseif (ii~=0)&&(point(1)-kk>0)&&(point(2)+ii<=size(img, 2))&&(img(point(1)-kk, point(2)+ii) == 1)
                correct_point=[point(1)-kk, point(2)+ii];
                break;
            elseif (point(1)+kk<=size(img, 1))&&(point(2)+ii<=size(img, 2))&&(img(point(1)+kk, point(2)+ii) == 1)
                correct_point=[point(1)+kk, point(2)+ii];
                break;
            elseif (ii~=0)&&(point(1)+kk<=size(img, 1))&&(point(2)-ii>0)&&(img(point(1)+kk, point(2)-ii) == 1)
                correct_point=[point(1)+kk, point(2)-ii];
                break;
            elseif (ii~=kk)&&(point(1)-ii>0)&&(point(2)-kk>0)&&(img(point(1)-ii, point(2)-kk) == 1)
                correct_point=[point(1)-ii, point(2)-kk];
                break;
            elseif (ii~=kk)&&(ii~=0)&&(point(1)+ii<=size(img, 1))&&(point(2)-kk>0)&&(img(point(1)+ii, point(2)-kk) == 1)
                correct_point=[point(1)+ii, point(2)-kk];
                break;
            elseif (ii~=kk)&&(point(1)-ii>0)&&(point(2)+kk<=size(img, 2))&&(img(point(1)-ii, point(2)+kk) == 1)
                correct_point=[point(1)-ii, point(2)+kk];
                break;
            elseif (ii~=kk)&&(ii~=0)&&(point(1)+ii<=size(img, 1))&&(point(2)+kk<=size(img, 2))&&(img(point(1)+ii, point(2)+kk) == 1)
                correct_point=[point(1)+ii, point(2)+kk];
                break;
            end            
        end
        if size(correct_point, 1)~=0
            break;
        end
    end
end
end
% The end of function point_on_filament


%% SubFnction two_filaments_cross
%The fulction imputs two flaments as two column arrays (position map of
%points on filament) and check if they have share or have at least two
%places nearest neighor to each other. 
%imput: filament1 and filament2, both two column array
%output: cross_yes_no = 0, 1, 2 or 3. 0-not crossing, 1-two filaments are
%nearest neighbor to each other at one place, 2-two filaments are nearest
%neighbor to each other at two places, and 3-two filaments are across with
%shared point.
function cross_yes_no = two_filaments_cross(filament1, filament2)
cross_yes_no = 0; %cross_yes_no = 0, 1, 2 or 3. 0 means not crossing, 1 means two filaments contact each other at one place,  2 means two filaments contact each other at two places, and 3 means crossing by shared points.
n = size(filament2, 1);
m = size(filament1, 1);
for mm = 1:m % loop to check if the new found filament segment crosses the previous segment
    for nn=1:n
        if (filament1(mm, 1) == filament2(nn, 1)) && (filament1(mm, 2)== filament2(nn, 2)) % Found the cross by a share point.
            cross_yes_no = 3;
            disp('two filaments cross each other by shared point')
            break;
        end
        if (filament1(mm, 1)-filament2(nn, 1))^2 +(filament1(mm, 2) - filament2(nn, 2))^2 <= 2 %found one place two filaments are nearest neighor to each other
            cross_yes_no = cross_yes_no +1; %add to found
            if cross_yes_no == 2 %two filaments are nearest each other at two places
                disp('two filaments are nearest each other at two places')
                break;
            end
        end
    end
    if cross_yes_no > 1
        break;
    end
end
end
% The end of function two_filaments_cross

%% SubFnction append_one_filament_to_another
%The fulction imputs skeletal image and two flaments as two column arrays (position map of
%points on filament) and append the filaments 2 to filament1. Function
%outputs the combind filament and the status of appending.
%imput: filament1 and filament2, both two column array, and skeletal image
%(skel_img)
%output: combind filament (combind_filament)
%  bridge_status = 0, 1, or 2.
% 0-appending is successful, 1-two filaments ends are more than two pixel
% away and no combind filament (0 length). 2-two filaments cross each other and no combind filament (0 length).
function [combind_filament, bridge_status] = append_one_filament_to_another(skel_img, filament1, filament2)
bridge_status = 0;
head_to_toe = (filament1(end, 1)-filament2(1, 1))^2+(filament1(end, 2)-filament2(1, 2))^2;
toe_to_head = (filament1(1, 1)-filament2(end, 1))^2 + (filament1(1, 2)-filament2(end, 2))^2;
head_to_head = (filament1(1, 1)-filament2(1, 1))^2 + (filament1(1, 2)-filament2(1, 2))^2;
toe_to_toe = (filament1(end, 1)-filament2(end, 1))^2 + (filament1(end, 2)-filament2(end, 2))^2;
if head_to_toe <= 2 %head and toe are nearest neighbor to each other or are the same point, append directly
    if head_to_toe == 0 %head and toe are the same point, delelete the head in the newly found segment
        filament2(1, :) = [];
    end
    If_crossing = two_filaments_cross(filament1, filament2); %to see if two filament cross each other
    if If_crossing > 1 %they cross each other, return empty
        bridge_status = 2;
        combind_filament = [];
    else %they don't cross each other
        combind_filament = [filament1; filament2]; % append the segment directly.
    end
elseif toe_to_head <=2 %toe to head are nearest neighbor to each other or are the same point, append directly
    if toe_to_head == 0 %toe to head are the same point, delelete the toe in the newly found segment
        filament2(end, :) = [];
    end
    If_crossing = two_filaments_cross(filament1, filament2); %to see if two filament cross each other
    if If_crossing > 1 %they cross each other, return empty
        bridge_status = 2;
        combind_filament = [];
    else %they don't cross each other
        combind_filament = [filament2; filament1]; % append the segment directly.
    end
elseif head_to_head <= 2 %head to head are nearest neighbor to each other or are the same point, append directly
    if head_to_head == 0 %head to head are the same point, delelete the head in the newly found segment
        filament2(1, :) = [];
    end
    If_crossing = two_filaments_cross(filament1, filament2); %to see if two filament cross each other
    if If_crossing > 1 %they cross each other, return empty
        bridge_status = 2;
        combind_filament = [];
    else %they don't cross each other
        combind_filament = [flipud(filament2); filament1]; % append the segment directly.
    end
elseif toe_to_toe <= 2 %toe to toe are nearest neighbor to each other or are the same point, append directly
    if toe_to_toe == 0 %toe to toe are the same point, delelete the toe in the newly found segment
        filament2(end, :) = [];
    end
    If_crossing = two_filaments_cross(filament1, filament2); %to see if two filament cross each other
    if If_crossing > 1 %they cross each other, return empty
        bridge_status = 2;
        combind_filament = [];
    else %they don't cross each other
        combind_filament = [filament1; flipud(filament2)]; % append the segment directly.
    end
elseif head_to_toe <= 8 %head and toe are second nearest neighbor to each other, need to find a pixel that is nearest to both and  has value of 1 to bridge them or creat one to fill the gap if not exist.
    If_crossing = two_filaments_cross(filament1, filament2); %to see if two filament cross each other
    if If_crossing > 0 %Since end to end are not nearest neighbor to each other, the two filaments must touch at middle, i.e. they cross each other.  return empty
        bridge_status = 2;
        combind_filament = [];
        return;
    end
    if head_to_toe == 8 %head to toe are diagonal to each other and the only pixel nearest to both is at the middle between the two, and choose this one to bridge no matter its value is 1 or not. If it is 1 is natural, if not, it is created one.
        middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
    elseif head_to_toe == 4 %head to toe are at the same horizontal or vertical line and there are three pixels in the middle vertical dividing line that are nearest to both. find one works if not creat one at the middle point.
        if filament1(end, 1) == filament2(1, 1) %they are at the same horizontal line
            if skel_img(filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            elseif skel_img(filament1(end, 1)+1, (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1)+1, (filament1(end, 2)+filament2(1, 2))/2];
            elseif skel_img(filament1(end, 1)-1, (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1)-1, (filament1(end, 2)+filament2(1, 2))/2];
            else %all three are not pixel whose value is 1, so create one in the middle
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            end
        else %they are at the same verical line
            if skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)) == 1
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            elseif skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)+1) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)+1];
            elseif skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)-1) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)-1];
            else %all three are not pixel whose value is 1, so create one in the middle
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            end
        end
    else %if horizontal position difference is 1, vertical position difference must be 2, or other way around.
        if (filament1(end, 1)-filament2(1, 1))^2 == 1 %vertical difference is 1 and there are two middle points that are nearest to both and whose are in the same horizontal ine with either one of them
            if skel_img(filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2];
            else %choose the second one no matter its pixel value is 1 or not.
                middle_bridge_point = [filament2(1, 1), (filament1(end, 2)+filament2(1, 2))/2];
            end
        else %horizontal difference is 1 and there are two middle points that are nearest to both and whose are in the same horizontal ine with either one of them
            if skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)];
            else %choose the second one no matter its pixel value is 1 or not.
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament2(1, 2)];
            end
        end
    end
    combind_filament = [filament1; middle_bridge_point; filament2]; % append the segment after bridge point.
elseif toe_to_head <= 8 %toe and head are second nearest neighbor to each other, need to find a pixel that is nearest to both and  has value of 1 to bridge them or creat one to fill the gap if not exist.
    If_crossing = two_filaments_cross(filament1, filament2); %to see if two filament cross each other
    if If_crossing > 0 %Since end to end are not nearest neighbor to each other, the two filaments must touch at middle, i.e. they cross each other.  return empty
        bridge_status = 2;
        combind_filament = [];
        return;
    end
    if toe_to_head == 8 %toe to head are diagonal to each other and the only pixel nearest to both is at the middle between the two, and choose this one to bridge no matter its value is 1 or not. If it is 1 is natural, if not, it is created one.
        middle_bridge_point = (filament1(1, :)+filament2(end, :))/2;
    elseif toe_to_head == 4 %toe to head are at the same horizontal or vertical line and there are three pixels in the middle vertical dividing line that are nearest to both. find one works if not creat one at the middle point.
        if filament1(1, 1) == filament2(end, 1) %they are at the same horizontal line
            if skel_img(filament1(1, 1), (filament1(1, 2)+filament2(end, 2))/2) == 1
                middle_bridge_point = (filament1(1, :)+filament2(end, :))/2;
            elseif skel_img(filament1(1, 1)+1, (filament1(1, 2)+filament2(end, 2))/2) == 1
                middle_bridge_point = [filament1(1, 1)+1, (filament1(1, 2)+filament2(end, 2))/2];
            elseif skel_img(filament1(1, 1)-1, (filament1(1, 2)+filament2(end, 2))/2) == 1
                middle_bridge_point = [filament1(1, 1)-1, (filament1(1, 2)+filament2(end, 2))/2];
            else %all three are not pixel whose value is 1, so create one in the middle
                middle_bridge_point = (filament1(1, :)+filament2(end, :))/2;
            end
        else %they are at the same verical line
            if skel_img((filament1(1, 1)+filament2(end, 1))/2, filament1(1, 2)) == 1
                middle_bridge_point = (filament1(1, :)+filament2(end, :))/2;
            elseif skel_img((filament1(1, 1)+filament2(end, 1))/2, filament1(1, 2)+1) == 1
                middle_bridge_point = [(filament1(1, 1)+filament2(end, 1))/2, filament1(1, 2)+1];
            elseif skel_img((filament1(1, 1)+filament2(end, 1))/2, filament1(1, 2)-1) == 1
                middle_bridge_point = [(filament1(1, 1)+filament2(end, 1))/2, filament1(1, 2)-1];
            else %all three are not pixel whose value is 1, so create one in the middle
                middle_bridge_point = (filament1(1, :)+filament2(end, :))/2;
            end
        end
    else %if horizontal position difference is 1, vertical position difference must be 2, or other way around.
        if (filament1(1, 1)-filament2(end, 1))^2 == 1 %vertical difference is 1 and there are two middle points that are nearest to both and whose are in the same horizontal ine with either one of them
            if skel_img(filament1(1, 1), (filament1(1, 2)+filament2(end, 2))/2) == 1
                middle_bridge_point = [filament1(1, 1), (filament1(1, 2)+filament2(end, 2))/2];
            else %choose the second one no matter its pixel value is 1 or not.
                middle_bridge_point = [filament2(end, 1), (filament1(1, 2)+filament2(end, 2))/2];
            end
        else %horizontal difference is 1 and there are two middle points that are nearest to both and whose are in the same horizontal ine with either one of them
            if skel_img((filament1(1, 1)+filament2(end, 1))/2, filament1(1, 2)) == 1
                middle_bridge_point = [(filament1(1, 1)+filament2(end, 1))/2, filament1(1, 2)];
            else %choose the second one no matter its pixel value is 1 or not.
                middle_bridge_point = [(filament1(1, 1)+filament2(end, 1))/2, filament2(end, 2)];
            end
        end
    end
    combind_filament = [filament2; middle_bridge_point; filament1]; % append the segment after bridge point.
elseif head_to_head <= 8 %head and head are second nearest neighbor to each other, need to find a pixel that is nearest to both and  has value of 1 to bridge them or creat one to fill the gap if not exist.
    If_crossing = two_filaments_cross(filament1, filament2); %to see if two filament cross each other
    if If_crossing > 0 %Since end to end are not nearest neighbor to each other, the two filaments must touch at middle, i.e. they cross each other.  return empty
        bridge_status = 2;
        combind_filament = [];
        return;
    end
    filament1 = flipud(filament1); %after flip filament1, the two become head to toe and treat them as head to toe
    if head_to_head == 8 %head to head are diagonal to each other and the only pixel nearest to both is at the middle between the two, and choose this one to bridge no matter its value is 1 or not. If it is 1 is natural, if not, it is created one.
        middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
    elseif head_to_head == 4 %head to head are at the same horizontal or vertical line and there are three pixels in the middle vertical dividing line that are nearest to both. find one works if not creat one at the middle point.
        if filament1(end, 1) == filament2(1, 1) %they are at the same horizontal line
            if skel_img(filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            elseif skel_img(filament1(end, 1)+1, (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1)+1, (filament1(end, 2)+filament2(1, 2))/2];
            elseif skel_img(filament1(end, 1)-1, (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1)-1, (filament1(end, 2)+filament2(1, 2))/2];
            else %all three are not pixel whose value is 1, so create one in the middle
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            end
        else %they are at the same verical line
            if skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)) == 1
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            elseif skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)+1) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)+1];
            elseif skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)-1) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)-1];
            else %all three are not pixel whose value is 1, so create one in the middle
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            end
        end
    else %if horizontal position difference is 1, vertical position difference must be 2, or other way around.
        if (filament1(end, 1)-filament2(1, 1))^2 == 1 %vertical difference is 1 and there are two middle points that are nearest to both and whose are in the same horizontal ine with either one of them
            if skel_img(filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2];
            else %choose the second one no matter its pixel value is 1 or not.
                middle_bridge_point = [filament2(1, 1), (filament1(end, 2)+filament2(1, 2))/2];
            end
        else %horizontal difference is 1 and there are two middle points that are nearest to both and whose are in the same horizontal ine with either one of them
            if skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)];
            else %choose the second one no matter its pixel value is 1 or not.
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament2(1, 2)];
            end
        end
    end
    combind_filament = [filament1; middle_bridge_point; filament2]; % append the segment after bridge point.
elseif toe_to_toe <= 8 %toe and toe are second nearest neighbor to each other, need to find a pixel that is nearest to both and  has value of 1 to bridge them or creat one to fill the gap if not exist.
    If_crossing = two_filaments_cross(filament1, filament2); %to see if two filament cross each other
    if If_crossing > 0 %Since end to end are not nearest neighbor to each other, the two filaments must touch at middle, i.e. they cross each other.  return empty
        bridge_status = 2;
        combind_filament = [];
        return;
    end
    filament2 = flipud(filament2); %after flip filament2, the two become head to toe and treat them as head to toe
    if toe_to_toe == 8 %toe to toe are diagonal to each other and the only pixel nearest to both is at the middle between the two, and choose this one to bridge no matter its value is 1 or not. If it is 1 is natural, if not, it is created one.
        middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
    elseif toe_to_toe == 4 %toe to toe are at the same horizontal or vertical line and there are three pixels in the middle vertical dividing line that are nearest to both. find one works if not creat one at the middle point.
        if filament1(end, 1) == filament2(1, 1) %they are at the same horizontal line
            if skel_img(filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            elseif skel_img(filament1(end, 1)+1, (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1)+1, (filament1(end, 2)+filament2(1, 2))/2];
            elseif skel_img(filament1(end, 1)-1, (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1)-1, (filament1(end, 2)+filament2(1, 2))/2];
            else %all three are not pixel whose value is 1, so create one in the middle
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            end
        else %they are at the same verical line
            if skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)) == 1
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            elseif skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)+1) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)+1];
            elseif skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)-1) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)-1];
            else %all three are not pixel whose value is 1, so create one in the middle
                middle_bridge_point = (filament1(end, :)+filament2(1, :))/2;
            end
        end
    else %if horizontal position difference is 1, vertical position difference must be 2, or other way around.
        if (filament1(end, 1)-filament2(1, 1))^2 == 1 %vertical difference is 1 and there are two middle points that are nearest to both and whose are in the same horizontal ine with either one of them
            if skel_img(filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2) == 1
                middle_bridge_point = [filament1(end, 1), (filament1(end, 2)+filament2(1, 2))/2];
            else %choose the second one no matter its pixel value is 1 or not.
                middle_bridge_point = [filament2(1, 1), (filament1(end, 2)+filament2(1, 2))/2];
            end
        else %horizontal difference is 1 and there are two middle points that are nearest to both and whose are in the same horizontal ine with either one of them
            if skel_img((filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)) == 1
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament1(end, 2)];
            else %choose the second one no matter its pixel value is 1 or not.
                middle_bridge_point = [(filament1(end, 1)+filament2(1, 1))/2, filament2(1, 2)];
            end
        end
    end
    combind_filament = [filament1; middle_bridge_point; filament2]; % append the segment after bridge point.
else %the two filaments end to end are more than two pixel away
    bridge_status = 1;
    combind_filament = [];
end
end
%

function [out1,out2,out3] = myginput(arg1,strpointertype)
%MYGINPUT Graphical input from mouse with custum cursor pointer.
%   [X,Y] = MYGINPUT(N) gets N points from the current axes and returns 
%   the X- and Y-coordinates in length N vectors X and Y.
%
%   [X,Y] = MYGINPUT(N, POINTER) also specifies the cursor pointer, e.g.
%   'crosshair', 'arrow', 'circle' etc. See "Specifying the Figure Pointer"
%   in Matlab's documentation  to see the list of available pointers.
%   
%   MYGINPUT is strictly equivalent to Matlab's original GINPUT, except
%   that a second argument specifies the cursor pointer instead of the
%   default 'fullcrosshair' pointer.
%
%   Example:
%     plot(1:2,1:2,'s');
%     hold on
%     [x,y] = myginput(1,'crosshair');
%     plot(x,y,'o');
%     hold off
%
%   MYGINPUT is copied from Matlab's GINPUT rev. 5.32.4.4.
%
%   See also GINPUT.

%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 1.02,  Date: 2006/10/24

% History:
% 2005/10/31: v1.00, first version, from GINPUT rev. 5.32.4.4.
% 2005/11/25: v1.01, line 'uisuspend' modified (for compatibility with
%                    ML7.00)
% 2006/10/24: v1.02, help text improved

out1 = []; out2 = []; out3 = []; y = [];

if nargin<1     % modified MYGINPUT
    strpointertype='fullcrosshair'; % default GINPUT pointer
end

c = computer;
if ~strcmp(c(1:2),'PC') 
   tp = get(0,'TerminalProtocol');
else
   tp = 'micro';
end

if ~strcmp(tp,'none') && ~strcmp(tp,'x') && ~strcmp(tp,'micro')
   if nargout == 1
      if nargin == 1
         out1 = trmginput(arg1);
      else
         out1 = trmginput;
      end
   elseif nargout == 2 || nargout == 0
      if nargin == 1
         [out1,out2] = trmginput(arg1);
      else
         [out1,out2] = trmginput;
      end
      if  nargout == 0
         out1 = [ out1 out2 ];
      end
   elseif nargout == 3
      if nargin == 1
         [out1,out2,out3] = trmginput(arg1);
      else
         [out1,out2,out3] = trmginput;
      end
   end
else
   
   fig = gcf;
   figure(gcf);
   
   if nargin == 0
      how_many = -1;
      b = [];
   else
      how_many = arg1;
      b = [];
      if  isstr(how_many) ...
            | size(how_many,1) ~= 1 | size(how_many,2) ~= 1 ...
            | ~(fix(how_many) == how_many) ...
            | how_many < 0
         error('Requires a positive integer.')
      end
      if how_many == 0
         ptr_fig = 0;
         while(ptr_fig ~= fig)
            ptr_fig = get(0,'PointerWindow');
         end
         scrn_pt = get(0,'PointerLocation');
         loc = get(fig,'Position');
         pt = [scrn_pt(1) - loc(1), scrn_pt(2) - loc(2)];
         out1 = pt(1); y = pt(2);
      elseif how_many < 0
         error('Argument must be a positive integer.')
      end
   end
   
   % Suspend axes functions
       %haxes = findobj(fig,'type','axes');
        state = uisuspend(fig);
   %haxes = findobj(fig,'type','axes');
   %state = uisuspend(haxes);
   pointer = get(gcf,'pointer');
   set(gcf,'pointer',strpointertype);  % modified MYGINPUT
   fig_units = get(fig,'units');
   char = 0;

   % We need to pump the event queue on unix
   % before calling WAITFORBUTTONPRESS 
   drawnow
   
   while how_many ~= 0
      % Use no-side effect WAITFORBUTTONPRESS
      waserr = 0;
      try
	keydown = wfbp;
      catch
	waserr = 1;
      end
      if(waserr == 1)
         if(ishandle(fig))
            set(fig,'units',fig_units);
	    uirestore(state);
            error('Interrupted');
         else
            error('Interrupted by figure deletion');
         end
      end
      
      ptr_fig = get(0,'CurrentFigure');
      if(ptr_fig == fig)
         if keydown
            char = get(fig, 'CurrentCharacter');
            button = abs(get(fig, 'CurrentCharacter'));
            scrn_pt = get(0, 'PointerLocation');
            set(fig,'units','pixels')
            loc = get(fig, 'Position');
            pt = [scrn_pt(1) - loc(1), scrn_pt(2) - loc(2)];
            set(fig,'CurrentPoint',pt);         
         else
            button = get(fig, 'SelectionType');
            if strcmp(button,'open') 
               button = 1;
            elseif strcmp(button,'normal') 
               button = 1;
            elseif strcmp(button,'extend')
               button = 2;
            elseif strcmp(button,'alt') 
               button = 3;
            else
               error('Invalid mouse selection.')
            end
         end
         pt = get(gca, 'CurrentPoint');
         
         how_many = how_many - 1;
         
         if(char == 13) % & how_many ~= 0)
            % if the return key was pressed, char will == 13,
            % and that's our signal to break out of here whether
            % or not we have collected all the requested data
            % points.  
            % If this was an early breakout, don't include
            % the <Return> key info in the return arrays.
            % We will no longer count it if it's the last input.
            break;
         end
         
         out1 = [out1;pt(1,1)];
         y = [y;pt(1,2)];
         b = [b;button];
      end
   end
   
   uirestore(state);
   set(fig,'units',fig_units);
   
   if nargout > 1
      out2 = y;
      if nargout > 2
         out3 = b;
      end
   else
      out1 = [out1 y];
   end
   
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function key = wfbp
%WFBP   Replacement for WAITFORBUTTONPRESS that has no side effects.

fig = gcf;
current_char = [];

% Now wait for that buttonpress, and check for error conditions
waserr = 0;
try
  h=findall(fig,'type','uimenu','accel','C');   % Disabling ^C for edit menu so the only ^C is for
  set(h,'accel','');                            % interrupting the function.
  keydown = waitforbuttonpress;
  current_char = double(get(fig,'CurrentCharacter')); % Capturing the character.
  if~isempty(current_char) & (keydown == 1)           % If the character was generated by the 
	  if(current_char == 3)                       % current keypress AND is ^C, set 'waserr'to 1
		  waserr = 1;                             % so that it errors out. 
	  end
  end
  
  set(h,'accel','C');                                 % Set back the accelerator for edit menu.
catch
  waserr = 1;
end
drawnow;
if(waserr == 1)
   set(h,'accel','C');                                % Set back the accelerator if it errored out.
   error('Interrupted');
end

if nargout>0, key = keydown; end

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
